import QtQuick 2.0
import DataStoreComponents 1.0
import S7ConnectionModelComponents 1.0
import S7TagModelComponents 1.0
import S7InfoModelComponents 1.0
import QtQuick.Controls 1.1
import QtQuick.Window 2.1
import QtGraphicalEffects 1.0
import ConnectionControllerComponent 1.0
import TagControllerComponent 1.0
import LoggerComponents 1.0
import IOStatusTableModelComponent 1.0
import IOStatusControllerComponent 1.0
import DTYearComponent 1.0
import IOStatusListModelComponent 1.0
import DiagBufferControllerComponet 1.0
import DiagnosticBufferModelComponet 1.0
import ModuleInfoModelComponent 1.0

import "." as S7App_Components

Rectangle {
    width: 900
    height: 600

    id: mainObject

    focus: true

    Screen.onHeightChanged: def.getScreenInformation()


    property string currentConnection: "myConnection"
    property string currentIP: "0.0.0.0"
    property int currentIndex: 0

    property string currentApp: "LOGO!"
    property string errorMsg:""
    
    ModalDialog {
       id: controllerMsg
       button_OK: true
    }


    AppDef{
        id: def
    }

    StringStore
    {
        id:stringStore
    }


    ConnectionListDelegateModel{
        id:connectionDelegateModel
    }

    TagListDelegateModel{
        id:tagDelegateModel
    }

    ConnectionModel
    {
        id: connectionModel
        onDataChanged: {
          screen_systemInfo.plcInfo = stringStore.firmwareTag + getBasicInfo(mainObject.currentIndex)
          Logger.consoleLog(def.logInfo,"onDataChanged............"+screen_systemInfo.plcInfo)
        }
    }



    TagModel
    {
        id: tagModel
        onDataChanged: {
        	if(busyIndicator.visible == true)
           	busyIndicator.visible = false
        }
    }

    InfoModel
    {
        id: infoModel
    }

    ModuleInfoModel
    {
        id: detailedInfoModel
    }

    ConnectionListItem
    {
        id: connectionDelegate
    }
    TagListItem
    {
        id: tagListDelegate
    }

    DiagnosticListItem
    {
        id: diagnosticListItem
    }
    
    TrendViewListItem{
        id:trendViewListItem
    }

    TrendViewIOListItem{
        id:trendViewIOListItem
    }

    DataGridItem{
        id:dataGridItem
    }

    ToastMessage{
        id:toastMsg
    }

    ConnectionController
    {
        id:conController
        connectionType : ConnectionController.enLogoConnection
        Component.onCompleted:
        {
            setConnectionModel(connectionModel);
        }
    }

    IOStatusTableModel{
        id: ioStatusTableModel
    }

    IOStatusListModel{
        id: ioStatusListModel
    }

    IOStatusController{
    id:ioStatusController
    Component.onCompleted: {
        setIOStatusModel(ioStatusTableModel)
        }
    }

    DTYearModel{
        id:dtyearModel
    }

    DiagBufferModel{
        id: diagnosticBufferModel;
    }
    DiagBuffController{
        id:  diagnosticbuffController
        Component.onCompleted:
        {
            setDiagModel(diagnosticBufferModel)
        }
    }
    TagController
    {
        id:tagController
        Component.onCompleted:
        {
            setTagModel(tagModel);
        }
    }

    LinearGradient {
            anchors.fill: parent
            start: Qt.point(0, 0)
            end: Qt.point(parent.width, 0)
            gradient: Gradient {
                GradientStop { position: 0.0; color: def.backgroundgradientleft }
                GradientStop { position: 1.0; color: def.backgroundgradientright}
            }
        }
	

    WaitIndicator{
        id: busyIndicator
        visible:false
        anchors.horizontalCenter:parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        z:20
    }

    ScreenDeviceOverview{
        id: screen_overview
        width: parent.width
        height: parent.height
        screenMode: 0           // 0: normal view, 1: editing view
        opacity: 0
        visible: false

    }

    ScreenSplash{
        id: screen_splash
        width: parent.width
        height: parent.height
    }

    ScreenSystemInfo{
        id: screen_systemInfo
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenAuthentication{
        id:screen_authentication
        width: parent.width
        height: parent.height
        visible:false
    }

    ScreenTagList{
        id: screen_tagList
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false
        screenMode: 0           // 0: normal view, 1: editing view
    }

    ScreenTagEdit{
        id: screen_tagEdit
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false
    }

    ScreenTrendView{
        id: screen_trendView
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenTrendView_FullScreen{
        id: screen_trendView_FullScreen
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenDataLog{
        id: screen_dataLog
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenEditConnection
    {
        id: screen_editConnection
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenWriteToPLC
    {
        id: screen_writeToPLC
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenLogin{
       id: appLogin
       width: parent.width
       height: parent.height
       opacity: 0
       visible: false

    }

    ScreenSettings{
        id: screen_settings
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenAppInformation{
        id: screen_appInfo
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenAppInformationLicense{
        id: screen_appInfoLicense
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false
    }

    ScreenAppInformationContent{
        id: screen_appInfoContent
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }
    
    ScreenFrequencySettings{
        id: refresh_freq_settings
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    LanguageSettings {
        id: language_settings
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenChangePassword {
        id: screen_changepwd
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenIOStateOverview{
        id: screen_iostateoverview
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenIOState{
        id: screen_iostate
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }

    ScreenDiagnostics{
        id: screen_diag
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false

    }


    StackView{
        id: stackview
        anchors.fill: parent
        initialItem: screen_splash

        function popCurrentPage(currentScreen)
        {
            if(currentScreen === stackview.currentItem)
                stackview.pop()
        }

        function popPage()
        {
            stackview.pop()

        }

        function pushPage(page)
        {
            if(stackview.currentItem !== page)
            {
                stackview.push(page)
            }
            Logger.consoleLog(def.logInfo,"page count =========== " + stackview.depth)
        }


        delegate: StackViewDelegate {
            function transitionFinished(properties)
            {
                properties.enterItem.opacity = 1
                properties.exitItem.opacity = 0
                properties.enterItem.focus = true
                properties.exitItem.focus = false
                properties.enterItem.visible = true
                properties.exitItem.visible = false
                Qt.inputMethod.hide()
            }
            pushTransition: StackViewTransition {
                /*
                NumberAnimation {
                    target: enterItem;
                    property: "x";
                    to: 0
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
                NumberAnimation {
                    target: exitItem;
                    property: "x";
                    to: 0-exitItem.width;
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
            }

            popTransition: StackViewTransition {
                NumberAnimation {
                    target: enterItem;
                    property: "x";
                    to: 0
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
                NumberAnimation {
                    target: exitItem;
                    property: "x";
                    to: exitItem.width;
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
            }
            replaceTransition: StackViewTransition {*/
                NumberAnimation {
                    target: enterItem
                    property: "opacity"
                    from: 0
                    to: 1
                    easing.type: Easing.InOutQuad
                    duration: def.screenchangetime
                }
                PropertyAnimation {
                    target: exitItem
                    property: "opacity"
                    from: 1
                    to: 0
                    duration: def.screenchangetime

                }
            }
        }

    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed: Logo main" )
        if (event.key === Qt.Key_Back)
        {
            event.accepted = true;
            if(stackview.currentItem == screen_overview)
            {
                if(screen_overview.screenMode == 1)
                {
                    screen_overview.screenMode = 0
                }
                else
                {
                    Qt.quit();
                }
            }
            else if(stackview.currentItem == appLogin)
            {
                Qt.quit();
            }
            else
                stackview.pop()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }
    Component.onCompleted: {
           appController.appStatusChanged.connect(screen_authentication.onAppstatusChanged)
       }

}
