import QtQuick 2.0
import DataStoreComponents 1.0
import S7ConnectionModelComponents 1.0
import S7TagModelComponents 1.0
import S7InfoModelComponents 1.0
import QtQuick.Controls 1.1
import QtQuick.Window 2.1
import QtGraphicalEffects 1.0
import ConnectionControllerComponent 1.0
import TagControllerComponent 1.0
import LoggerComponents 1.0
import IOStatusTableModelComponent 1.0
import IOStatusControllerComponent 1.0
import IOStatusListModelComponent 1.0
import DiagBufferControllerComponet 1.0
import DiagnosticBufferModelComponet 1.0
import ModuleInfoModelComponent 1.0


import "." as S7App_Components

Rectangle {
    width: 900
    height: 600

    id: mainObject

    focus: true

    Screen.onHeightChanged: def.getScreenInformation()


    property string currentConnection: "myConnection"
    property string currentIP: "0.0.0.0"
    property int currentIndex: 0

    property string currentApp: "SIMATIC S7"
    property string errorMsg:""
    ModalDialog {
        id: authenticationErrorMsg
        title: stringStore.autFailedMsgTitle
        text: stringStore.authenticationRequired
        button_OK: true
    }


    AppDef{
        id: def
    }

    StringStore
    {
        id:stringStore
    }


    ConnectionListDelegateModel{
        id:connectionDelegateModel
    }

    TagListDelegateModel{
        id:tagDelegateModel
    }

    ConnectionModel
    {
        id: connectionModel
        onDataChanged: {
            screen_systemInfo.plcInfo = conController.getInfoFromBasicInfo(mainObject.currentIndex,"station-name")           
           

            Logger.consoleLog(def.logInfo,"onDataChanged.....PLCInfo......."+screen_systemInfo.plcInfo)
            
        }
    }
    TagModel
    {
        id: tagModel
        onDataChanged: {
            if(busyIndicator.visible == true)
                busyIndicator.visible = false
        }
    }

    InfoModel
    {
        id: infoModel
    }

    ModuleInfoModel
    {
        id: detailedInfoModel
    }

    ConnectionListItem
    {
        id: connectionDelegate
    }
    TagListItem
    {
        id: tagListDelegate
    }
    DiagnosticListItem
    {
        id: diagnosticListItem
    }
    
    TrendViewListItem{
        id:trendViewListItem
    }

    TrendViewIOListItem{
        id:trendViewIOListItem
    }
    DataGridItem{
        id:dataGridItem
    }
    ToastMessage{
        id:toastMsg
    }


    ConnectionController
    {
        id:conController
        connectionType : ConnectionController.enS7Connection
        Component.onCompleted:
        {
            setConnectionModel(connectionModel);
            setModuleInfoModel(detailedInfoModel);
        }
     }

    IOStatusTableModel{
        id: ioStatusTableModel
    }

    IOStatusListModel{
        id: ioStatusListModel
    }

    IOStatusController{
    id:ioStatusController
    Component.onCompleted: {
        setIOStatusModel(ioStatusTableModel)
        }
    }

    DiagBufferModel{
        id: diagnosticBufferModel;
    }
    DiagBuffController{
        id:  diagnosticbuffController
        Component.onCompleted:
        {
            setDiagModel(diagnosticBufferModel)
        }
    }


    TagController
    {
        id:tagController
        Component.onCompleted:
        {
            setTagModel(tagModel);
        }
    }

    LinearGradient {
            anchors.fill: parent
            start: Qt.point(0, 0)
            end: Qt.point(parent.width, 0)
            gradient: Gradient {
                GradientStop { position: 0.0; color: def.backgroundgradientleft }
                GradientStop { position: 1.0; color: def.backgroundgradientright}
            }
        }
	
	
    WaitIndicator{
        id: busyIndicator
        visible:false
        anchors.horizontalCenter:parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        z:20
    }

    ScreenDeviceOverview{
        id: screen_overview
        width: parent.width
        height: parent.height
        x: parent.width
        screenMode: 0           // 0: normal view, 1: editing view
        opacity: 0

    }

    ScreenSplash{
        id: screen_splash
        width: parent.width
        height: parent.height
    }

    ScreenSystemInfo{
        id: screen_systemInfo
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenAuthentication{
        id:screen_authentication
        width: parent.width
        height: parent.height
        visible:false
    }

    ScreenTagList{
        id: screen_tagList
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
        screenMode: 0           // 0: normal view, 1: editing view
    }

    ScreenTagEdit{
        id: screen_tagEdit
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenTrendView{
        id: screen_trendView
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenTrendView_FullScreen{
        id: screen_trendView_FullScreen
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0

    }

    ScreenDataLog{
        id: screen_dataLog
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenEditConnection
    {
        id: screen_editConnection
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenWriteToPLC
    {
        id: screen_writeToPLC
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0

    }

    ScreenLogin{
       id: appLogin
       width: parent.width
       height: parent.height
       x: parent.width
       opacity: 0
    }

    ScreenSettings{
        id: screen_settings
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenAppInformation{
        id: screen_appInfo
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenAppInformationLicense{
        id: screen_appInfoLicense
        width: parent.width
        height: parent.height
        opacity: 0
        visible: false
    }
    
    ScreenAppInformationContent{
        id: screen_appInfoContent
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }    
    
    ScreenFrequencySettings{
        id: refresh_freq_settings
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    LanguageSettings {
        id: language_settings
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }    

    ModalDialog {
        id: sslCertDialog
        dialogWidth: parent.width-2*def.borderDistanceLeft
        title: stringStore.sslHeader
        useScrollable: true
        text_scrollable:""

        onAcceptButtonClicked: {
            if(errorMsg === "S7_INTERNALSERVER_ERROR")
                screen_overview.conFailedMsg.openWithErrorMessage(errorMsg)
            errorMsg = ""
            conController.sslCertUserAction(true)
        }

        onDeclineButtonClicked:{
            errorMsg = ""
            conController.sslCertUserAction(false)
        }
    }

    ScreenChangePassword {
        id: screen_changepwd
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenIOStateOverview{
        id: screen_iostateoverview
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }

    ScreenIOState{
        id: screen_iostate
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }
    ScreenDiagnostics{
        id: screen_diag
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }
    ScreenDiagnosticsDetails{
        id: screen_diagdetails
        width: parent.width
        height: parent.height
        x: parent.width
        opacity: 0
    }


    StackView{
        id: stackview
        anchors.fill: parent
        initialItem: screen_splash

        function popCurrentPage(currentScreen)
        {
            if(currentScreen === stackview.currentItem)
                stackview.pop()
        }

        function popPage()
        {

            stackview.pop()
        }

        function pushPage(page)
        {
            if(stackview.currentItem !== page)
            {
                stackview.push(page)
            }
            
        }


        delegate: StackViewDelegate {
            function transitionFinished(properties)
            {
                properties.enterItem.opacity = 1
                properties.enterItem.x = 0
                properties.exitItem.focus = false
                properties.enterItem.focus = true
                properties.exitItem.x = parent.width
                properties.exitItem.opacity = 0
                Qt.inputMethod.hide()
            }
            pushTransition: StackViewTransition {
                /*
                NumberAnimation {
                    target: enterItem;
                    property: "x";
                    to: 0
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
                NumberAnimation {
                    target: exitItem;
                    property: "x";
                    to: 0-exitItem.width;
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
            }

            popTransition: StackViewTransition {
                NumberAnimation {
                    target: enterItem;
                    property: "x";
                    to: 0
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
                NumberAnimation {
                    target: exitItem;
                    property: "x";
                    to: exitItem.width;
                    duration: def.screenchangetime;
                    easing.type: Easing.InOutQuad
                }
            }
            replaceTransition: StackViewTransition {*/
                NumberAnimation {
                    target: enterItem
                    property: "opacity"
                    from: 0
                    to: 1
                    easing.type: Easing.InOutQuad
                    duration: def.screenchangetime
                }
                PropertyAnimation {
                    target: exitItem
                    property: "opacity"
                    from: 1
                    to: 0
                    duration: def.screenchangetime

                }
            }
        }

    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed: S7 main" )
        if (event.key === Qt.Key_Back)
        {
            event.accepted = true;
            if(stackview.currentItem == screen_overview)
            {
                if(screen_overview.screenMode == 1)
                {
                    screen_overview.screenMode = 0
                }
                else
                {
                    Qt.quit();
                }
            }
            else if(stackview.currentItem == appLogin)
            {
                Qt.quit();
            }
            else
                stackview.pop()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }

 Component.onCompleted: {
        appController.appStatusChanged.connect(screen_authentication.onAppstatusChanged)
    }

}
