import QtQuick 2.0
import DataStoreComponents 1.0
import LoggerComponents 1.0

import "." as S7App_Components

Rectangle {

    id:changePwdRect
    property int screenMode: -1 //-1: Invalid 0: Reset Password; 1: Set Password ; 2: Change password


    width: parent.width
    height: parent.height

    color: "transparent"

    Headline{
        id: headline
        z: 2
        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Change Password clicked!")
            backButtonOperation()

        }
    }

    ModalDialog {
        id: pwdSavedDialog
        title:""
        text: stringStore.password_saved
        button_OK: true
        button_Cancel: true
        onOkButtonClicked: {
            if(screenMode === 0 || screenMode === 1)
            {
                stackview.replace(screen_overview)
            }
        }
        onCancelButtonClicked: {
            screen_overview.clearConnectionList()
            stackview.push(appLogin)
        }

    }

    ModalDialog {
        id: pwdErrorDialog
        title:""
        button_OK: true
    }



    Flickable{
        id:flickCtl
        width: parent.width
        height: parent.height-headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height + saveButton.height
        contentWidth: parent.width
        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column
        {
            id: col
            anchors.top: parent.top
            anchors.left: parent.left
            width: parent.width

            Rectangle{ // just space
                width: parent.width
                height: def.borderDistanceTop
                color: "transparent"
            }

            DefaultText{
                id: explanation
                text: (screenMode === 0) ? stringStore.resetpassword_explanation : stringStore.setpassword_explanation

                color: def.colFontGeneral

                font.pixelSize: def.fontSizeNormalText
                font.bold: false
                wrapMode: Text.Wrap

                width: parent.width-def.borderDistanceLeft-def.borderDistanceRight
                anchors {left: parent.left; leftMargin: def.borderDistanceLeft}

            }

            Rectangle{ // just space
                width: parent.width
                height: def.borderDistanceTop
                color: "transparent"
            }

            InputField{
                id: oldPwdText
                label: stringStore.enterOldPW
                input: ""
                echoMode: TextInput.Password
                visible:false
            }

            InputField{
                id: newPwdText
                label: stringStore.enterNewPW
                input: ""
                echoMode: TextInput.Password
            }

            InputField{
                id: confirmNewPwdText
                label: stringStore.confirmNewPW
                input: ""
                echoMode: TextInput.Password
            }

            Button{
                id:saveButton
                width: parent.width/3 // to be done with designer
                anchors{right:parent.right; rightMargin: def.borderDistanceRight}

                button_Text: stringStore.save
                isTextbutton: true


                onButtonClicked: {
                    Logger.consoleLog(def.logInfo,"Save from ChangePassword clicked!")

                    if(newPwdText.input === confirmNewPwdText.input)
                    {

                        Logger.consoleLog(def.logInfo,"New Pwd and confirm pwd identical") // + newPwdText.input)
                        // Logger.consoleLog(def.logInfo,"Confirm Pwd: " + confirmNewPwdText.input)

                        if(screenMode === 0 || screenMode === 1)
                        {
                            if(screenMode === 0)
                            {
                                screen_overview.initializeConnList()
                                DataStore.resetPassword("LogInPwd", newPwdText.input);
                                screen_overview.updateConnectionList();
                            }
                            else if(screenMode === 1)
                            {
                                DataStore.resetPassword("LogInPwd", newPwdText.input , false)
                                screen_overview.initializeConnList()

                            }
                            if(newPwdText.input == "")
                                stackview.replace(screen_overview)
                            else
                                pwdSavedDialog.open()
                        }
                        else if(screenMode === 2)
                        {
                            if(DataStore.authenticateUser(oldPwdText.input))
                            {
                               // pm/design decision 29.10.2014 - NO TOAST MESSAGE NEEDED (see RQ 1111428)
                               // toastMsg.toast.show(stringStore.password_saved_short)

                                screen_overview.initializeConnList()
                                DataStore.resetPassword("LogInPwd", newPwdText.input,false);
                                screen_overview.updateConnectionList();
                                stackview.replace(screen_overview)


                            }
                            else
                            {
                                oldPwdText.input = ""
                                pwdErrorDialog.text = stringStore.wrongOldPasswordMsg
                                pwdErrorDialog.open()
                            }
                        }
                        else
                        {
                            stackview.popPage()
                        }

                        //TODO: some more activities ???
                    }
                    else
                    {

                        newPwdText.input =""
                        confirmNewPwdText.input = ""
                        pwdErrorDialog.text = stringStore.wrongNewPasswordsMsg
                        pwdErrorDialog.open()

                    }

                }
            }
        }
    }

    Component.onCompleted: {
        if( DataStore.getSettings("TermsDialogLaunched") == "0" )
        {
            firstLaunchTermsDialog.open();
        }

        appController.appStatusChanged.connect(onAppstatusChanged)
    }

    function onAppstatusChanged()
    {

        if(appController.appStaus !== def.appActive)
        {
            
            newPwdText.input = ""
            confirmNewPwdText.input = ""
            oldPwdText.input = ""
        }
    }

    ModalDialog {
        id: firstLaunchTermsDialog
        dialogWidth: parent.width-2*def.borderDistanceLeft
        title: stringStore.info_TermsOfUse
        useScrollable: true
        text_scrollable: infoModel.getInfoFile(stringStore.info_lang, 0)
        button_OK: true
        button_Cancel: true

        onCancelButtonClicked: {
            Qt.quit();
        }

        onOkButtonClicked: {
            DataStore.saveSettings("TermsDialogLaunched", "1" );
        }
    }

    ModalDialog{
        id: changePwdSettingsGoBackMsg
        title: (screenMode === 2) ? stringStore.changePassword : stringStore.reset_password
        text: stringStore.goBackAlertMessage
        button_Yes: true
        button_No: true

        onYesButtonClicked: {
            stackview.popPage()
        }
    }
    onOpacityChanged: {
        if(opacity == 1)
        {
            newPwdText.input = ""
            confirmNewPwdText.input = ""
            oldPwdText.input = ""
        }
        else if(opacity == 0)
        {
            flickCtl.contentY = 0
            screenMode = -1
        }
    }

    onScreenModeChanged:{

        if(screenMode === 0)
        {
            oldPwdText.visible = false
            headline.backbutton_visible = true
            headline.headline_text = stringStore.reset_password
            pwdErrorDialog.title = stringStore.reset_password
            pwdSavedDialog.title = stringStore.reset_password

            
        }
        else if(screenMode === 1)
        {
            oldPwdText.visible = false
            headline.backbutton_visible = false
            headline.headline_text = stringStore.set_password
            pwdErrorDialog.title = stringStore.set_password
            pwdSavedDialog.title = stringStore.set_password
            
        }

        else if(screenMode === 2)
        {
          
           oldPwdText.visible = true
           headline.backbutton_visible = true
           headline.headline_text = stringStore.changePassword
           pwdErrorDialog.title = stringStore.changePassword
           pwdSavedDialog.title = stringStore.changePassword
        }        
    }

    function backButtonOperation()
    {
        if(screenMode === 2 || screenMode === 0)
        {
            if(newPwdText.input !="" || confirmNewPwdText.input !="")
            {
                changePwdSettingsGoBackMsg.open()
                return;
            }
        }
        stackview.popPage();
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Change Password settings" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }
}
