import QtQuick 2.0
import QtQml.Models 2.1
import LoggerComponents 1.0

import "." as S7App_Components

Component {

    Rectangle {
        id: tagItem
        width: mainObject.width
        height: def.tagListEntry  //mainObject.height / (def.nListviewEntries+1);
        color: "transparent"

        property int screenState:screen_tagList.screenMode
        property bool clearSelectedElement:screen_tagList.clearSelectedItem

        MouseArea {
            id: mouseArea
            enabled:true
            height: def.listviewEntryHeight;
            width:parent.width - def.standardButtonWidth
            anchors {
                left: parent.left
                verticalCenter: parent.verticalCenter
            }

            onPressed:{
                if(screenState === 0)
                tagItem.color = def.pressedStateButtonColor
            }

            onReleased: {
                tagItem.color ="transparent"
            }

            onContainsMouseChanged: {
               if (!containsMouse)
                    tagItem.color = "transparent"
            }


            onClicked: {
                Logger.consoleLog(def.logInfo,"Tag clicked = " + index)
                screen_tagList.tagListObj.setTagListCurrentIndex(index)
                if(screenState === 0)
                {
                        screen_writeToPLC.currentIndex = index
               		screen_writeToPLC.curValLabel = tagModel.getValue(index)
               		stackview.pushPage(screen_writeToPLC)
                }
            }
        }
        MouseArea {
           id: delegateRoot
            enabled:true
            property int visualIndex: index

            width: movebutton.width*1.4
            height: movebutton.height*1.4

            anchors.centerIn: movebutton

            drag.target: icon
            clip: false

            Rectangle {
               id: icon
               height:def.listviewButtons
               width: height
               radius: 5
               color: "transparent"
               visible:false
               property bool isDragActive:delegateRoot.drag.active

               anchors {
                   horizontalCenter: parent.horizontalCenter
                   verticalCenter: parent.verticalCenter
               }

               Drag.active: delegateRoot.drag.active
               Drag.source: delegateRoot

               onIsDragActiveChanged: {
                   screen_tagList.tagListObj.listViewObj.currentIndex = index
                   if(isDragActive)
                   {
                       scrollTimer.start()
                       Logger.consoleLog(def.logCritical,"Timer active ")
                       screen_tagList.clearSelectedItem = true
                   }
                   else
                   {
                       scrollTimer.stop()
                       Logger.consoleLog(def.logCritical,"Timer stopped ")
                       screen_tagList.clearSelectedItem = false
                   }

               }

               states: [
                   State {
                       when: icon.Drag.active
                       ParentChange {
                           target: icon
                           parent: screen_tagList
                       }

                       AnchorChanges {
                           target: icon;
                           anchors.horizontalCenter: undefined;
                           anchors.verticalCenter: undefined
                       }
                   }
               ]
            }

            DropArea {
                id:dropArea
                width: mainObject.width - 2*def.borderDistanceLeft
                height: parent.height*2
                anchors.right: parent.right
                anchors.verticalCenter: parent.verticalCenter

                onEntered: {
                    Logger.consoleLog(def.logInfo,"on Entered")
                    if(screenState === 1) // Move Mode
                    {
                        Logger.consoleLog(def.logInfo,"Grag index from toTo = " + delegateRoot.visualIndex + "  " +drag.source.visualIndex )
                        //Logger.consoleLog(def.logInfo,"Current on enterd X =" + drag.x + "Current on enterd Y = " + drag.y)
                        if(drag.source.visualIndex !== delegateRoot.visualIndex)
                        {
                            if(delegateRoot.visualIndex > drag.source.visualIndex )
                                tagModel.moveElement(drag.source.visualIndex, delegateRoot.visualIndex)
                            else
                                tagModel.moveElement(delegateRoot.visualIndex, drag.source.visualIndex)
                        }

                    }
                }
            }
        }


        Checkmark{
            id: checkbox
            visible: (screenState === 1) ? true : false
            anchors {left: parent.left; leftMargin: def.borderDistanceLeft;verticalCenter: parent.verticalCenter}
            onCheckmarkClicked: {
                isSelected = !isSelected;
                tagModel.selectElement(index,isSelected);
            }
            Component.onCompleted: {
                isSelected = tagModel.isElementSelected(index)
            }
        }


        Column
        {
            id: connectionColumn
            spacing: def.listviewSpaceColumn
            anchors.left: (screenState === 1) ? checkbox.right : parent.left
            anchors.leftMargin: def.borderDistanceLeft
            anchors.top: parent.top
            anchors.topMargin: def.borderDistanceTop
            anchors.right: parent.right
            anchors.rightMargin: def.borderDistanceRight

            DefaultText{
                id: idName
                text: "Name: " + Id
                font.pixelSize:  def.listviewFontDetails
                elide:Text.ElideRight
                width:tagItem.width/2
                maximumLineCount: 20

            }

            VerticalSpace{
                height: tagItem.height /5
            }

            DefaultText{
                id: valueName
                text:"Value: " + Value
                width: (screenState === 1) ? parent.width-movebutton.width- def.borderDistanceRight - tagSettings_background.width : parent.width
                wrapMode: Text.WrapAnywhere
                font.pixelSize:  def.listviewFontHL
                font.bold: (screenState === 0)?true:false
            }

        }

        Image {
            id: movebutton
            width: def.headlineButtonSize
            height: width
            visible: icon.visible
            anchors{verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: def.borderDistanceRight}
            source: "pictures/ICO_App_ChangeListentry.svg"
        }
        Rectangle{
            id:tagSettings_background
            width:tagSettings.width*1.4
            height:width
            color:"transparent"
            visible:false
            anchors{verticalCenter: parent.verticalCenter; right: movebutton.left; rightMargin: def.borderDistanceRight}

            MouseArea{
                anchors.fill: parent

                onClicked: {
                    Logger.consoleLog(def.logInfo,"Initiating screen change")
                    screen_tagList.tagListObj.setTagListCurrentIndex(index)
                    screen_tagEdit.addressLabel = Address
                    if(def.isS7App && connectionModel.getPlcType(mainObject.currentIndex) === 1)
                    {
                        screen_tagEdit.formatLabel = screen_tagEdit.getFormat(Format)
                        screen_tagEdit.spinnerFormatType.currentIndex = Format
                    }
                    else
                    {
                        if(def.isS7App && Format == "DEC")
                        {
                            screen_tagEdit.formatLabel = "DEC+/-"
                            screen_tagEdit.formatvalue = "DEC"
                        }
                        else
                            screen_tagEdit.formatLabel = screen_tagEdit.formatvalue = Format
                        screen_tagEdit.spinnerFormatType.currentIndex = screen_tagEdit.formatIndex(Format)
                    }
                    screen_tagEdit.idLabel = Id
                    screen_tagList.currentIndex = index
                    Logger.consoleLog(def.logInfo,"Values for edit mode " + screen_tagEdit.addressLabel  + screen_tagEdit.formatLabel)
                    stackview.pushPage(screen_tagEdit)
                }
                onPressed: {
                    tagSettings_background.color = def.pressedStateButtonColor
                    tagSettings.source = "pictures/ICO_APP_Settings_pressed.svg"
                }
                onReleased: {
                    tagSettings_background.color = "transparent"
                    tagSettings.source = "pictures/ICO_APP_Settings.svg"
                }
                onContainsMouseChanged: {
                    if(!containsMouse)
                    {
                        tagSettings_background.color = "transparent"
                        tagSettings.source = "pictures/ICO_APP_Settings.svg"
                    }
                }
            }
            Image{
                id:tagSettings
                width:def.headlineButtonSize
                height:width
                visible:tagSettings_background.visible
                anchors.centerIn: parent
                fillMode: Image.PreserveAspectFit
                source: "pictures/ICO_APP_Settings.svg"
            }
        }

       onScreenStateChanged:
        {
            if (screenState == 0)
            {
                icon.visible = false
                dropArea.visible = false
                delegateRoot.drag.target = undefined
                tagSettings_background.visible = false

            }
            else if (screenState == 1)
            {
                icon.visible = true
                dropArea.visible = true
                delegateRoot.drag.target = icon
                tagSettings_background.visible = true

            }

        }

       ListSeperator{
           anchors.bottom: parent.bottom
           // visible: index==0 ? false : true // no seperator above first entry
       }


       function checkDeviceState()
       {
           var deviceState = conController.getDeviceState()
           if(deviceState === 1 ) //ConnectionController.enStopState
           {
               changemodeMsg.text = "Device is Stop mode. Do you want to change it to RUN mode?"
               changemodeMsg.open()
           }
           else if(deviceState === 0) //ConnectionController.enRunState
           {
               screen_writeToPLC.currentIndex = index
               screen_writeToPLC.curValLabel = tagModel.getValue(index)
               stackview.pushPage(screen_writeToPLC)
           }
           else if(deviceState === 2)
           {
               tagController.stopRefreshTags();
               conController.disConnectFromHost()

               stackview.replace(screen_overview)

               screen_overview.conFailedMsg.openWithErrorMessage("")

               tagModel.removeAll()
               tagController.stopRefreshTags();
           }
       }

       ModalDialog {
           id: changemodeMsg
           title: stringStore.logoChangeModeMsgTitle
           button_Yes: true
           button_No: true
           onYesButtonClicked: {
               conController.setPLCMode(0) //change to run mode
               var deviceState = conController.getDeviceState()

               if(deviceState === 0) //ConnectionController.enRunState
               {
                   screen_writeToPLC.currentIndex = index
                   screen_writeToPLC.curValLabel = tagModel.getValue(index)
                   stackview.pushPage(screen_writeToPLC)
               }
               else if(deviceState === 2)
               {
                   conController.disConnectFromHost()
                   stackview.replace(screen_overview)
                   screen_overview.conFailedMsg.openWithErrorMessage("")

                   tagModel.removeAll()
               }
           }
       }
       onClearSelectedElementChanged: {
           if(checkbox.isSelected )
           {
               checkbox.isSelected = false
               tagModel.clearElementSelection();
           }

       }

       Timer{
           id:scrollTimer
           interval: 125; running: false; repeat:true
           property int runs:0
           onTriggered: {
               var currPos = delegateRoot.mapToItem(mainObject,mainObject.x,mainObject.y)
               Logger.consoleLog(def.logCritical,"Current Pos " + currPos.y)

               if(currPos.y < screen_tagList.scrollUpLimit && currPos.y > def.headlineHeight)
               {
                   screen_tagList.tagListObj.scrollUpListView()
               }
               else if(currPos.y > screen_tagList.scrollDownLimit)
               {
                   screen_tagList.tagListObj.scrollDownListView()
               }
           }
           }

    }

}


