
import QtQuick 2.2
import QtCharts 2.0
import DataStoreComponents 1.0
import TrendDataSourceComponents 1.0
import LoggerComponents 1.0

import "." as S7App_Components

ChartView {
    id:trendView
    width:parent.width
    height:parent.height
    //title: "Live Tag Values"
    antialiasing: true
    backgroundColor: "transparent"
    legend.visible: false
    margins.top:0
    margins.left:0
    margins.bottom:0
    margins.right:0

    property bool isTrendInitialized:false
    property bool isTrendingStarted:false
    property int bufferSize:150000 //time in miliseconds
    property int yAxisMinima:0
    property int yAxisMaxima:100
    property int valueCount:0
    property int bufferValues:0
    property var dataModel:0

    ValueAxis {
        id: valueAxisYRight
        min: yAxisMinima
        max: yAxisMaxima
        labelsVisible:false
        color:def.trendViewAxisColor
        Component.onCompleted: {
            TrendDataSource.setAxisGridLineColor(valueAxisYRight,def.trendViewAxisColor)
        }
    }

    DateTimeAxis {
        id: dtAxisX
        tickCount: 5
        labelsVisible:false
        color:def.trendViewAxisColor
        Component.onCompleted: {
            TrendDataSource.setAxisGridLineColor(dtAxisX,def.trendViewAxisColor)
        }
    }

    LineSeries {
        id: defaultSeries
        name:"default"
        axisX: dtAxisX
        axisYRight: valueAxisYRight
    }

    MouseArea{
        id:trendMouseArea
        anchors.fill:parent
        property int initX:0
        property int initY:0
        onClicked:{
            Logger.consoleLog(def.logInfo,"Clicked on preview Trend ============================ ")

            if(dataModel.getSelectedDataElementCount() > 0)
            {
                if((mouseX > initX - 20 && mouseX < initX + 20) && (mouseY > initY - 20 & mouseY < initY + 20 ) )
                {
                    busyIndicator.visible = true
                    stackview.pushPage(screen_trendView_FullScreen)
                    trendView.clearTrendView()
                }
            }
        }
        onPressed: {
            initX = mouseX
            initY = mouseY
        }

    }

    function createTagSeries(index)
    {
        if(isTrendInitialized)
        {
            trendView.removeSeries(trendView.series("default"))
            isTrendInitialized = false
            var freq = DataStore.getSettings("rf_seconds")
            bufferValues = bufferSize/(freq * 1000)
            Logger.consoleLog(def.logInfo,"buffer value  " + bufferValues)
            TrendDataSource.setBufferSize(bufferValues)
        }
        valueCount = 0
        var trend = trendView.createSeries(ChartView.SeriesTypeLine, "" + index,dtAxisX,valueAxisYRight)
        trend.color = TrendDataSource.getColor()
        trend.width = def.trendViewLineWidth
        return trend.color;
    }

    function updateTagSeries(index,value)
    {
        var trend = trendView.series("" + index)
        if(trend !== null)
        {

            updateXAxisRange()

            updateYAxisRange(value)

            if(valueCount >= bufferValues + (bufferValues * 0.1))
            {
                var point = trend.at(0)
                trend.remove(point.x,point.y)
                trend.append(dtAxisX.max,value)
            }
            else
            {
                trend.append(dtAxisX.max,value)
            }
            Logger.consoleLog(def.logInfo,"total values in the trend == " + valueCount + " == " +  bufferValues )
        }

    }

    function removeTagSeries(index,trendColor)
    {
        trendView.removeSeries(trendView.series("" + index))
        TrendDataSource.removeValue(("" + index),trendColor,index)
        valueAxisYRight.max = yAxisMaxima
        valueAxisYRight.min = yAxisMinima
        Logger.consoleLog(def.logInfo,"Trend cout after delete   ============== " + trendView.count )
    }

    function removeAllTagSeries()
    {
        valueCount = 0
        trendView.removeAllSeries()
        TrendDataSource.removeAllValue()
        screen_trendView_FullScreen.trendViewObj.removeAllTagSeries()
    }

    function clearTrendView()
    {
        valueCount = 0
        trendView.removeAllSeries()
        initializeTrendView()
    }

    function reStartTrendView()
    {
        var trendList = dataModel.getSelectedDataElement();
        valueCount = 0
        for(var i = 0; i< dataModel.getSelectedDataElementCount(); i++)
        {
            Logger.consoleLog(def.logInfo,"Restart preview trend screen" + trendList[i])
            if(trendList[i] >= 0)
            {
                var trend = trendView.createSeries(ChartView.SeriesTypeLine, "" + trendList[i],dtAxisX,valueAxisYRight)
                trend.color = TrendDataSource.getFullScreenTrendColor(i)
                trend.width = def.trendViewLineWidth
                screen_trendView_FullScreen.trendViewObj.createTagSeries(trendList[i],TrendDataSource.getFullScreenTrendColor(i))
            }
        }
    }

    function updateXAxisRange()
    {
        dtAxisX.max = new Date()
        var startTime = dtAxisX.max

        startTime.setTime(startTime.getTime() - bufferSize)
        dtAxisX.min = startTime
    }

    function updateYAxisRange(value)
    {
        if(value !== undefined)
        {
            if(valueAxisYRight.max <= value)
            {
                Logger.consoleLog(def.logInfo,"In High tag Seies ")
                valueAxisYRight.max = value
            }
            else
            {
                if(!isTrendingStarted)
                {
                    valueAxisYRight.min = value
                    isTrendingStarted = true
                }
                else if(valueAxisYRight.min > value)
                {
                    Logger.consoleLog(def.logInfo,"In Low tag Seies " + valueAxisYRight.min  + "  " + value)
                    valueAxisYRight.min = value
                }
            }
        }
    }

    function getTrendColor(index)
    {
        var trend = trendView.series("" + index)
        return trend.color
    }

    function onDataUpdate()
    {
        if(screen_trendView.opacity == 1)
        {
            var result;
            result = dataModel.getUpdatedData()
            TrendDataSource.setUpdatedTrendValue(result)

            valueCount++;
            for(var j = 0 ; j < dataModel.getSelectedDataElementCount() ; j++)
            {
                var trendIndex;
                var trendvalue;
                trendvalue = TrendDataSource.getUpdatedTrendValue(j)
                if(trendvalue !== -1)
                {
                    trendIndex = TrendDataSource.getTrendIndex(j)
                    Logger.consoleLog(def.logInfo,"Updating the trend value ============== " + trendvalue + " at index  ============== " + trendIndex )
                    updateTagSeries(trendIndex,trendvalue)
                    busyIndicator.visible = false
                }

            }
        }
    }

    function onAppStatusChanged(isActive)
    {
        if(appController.appStaus !== def.appActive)
        {
            clearTrendView()
            reStartTrendView()
            if(screen_trendView.dataController !=0)
            screen_trendView.dataController.stopRefreshing()
            Logger.consoleLog(def.logInfo,"Current app status ============= " +  appController.appStaus)
        }
    }

    Component.onCompleted: {
    }

    Component.onDestruction: {
        trendView.removeAllSeries()
    }

    function initializeTrendView()
    {
        isTrendInitialized = true
        updateXAxisRange()
        setAxisRange(yAxisMinima,yAxisMaxima)

        screen_trendView_FullScreen.trendViewObj.initializeTrendView()
    }

    function setDataModel(model)
    {
        dataModel = model
    }

    function setAxisRange(minima,maxima)
    {
        valueAxisYRight.min = minima
        valueAxisYRight.max = maxima
        yAxisMinima = minima
        yAxisMaxima = maxima
        Logger.consoleLog(def.logInfo," Preview Updating the trend axis range  ============== " + minima +  maxima)
        screen_trendView_FullScreen.trendViewObj.setAxisRange(minima,maxima)
    }

    function setTickCount(count)
    {
        valueAxisYRight.tickCount = count

        screen_trendView_FullScreen.trendViewObj.setTickCount(count)
    }

}
