import QtQuick 2.2
import LoggerComponents 1.0

import "." as S7App_Components

Component {


    Item {
        id: tagItem
        width: parent.width
        height:def.trendviewListItemHeight
        property int time:0
        property bool clearSelectedTrend:screen_trendView.clearSelectedTrend
        property color trendColor:def.trendViewIndicatorDefaultColor

        MouseArea {
            id: mouseArea
            enabled:true
            anchors.fill: parent
            onClicked: {
                Logger.consoleLog(def.logInfo,"Tag clicked = " + index)
            }
        }

        Checkmark{
            id: checkbox
            visible: true
            height:def.trendViewListCheckBoxHeight
            width:height
            anchors {left: parent.left; leftMargin: def.borderDistanceLeft;verticalCenter: parent.verticalCenter}
            onCheckmarkClicked: {

                isSelected = !isSelected;              
                if(screen_trendView.dataModel.getSelectedDataElementCount() === 8 && isSelected)
                {
                    isSelected = false

                    screen_trendView.trendMsg.title = stringStore.trendAlertMsgTitle
                    screen_trendView.trendMsg.text = stringStore.trendLimitationMsg
                    screen_trendView.trendMsg.open()
                }
                else
                {
                    var result = isFormatTypeSupported(Format)
                    Logger.consoleLog(def.logCritical,"Selected data type " + Format)

                    if(result === true)
                    {
                        if(isSelected)
                        {
                            trendColor = screen_trendView.trendViewObj.createTagSeries(index)
                            Logger.consoleLog(def.logCritical,"==================Trend color selected ========================= " + trendColor + " " +  index)
                            screen_trendView_FullScreen.trendViewObj.createTrend(index,trendColor)
                        }
                        else
                        {
                            screen_trendView.trendViewObj.removeTagSeries(index,trendColor)
                            screen_trendView_FullScreen.trendViewObj.removeTagSeries(index)
                            trendColor = def.trendViewIndicatorDefaultColor
                        }
                        screen_trendView.dataModel.selectDataElement(index,isSelected)
                    }
                    else
                    {
                        isSelected = !isSelected;
                        if(def.isS7App)
                        {
                            screen_trendView.trendMsg.text = stringStore.trendDataTypeLimitationMsgS7
                        }
                        else
                        {
                            screen_trendView.trendMsg.text = stringStore.trendDataTypeLimitationMsgLogo
                        }
                        screen_trendView.trendMsg.title = stringStore.trendAlertMsgTitle
                        screen_trendView.trendMsg.open()
                    }

                }
            }

            Component.onCompleted: {
                if(screen_trendView.dataModel != 0)
                {
                    isSelected = screen_trendView.dataModel.isSelectedDataElement(index)
                    if(isSelected)
                    {
                        trendColor = screen_trendView.trendViewObj.getTrendColor(index)
                        if(trendColor === 0)
                        {
                            trendColor = def.trendViewIndicatorDefaultColor
                        }
                    }
                }
            }
        }

        Column
        {
            id: connectionColumn
            spacing: def.trendViewListSpaceColumn
            anchors.left: checkbox.right
            anchors.top: checkbox.top
            anchors.leftMargin: def.trendViewListTextOffset

            DefaultText{
                id: idName
                text: Id
                font.pixelSize:  def.trendViewListTextHeight
                elide:Text.ElideRight
                width:tagItem.width/2
                maximumLineCount: 20
                color:def.trendViewItemColor

            }

        }

        Rectangle {
           id: icon
           height:def.trendViewListIndicatorHeight
           width: height
           color: trendColor
           border.color: trendColor
           border.width: 1
           visible:true

           anchors {
               right:parent.right
               rightMargin:def.borderDistanceRight
               verticalCenter: parent.verticalCenter
           }
        }

        ListSeperator{
            anchors.bottom: parent.bottom
        }

        onClearSelectedTrendChanged: {
            checkbox.isSelected = false
            trendColor = def.trendViewIndicatorDefaultColor
        }

        function isFormatTypeSupported(dataType)
        {
            if(def.isS7App && 1 === connectionModel.getPlcType(mainObject.currentIndex))
            {
                //only BIN,HEX,DEC+/- and DEC_UNSIGNED is supooted data format for S7 1500 PLC
                if(Format == "0" ||  Format == "1" || Format == "2" || Format == "15")
                {
                    return true;
                }
                else
                {
                    return false;
                }

            }
            else if(def.isS7App && 0 === connectionModel.getPlcType(mainObject.currentIndex))
            {
                //only BIN,HEX,DEC+/- and DEC_UNSIGNED is supooted data format for S7 1200 PLC
                if(Format == "BIN" ||  Format == "HEX" || Format == "DEC" || Format == "DEC_UNSIGNED")
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                 if(Format == "ASCII" ||  Format == "BIT" || Format == "CHARACTER"  || Format == "DATE" )
                 {
                     return false;
                 }
                 else
                 {
                     return true;
                 }

            }

        }

   }

}


