import QtQuick 2.0
import QtQuick.Window 2.1
import LoggerComponents 1.0

Item {

    property int screenSizeClass: 2 // 0: 3,5" - 4"
                                    // 1: 4" - 6"
                                    // 2: 6" - 10"
                                    // 3: > 10"

    property bool isS7App: currentApp === "SIMATIC S7" ? true : false
    property bool isLogoApp: !isS7App
    property string splashScreen:""
    property real heightInMM:0
    property real widthInMM:0
    property bool isProtraitMode:false
    property bool isSplashScreenVisible: true
    property real screenSize:0

    //due to Qt enum bug not able to pass C++ enum as function parameters
    //keep sync with Logger.h enLogType will fix it later
    property int logInfo:0
    property int logWarning:1
    property int logCritical:2

    //due to Qt enum bug not able to pass C++ enum as function parameters
    //keep sync with Appcontroller.h enAppstatus will fix it later
    property int appSuspended:0
    property int appHidden:1
    property int appInActive:2
    property int appActive:4


    property bool isPLCConnectd : false


    function getScreenInformation ()
    {
        //isProtraitMode = (Screen.orientation == Qt.PortraitOrientation) ? true : false
        if(Screen.height > Screen.width)
        {
            isProtraitMode = true
        }
        else
        {
            isProtraitMode = false
        }


        if (heightInMM == 0) // not yet initialized
        {
            heightInMM = Screen.height/Screen.pixelDensity;
            widthInMM = Screen.width/Screen.pixelDensity;

            Logger.consoleLog(def.logInfo,"Screen heightInMM: " + heightInMM + "  - widthInMM: "+ widthInMM)

            if (heightInMM == 0) //no screen visible, function will be called later again
            {
                return;
            }

            screenSize = Math.sqrt (heightInMM*heightInMM+widthInMM*widthInMM)*0.039370

            if (screenSize > 10)
                screenSizeClass = 3;
            else if (screenSize > 6)
                screenSizeClass = 2;
            else if (screenSize > 4)
                screenSizeClass = 1
            else if (screenSize > 2)
                screenSizeClass = 0
            else
                screenSizeClass = 2 // most used mobiles

            Logger.consoleLog(def.logInfo,"Screen Inch: " + screenSize + " - Class: "+ screenSizeClass)
        }

        if (isSplashScreenVisible)
        {

            var appName = "logo"
            if (def.isS7App)
                appName = "S7"

            if (!isProtraitMode)
            {
                if (widthInMM/heightInMM > 1.5)
                    splashScreen = "pictures/"+appName+"Splash_Land_16_9.png"
                else
                    splashScreen = "pictures/"+appName+"Splash_Land_4_3.png"
            }
            else
            {
                if (heightInMM/widthInMM > 1.5)
                    splashScreen = "pictures/"+appName+"Splash_Port_16_9.png"
                else
                    splashScreen = "pictures/"+appName+"Splash_Port_4_3.png"
            }
            Logger.consoleLog(def.logInfo,"choosen splashscreen: " + splashScreen)
        }
    }

    function isOBA7()
    {
        if(isPLCConnectd)
        {
            if(isLogoApp && connectionModel.getPlcType(mainObject.currentIndex) === 0)
                return true;
            else
                return false;
        }
        return false;
    }

    function isOBA8()
    {
        if(isPLCConnectd)
        {
            if(isLogoApp && connectionModel.getPlcType(mainObject.currentIndex) === 1)
                return true;
            else
                return false;
        }
        return false;
    }

 // TIMES im ms
    readonly property int splashscreentime: 1500
    readonly property int screenchangetime: 200
    readonly property int checkStateTime: 5000


 // COLORS

    readonly property color colHeadlineBackground: "#d2dce1" //210.220.225
    readonly property color colHeadlineSep: "#b1bfc8"
    readonly property color colHeadlineShadow: "#ced6da"
    readonly property color colFontTitle: "#5a6e7d" //90.110.125
    readonly property color colFontHeadline: "#243746"    //36.55.70
    readonly property color colFontGeneral: colFontHeadline
    readonly property color backgroundgradientleft: "#f0f6f8" //240.246.248
    readonly property color backgroundgradientright: "#dce6eb"  //220.230.235
    readonly property color colButtonlineBackground: colHeadlineBackground
    readonly property color colButtonLineBright: "#b7c4cc"
    readonly property color colButtonLineDark: "#7a8c99"
    readonly property color colStandardButtonBackground: "#879baa"
    readonly property color colStandardButtonDark: "#768795"
    readonly property color colStandardButtonBright: "#a0aeba"
    readonly property color colScreenButtonBackground: colButtonlineBackground
    readonly property color colInputFieldText: "black"
    readonly property color colInputFieldBorder: colFontTitle
    readonly property color colDetailsBackground: "#fafcfc"
    readonly property color colListSeperator: colFontTitle
    readonly property color colorRun: "#647d2d"
    readonly property color colorStop: "#cc0000"
    readonly property color colInputFieldsFrame: "#c5c5c5"
    readonly property color colSelectedOn: "#0f55b9"
    readonly property color listviewHighLightColor:"#0f55b9"
    readonly property color normalStateButtonColor:"#879baa"
    readonly property color pressedStateButtonColor:listviewHighLightColor
    readonly property color activatedStateButtonColor:"#0f55e1"
    readonly property color focusedStateButtonColor:"#699be6"
    readonly property color disabledStateButtonColor:"#afbec8"
    readonly property color trendViewIndicatorDefaultColor:"#d0dadf"
    readonly property color trendViewAxisColor:"#879baa"
    readonly property color trendViewItemColor:"#384a58"
    readonly property color ioTableLineColor: colListSeperator
    readonly property color colDialog: "#dce6eb"
    readonly property color colDiagFont: "#233746"

    readonly property color fullScreenTrendViewItemColor:"#536470"

 // ALL SCREENS
    readonly property int deviceWidth: Math.min(Screen.height, Screen.width)
    readonly property int borderDistanceRight:  headlineHeight*0.17
    readonly property int borderDistanceLeft:   borderDistanceRight
    readonly property int borderDistanceTop:    standardFieldHeight*0.08
    readonly property int borderDistanceBottom:    standardFieldHeight*0.08
    readonly property int borderDistanceTopSmall: standardFieldHeight*0.05
    readonly property int seperatorLine:  1 //screenSizeClass <=1 ? 1 : 2  // Size in Pixel

    readonly property int standardFieldHeight: headlineHeight*1.39

    readonly property int inputFieldHeight: standardFieldHeight
    readonly property int inputFieldDistanceToTop: inputFieldHeight*0.085
    readonly property int inputFieldEditFieldHeight: inputFieldHeight*0.44

    readonly property int stateButtonHeight: headlineHeight *0.44
    readonly property int stateButtonWidth: headlineHeight * 1.35
    readonly property int standardButtonHeight: stateButtonHeight
    readonly property int standardButtonWidth: stateButtonWidth
    readonly property int standardButtonWidthExtended: standardButtonWidth*1.35
    readonly property int standardButtonWidthExtendedPlus: standardButtonWidthExtended * 1.35
    readonly property int menuButtonHeight: headlineButtonSize
    readonly property int menuButtonWidth: menuButtonHeight
    readonly property int selectionButtonHeight: standardFieldHeight*0.30
    readonly property int selectionButtonWidth: selectionButtonHeight

    readonly property int dialogWidth: headlineHeight * 3.85
    readonly property int dialogMaxHeight: mainObject.height - 2* headlineHeight
    readonly property int dialogTitleHeigth: headlineHeight * 0.54
    readonly property int dialogTitleFontSize: headlineHeight * 0.27

    readonly property int busyIndicatorSize: headlineHeight * 0.85;

    // FONTS
    readonly property string fontDefault:"Siemens Sans Regular"
    readonly property string fontBold:"Siemens Sans Bold"
    readonly property int fontSizeStationName: standardFieldHeight*0.15
    readonly property int fontSizeNormalText: standardFieldHeight*0.15
    readonly property int fontSizeNormalCaption: standardFieldHeight*0.15
    readonly property int fontSizeCaption: standardFieldHeight*0.15
    readonly property int fontSizeFunctionList: standardFieldHeight*0.14
    readonly property int headlineFontSize:    headlineHeight*0.26
    readonly property int titleFontSize:      headlineHeight*0.16
    readonly property int inputFieldTextSize: standardFieldHeight*0.15
    readonly property int fontSizeDetailCaption: standardFieldHeight*0.14
    readonly property int fontSizeDetails: standardFieldHeight*0.12




 // HEADLINE
    readonly property var headlineHeightPercentArray: [23, 18, 12, 7]
    readonly property int headlineHeightPercent: headlineHeightPercentArray[screenSizeClass]
    readonly property int headlineHeight:  deviceWidth * headlineHeightPercent/100

    readonly property int headlineButtonSize:   headlineHeight*0.40

    readonly property int headlineDistanceObjToTop: headlineHeight*0.52
    readonly property int headlineDistanceBetweenTwoButtons: headlineHeight*0.23
    readonly property int headlineSepLineHeight: Math.ceil(headlineHeight*0.02)
    readonly property int headlineShadowHeight:  headlineHeight*0.1
    readonly property int headlineDistanceTitleToTop: headlineHeight*0.10

 // BUTTONLINE
    readonly property var buttonlineHeightPercentArray: [15, 12, 8 , 5]
    readonly property int buttonlineHeightPercent: buttonlineHeightPercentArray[screenSizeClass]
    readonly property int buttonlineHeight:  deviceWidth * buttonlineHeightPercent /100

    readonly property int buttonlineButtonHeight: buttonlineHeight*0.80 
    readonly property int buttonlineBorder: Math.ceil(buttonlineHeight*0.02) //screenSizeClass <=1 ? Math.ceil(buttonlineHeight*0.02) : Math.ceil(buttonlineHeight*0.04)
    readonly property int buttonlineShadowHeight:  headlineShadowHeight

 // SCREEN DEVICE OVERVIEW

    readonly property int listviewEntryHeight: standardFieldHeight

    readonly property int listviewPicHeight: listviewEntryHeight-2*listviewSpaceAbove
    readonly property int listviewLedHeight: listviewEntryHeight*0.22
    readonly property int listviewSpaceToLeft: 1.35*listviewPicHeight
    readonly property int listviewSpaceAbove: listviewEntryHeight*0.08
    readonly property int listviewSpaceColumn: listviewEntryHeight*0.04
    readonly property int listviewFontHL: fontSizeNormalCaption
    readonly property int listviewFontDetails: fontSizeNormalText

    readonly property int listviewButtons: listviewEntryHeight*0.2
    readonly property int listviewButtonSpace: listviewButtons

// SCREEN_TAGLIST
    readonly property int tagListEntry: listviewEntryHeight

// SCREEN SYSTEM INFO (Station Info)
    readonly property int distanceDetailedInfo: listviewEntryHeight*0.2


// SCREEN EDIT/CREATE Connection

    readonly property int editConnectionDeviceImageHeight: inputFieldHeight * 0.83

// Trend View
   readonly property int trendviewHeight:standardFieldHeight*2
   readonly property int trendViewWidth:headlineHeight * 3.0
   readonly property int trendviewShadow:headlineHeight* 0.10
   readonly property int trendviewListItemHeight:headlineHeight * 0.69
   readonly property int trendViewListTextOffset:headlineHeight * 0.12
   readonly property int trendViewListIndicatorHeight:headlineHeight * 0.30
   readonly property int trendViewListTextHeight:fontSizeCaption
   readonly property int trendViewListCheckBoxHeight:selectionButtonHeight
   readonly property int trendViewListSpaceColumn:headlineHeight * 0.12
   readonly property int trendViewLineWidth:2.0

// IO Status table
    readonly property int ioTableTimecolumnWidth: standardFieldHeight*1.5
    readonly property int ioTableColumnWidth: standardFieldHeight/2
    readonly property int ioTableFontSize: fontSizeNormalText
    readonly property real ioTableFieldHeight : (headlineHeight*1.39)/2

// Full Screen Trend View
   readonly property int fullScreenTrendviewHeadlineHeight:headlineHeight * 0.58
   readonly property int fullScreenTrendviewHeight:headlineHeight * 7.0
   readonly property int dataGridIndicatorHeight:headlineHeight * 0.16
   readonly property int dataGridHeightPortarit:headlineHeight * 1.32
   readonly property int dataGridHeightLandScape:headlineHeight * 0.74
   readonly property int dataGridIndicatorOffset:headlineHeight * 0.16
   readonly property int dataGridTextHeight:headlineHeight * 0.16

}
