import QtQuick 2.0
import ConnectionControllerComponent 1.0


import LoggerComponents 1.0
import "." as S7App_Components

Component {

    Rectangle {
        id: listItem
        width: mainObject.width
        height: def.listviewEntryHeight
        color: "transparent"

        property int screenState:screen_overview.screenMode
        property int isConnectionListVisible:screen_overview.opacity
        property bool clearSelectedItem:screen_overview.clearSelectedItem
        property bool showFailedMsg:true      
        property int plctype:-1

        MouseArea{
            id: mouseArea
            enabled:true
            height: def.listviewEntryHeight;
            width:parent.width - def.standardButtonWidth
            anchors {
                left: parent.left
                verticalCenter: parent.verticalCenter
            }
            onPressed:{
                if(screenState === 0)
                listItem.color = def.pressedStateButtonColor
            }

            onContainsMouseChanged: {
               if (!containsMouse)
                    listItem.color = "transparent"
            }

            onReleased: {
                listItem.color = "transparent"
            }

            onClicked: {
                Logger.consoleLog(def.logInfo,"Current Index & Index = " + mainObject.currentIndex + index)

                    if(!screen_overview.listElementSelected)
                    {

                        screen_overview.connListObj.setConnListCurrentIndex(index)
                        mainObject.currentIndex = index
                        mainObject.currentConnection = StationName
                        mainObject.currentIP = IPAddress

                        if(screenState === 0) // No Edit mode: show system info screen
                        {
                            screen_overview.listElementSelected = true
                            busyIndicator.visible = true
                            Logger.consoleLog(def.logInfo,"Clicked Save in PLC authentication ")
                            connectionModel.setBasicInfo(index,"UnKnown")
                            conController.connectedTohost.connect(onConnectedTohost)
                            conController.showSSLCertificate.connect(onShowSSLCertificate)
                            conController.connectToHost(IPAddress,index)
                            if(def.isS7App)
                            {
                                screen_systemInfo.runStateButtonInSysInfo.visible = true
                                conController.gotDeviceState.connect(onGotDeviceState)
                            }
                            screen_systemInfo.rectInSystemInfoHeight = def.standardFieldHeight
                        }
                    }
                }
            }

        MouseArea {
           id: delegateRoot
            enabled:true
            property int visualIndex: index

            width: movebutton.width*1.4
            height: movebutton.height*1.4

            anchors.centerIn: movebutton

            drag.target: icon
            clip: false

            Rectangle {
               id: icon
               height:def.listviewButtons
               width: height
               radius: 5
               color: "transparent"
               visible:false
               property bool isDragActive:delegateRoot.drag.active

               anchors {
                   horizontalCenter: parent.horizontalCenter
                   verticalCenter: parent.verticalCenter
               }

               Drag.active: delegateRoot.drag.active
               Drag.source: delegateRoot

               onIsDragActiveChanged: {
                   screen_overview.connListObj.listViewObj.currentIndex = index
                   if(isDragActive)
                   {
                       scrollTimer.start()
                       Logger.consoleLog(def.logCritical,"Timer active ")
                       screen_overview.clearSelectedItem = true
                   }
                   else
                   {
                       scrollTimer.stop()
                       Logger.consoleLog(def.logCritical,"Timer stopped")
                       screen_overview.clearSelectedItem = false
                   }
               }


               states: [
                   State {
                       when: icon.Drag.active
                       ParentChange {
                           target: icon
                           parent: screen_overview
                       }

                       AnchorChanges {
                           target: icon;
                           anchors.horizontalCenter: undefined;
                           anchors.verticalCenter: undefined
                       }
                   }
               ]
            }

            DropArea {
                id:dropArea
                width: mainObject.width - 2*def.borderDistanceLeft
                height: parent.height*2
                anchors.right: parent.right
                anchors.verticalCenter: parent.verticalCenter

                onEntered: {
                    Logger.consoleLog(def.logInfo,"on Entered")
                    if(screenState === 1) // Move Mode
                    {
                        Logger.consoleLog(def.logInfo,"Grag index from toTo = " + delegateRoot.visualIndex + "  " +drag.source.visualIndex )
                        if(drag.source.visualIndex !== delegateRoot.visualIndex)
                        {
                            if(delegateRoot.visualIndex > drag.source.visualIndex )
                                connectionModel.moveElement(drag.source.visualIndex, delegateRoot.visualIndex)
                            else
                                connectionModel.moveElement(delegateRoot.visualIndex, drag.source.visualIndex)
                        }

                    }
                }

            }
        }

        function onShowSSLCertificate(sslInfo , greenMsg )
        {            
            showFailedMsg = false;                       
            sslCertDialog.text_scrollable = (sslInfo==="") ? stringStore.sslNoCertAvailable :
                                             ("%1\n\n%2").arg(greenMsg  ? stringStore.sslCertGreenMsg :stringStore.sslCertYellowMsg).arg(sslInfo);
            sslCertDialog.button_Accept = true;
            sslCertDialog.button_Decline = true;
            sslCertDialog.button_OK = false;
            sslCertDialog.open();
        }

        function  onConnectedTohost(bConnected, strErrorMsg)
        {
            mainObject.errorMsg = strErrorMsg
            def.isPLCConnectd = bConnected
            if(bConnected)
            {
                if(def.isS7App)
                {
                    conController.basicInfoAuthentication.connect(onBasicInfoAuth)
                    Logger.consoleLog(def.logInfo,"**connected to authenticationError**")
                }
                else
                {
                    busyIndicator.visible = false
                    screen_systemInfo.diag_visible = def.isOBA7() ? true : false
                    stackview.pushPage(screen_systemInfo)
                    Logger.consoleLog(def.logInfo,"---Connection Succeded---");
                    screen_diag.ack_visible = def.isOBA8() ? false : true
                }

            }
            else
            {
                busyIndicator.visible = false
                if(showFailedMsg)
                {
                    screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
                }

                screen_overview.listElementSelected = false
                tagModel.removeAll()               
            }

            showFailedMsg = true;
            conController.showSSLCertificate.disconnect(onShowSSLCertificate);
            conController.connectedTohost.disconnect(onConnectedTohost)            
            Logger.consoleLog(def.logInfo,"--------------" + screen_systemInfo.systemInfo)
        }
        function onBasicInfoAuth(noPermission)
        {
            conController.basicInfoAuthentication.disconnect(onBasicInfoAuth)
            if(!noPermission)
            {
            Logger.consoleLog(def.logCritical,"***No Host Auth***")
                screen_systemInfo.runStateButtonInSysInfo.visible = false
            screen_systemInfo.timerInSysInfo.running = false
                screen_systemInfo.rectInSystemInfoHeight = screen_systemInfo.rectInSystemInfoHeight -(screen_systemInfo.runStateButtonInSysInfo.height+screen_systemInfo.plcInfoHeaderHeight)
            }
            busyIndicator.visible = false
            stackview.pushPage(screen_systemInfo)

        }
        function onGotDeviceState(nDeviceState)
        {
            conController.gotDeviceState.disconnect(onGotDeviceState)
            screen_systemInfo.runStateButtonInSysInfo.state = nDeviceState === 0 ? true : false
        }

        Image {
            id: deviceImage
            height: def.listviewPicHeight
            width: height*1.2
            fillMode: Image.PreserveAspectFit
            horizontalAlignment: Image.AlignLeft
            visible: (screenState === 0) ? true : false
            source : (def.isS7App)?"pictures/S7-Default.svg":"pictures/LOGO_0BA7.svg"
            anchors {left: parent.left; leftMargin: def.borderDistanceLeft; verticalCenter: parent.verticalCenter}
        }
    onPlctypeChanged: {
        if(def.isS7App)
        {
            if(PlcType == 0)
                deviceImage.source = "pictures/S7-1200.svg"
            else if(PlcType == 1)
                deviceImage.source = "pictures/S7-1500.svg"
            else
                deviceImage.source = "pictures/S7-Default.svg"
        }
        else
        {
            if(PlcType == 0)
                deviceImage.source = "pictures/LOGO_0BA7.svg"
            else
                deviceImage.source = "pictures/LOGO_0BA8.svg"
        }
    }

        Checkmark{
            id: checkbox
            visible: (screenState === 1) ? true : false
            anchors {left: parent.left; leftMargin: def.borderDistanceLeft; verticalCenter: parent.verticalCenter}
            onCheckmarkClicked: {
                isSelected = !isSelected;
                connectionModel.selectElement(index,isSelected);
            }
            Component.onCompleted: {
                isSelected = connectionModel.isElementSelected(index)
            }
        }

        Column
        {
            id: connectionColumn
            spacing: def.listviewSpaceColumn
            anchors.left: (screenState > 0) ? checkbox.right : deviceImage.right
            anchors.leftMargin: def.borderDistanceLeft
            anchors.bottom: deviceImage.bottom

            DefaultText{
                id: textStationName
                text: StationName
                font.pixelSize:  def.listviewFontHL
                font.bold: true
                elide:Text.ElideRight
                width:mainObject.width - checkbox.width -  movebutton.width- def.borderDistanceRight*2 - connectionSettings_background.width -  def.borderDistanceLeft*2
                maximumLineCount: 20
            }

            DefaultText{
                id: textResult
                visible: def.isS7App
                text:"MAC: "+ MacID
                font.pixelSize:  def.listviewFontDetails
            }

            DefaultText{
                id: textIPAddress
                text: IPAddress
                font.pixelSize:  def.listviewFontDetails
                elide:Text.ElideRight
                width: mainObject.width - checkbox.width -  movebutton.width- def.borderDistanceRight*2 - connectionSettings_background.width -  def.borderDistanceLeft*2
            }

        }

        Image {
            id: movebutton
            width: def.headlineButtonSize
            height: width
            visible: icon.visible
            anchors{verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: def.borderDistanceRight}
            source: "pictures/ICO_App_ChangeListentry.svg"
        }

        Rectangle {
            id:connectionSettings_background
            width:connectionSettings.width*1.4
            height: width
            color: "transparent"
            visible: false
            anchors{verticalCenter: parent.verticalCenter; right: movebutton.left; rightMargin: def.borderDistanceRight}

            MouseArea{
                anchors.fill: parent

                onClicked: {
		
                    screen_overview.connListObj.setConnListCurrentIndex(index)
                    mainObject.currentIndex = index
                    mainObject.currentConnection = StationName
                    mainObject.currentIP = IPAddress
                    Logger.consoleLog(def.logInfo,"Current Index & Index settings= " + mainObject.currentIndex + index)

                    screen_editConnection.stationNameLabel = StationName
                    screen_editConnection.ipAddressLabel = IPAddress
                    if(!(def.isLogoApp && PlcType ==0))
                    {
                        screen_editConnection.userNameLabel = Username
                        screen_editConnection.passwordLabel = Password
                    }
                    screen_editConnection.plcType = PlcType
                    Logger.consoleLog(def.logInfo,"Station Name"+screen_editConnection.stationNameLabel)
                    Logger.consoleLog(def.logInfo,"Ip Address"+screen_editConnection.ipAddressLabel)
                    stackview.pushPage(screen_editConnection)
                }
		
                onPressed: {
		
                    connectionSettings_background.color = def.pressedStateButtonColor
                    connectionSettings.source = "pictures/ICO_APP_Settings_pressed.svg"
                }
		
                onReleased: {

                    connectionSettings_background.color = "transparent"
                    connectionSettings.source = "pictures/ICO_APP_Settings.svg"
                }
		
                onContainsMouseChanged: {
		
                    if (!containsMouse)
                    {
                        connectionSettings_background.color = "transparent"
                        connectionSettings.source = "pictures/ICO_APP_Settings.svg"
                    }
                }

            }
            Image {
                id: connectionSettings
                width:def.headlineButtonSize
                height: width
                visible: connectionSettings_background.visible
                anchors.centerIn: parent
                fillMode: Image.PreserveAspectFit
                source:"pictures/ICO_APP_Settings.svg"
		 }
        }

        onScreenStateChanged:
         {
             if (screenState == 0)
             {
                 icon.visible = false
                 dropArea.visible = false
                 delegateRoot.drag.target = undefined
                 connectionSettings_background.visible = false

             }
             else if (screenState == 1)
             {
                 icon.visible = true
                 dropArea.visible = true
                 delegateRoot.drag.target = icon
                 connectionSettings_background.visible = true
             }

         }

        ListSeperator{
            anchors.bottom: parent.bottom
        }

        onClearSelectedItemChanged: {
            if(checkbox.isSelected)
            {
                checkbox.isSelected = false
                connectionModel.clearElementSelection();
            }

        }

        onIsConnectionListVisibleChanged: {
            if(isConnectionListVisible)
            {
                screen_overview.listElementSelected = false
                plctype = PlcType
            }
        }

        Timer{
            id:scrollTimer
            interval: 125; running: false; repeat:true
            property int runs:0
            onTriggered: {
                var currPos = delegateRoot.mapToItem(mainObject,mainObject.x,mainObject.y)
                Logger.consoleLog(def.logCritical,"Current Pos " + currPos.y)

                if(currPos.y < screen_overview.scrollUpLimit && currPos.y > def.headlineHeight)
                {
                    screen_overview.connListObj.scrollUpListView()
                }
                else if(currPos.y > screen_overview.scrollDownLimit)
                {
                    screen_overview.connListObj.scrollDownListView()
                }
            }
            }
        }
}




