import QtQuick 2.0
import ConnectionControllerComponent 1.0
import LoggerComponents 1.0
import QtQuick.Window 2.1

Rectangle {
    id:screenDateTime
    width: parent.width
    height: parent.height
    color: "transparent"
    property var deviceState : conController.enErrorState
    property string date;
    property string current_dt;
    property bool readOnLoad : false;
    property bool readOnReadClockBtnClick : false;
    property bool setClockOnSetClockBtnClick: false;
    property var dateTime;
    property var currMobileDate;
    property var currMobileTime;
    property bool timeCtrlVisible : false
    property bool dateCtrlVisible : false
    property bool backBtnClicked : false
    property bool potraitMode:def.isProtraitMode
    property var monthDays: [[31],[28],[31],[30],[31],[30],[31],[31],[30],[31],[30],[31]]
    property int defColHeight : currentDateText.height + currentTimeText.height +
                                 readTime.height + readMobiletime.height + setPLCtime.height
    property string plcCurrentDateTimeVal: ""
    property string currentDispVal:""
    Headline {
        id: headline
        width: parent.width
        z:50

        headline_text: stringStore.setClock
        backbutton_visible: true
        ipvisible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from DateTime Settings clicked!")
            backButtonOperation()
        }
    }
    Flickable{
        id:flickDateTimeCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height
        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds

        Rectangle{
            id: col
            height: flickDateTimeCtl.height
            width: parent.width
            color: "transparent"
            InputField{
                id: currentDateText
                label: stringStore.date
                input: ""
                readOnly : true
                hasfocus: true
                MouseArea{
                    anchors.fill: parent
                    onClicked: {
                        setCurrentDateSelection()
                        showDateCtrl()
                    }
                }
                onInputFieldEnabled:
                {
                    setCurrentDateSelection()
                }
            }

            InputField{
                id: currentTimeText
                label: stringStore.time
                readOnly : true
                hasfocus: true
                anchors.top : currentDateText.bottom
                anchors.topMargin: def.borderDistanceTop
                MouseArea{
                    anchors.fill: parent
                    onClicked: {
                        setCurrentTimeSelection()
                        showTimeCtrl()
                    }
                }
                onInputFieldEnabled: {
                    setCurrentTimeSelection()
                }
            }

            Button{
                id:readTime
                isTextbutton: true
                width: def.isProtraitMode ? parent.width/2.2 : def.standardButtonWidthExtendedPlus
                button_Text : stringStore.usePLCClock
                anchors{top:currentTimeText.bottom; topMargin:def.borderDistanceTop; left:parent.left; leftMargin: def.borderDistanceLeft}

                onButtonClicked: {

                    readOnReadClockBtnClick = false
                    conController.gotDeviceState.connect(gotDeviceStateOnReadClockBtnClick)
                    conController.getDeviceState()
                }
            }

            Button{
                id:readMobiletime
                isTextbutton: true
                width: def.isProtraitMode ? parent.width/2.2 : def.standardButtonWidthExtendedPlus

                button_Text: stringStore.useMobileClock
                anchors{top:currentTimeText.bottom; topMargin:def.borderDistanceTop; right:parent.right; rightMargin: def.borderDistanceRight}

                onButtonClicked: {
                    //show Time Control
                    var datetime = new Date()
                    currentDateText.input = Qt.formatDateTime(datetime, "dd.MM.yyyy")
                    currentTimeText.input = datetime.toLocaleTimeString(Qt.locale("en_IN"),"hh:mm AP")

                }
            }

            Button{
                id:setPLCtime
                isTextbutton: true
                width: def.standardButtonWidthExtendedPlus

                button_Text: stringStore.setClock
                anchors{bottom:parent.bottom; bottomMargin:def.borderDistanceBottom; horizontalCenter: parent.horizontalCenter}

                onButtonClicked: {
                    busyIndicator.visible = true
                    if(currentDateText.input == "" || currentTimeText.input == "")
                    {
                        busyIndicator.visible = false
                        datetimeMsg.open()
                        return;
                    }

                    setClockOnSetClockBtnClick = false
                    conController.gotDeviceState.connect(gotDeviceStateOnSetClockBtnClick)
                    conController.getDeviceState()
                }
            }
        }
    }

    TimeControl{
        id:timeCtrl
        onCancelButtonClicked: {
            Logger.consoleLog(def.logInfo,"DateTime settings : onCancelButtonClicked" )
            hideTimeCtrl()
        }
        onSelectButtonClicked: {
            currentTimeText.input = spinner_Hour.model.get(spinner_Hour.currentIndex).val + ":" +
                    spinner_Minutes.model.get(spinner_Minutes.currentIndex).val + " " +
                    spinner_AMPM.model.get(spinner_AMPM.currentIndex).val
            hideTimeCtrl()
        }
    }

    DateControl{
        id:dateCtrl
        onCancelButtonClicked: {
            hideDateCtrl()
        }
        onSelectButtonClicked: {

            if(checkDateRange(spinner_Day.model.get(spinner_Day.currentIndex).val,getMonth(),
                              dtyearModel.getYear(spinner_Year.currentIndex)))
            {
                currentDateText.input = spinner_Day.model.get(spinner_Day.currentIndex).val + "." +
                        getMonth() + "." +
                        dtyearModel.getYear(spinner_Year.currentIndex)
                hideDateCtrl()
            }
            else
            {
                datetimeMsg.title = stringStore.setClock
                datetimeMsg.text = stringStore.dateErrorMsg
                datetimeMsg.open()
            }
        }
    }

    ModalDialog {
        id: changemodeMsg
        title: stringStore.logoChangeModeMsgTitle
        button_Yes: true
        button_No: true
        button_OK: false
        onYesButtonClicked: {
            if(deviceState == 0) //Run State
            {
                conController.setPLCModeCompleted.connect(onSetPLCModeCompleted)
                conController.setPLCMode(1) //change to stop
            }
            else if(deviceState == 1) //Stop State
            {
                conController.setPLCModeCompleted.connect(onNavigateBack)
                conController.setPLCMode(0) //change to run
            }
        }
        onNoButtonClicked: {
            if(backBtnClicked)
            {
                backBtnClicked = false;
                stackview.popPage();
            }
        }
    }

    ModalDialog {
        id: datetimeMsg
        button_OK: true
        button_Yes: false
        button_No: false
        title : stringStore.datetimeMsgTitle
        text : stringStore.datetimeErrorMsg
    }

    Component.onCompleted: {
        readOnLoad = false
        conController.gotDeviceState.connect(gotDeviceStateOnPageLoaded)
        conController.getDeviceState()
    }
    
    ModalDialog {
        id: goBackWithoutChanges
        title: stringStore.setClock
        text: stringStore.goBackAlertMessage
        button_Yes: true
        button_No: true
        button_OK: false
        onYesButtonClicked:
        {
            getPlcMode()
        }
    }

    ParallelAnimation
    {
        id: animateTimeCtrlShow

        PropertyAnimation {
            id:animateTimeCtrlY
            target: timeCtrl
            properties: "y"
            to: parent.height - timeCtrl.spinnerHeight
            duration: 500
        }

        PropertyAnimation {
            id:animateTimeCtrlOpacity
            target: timeCtrl
            properties: "opacity"
            to: 1.0
            duration: 500
        }
    }

    ParallelAnimation
    {
        id:animateTimeCtrlHide

        PropertyAnimation {
            target: timeCtrl
            properties: "y"
            to: parent.height
            duration: 500
        }
        PropertyAnimation {
            target: timeCtrl
            properties: "opacity"
            to: 0
            duration: 500
        }
    }

    ParallelAnimation
    {
        id: animateDateCtrlShow

        PropertyAnimation {
            id:animateDateCtrlY
            target: dateCtrl
            properties: "y"
            to: parent.height - dateCtrl.spinnerHeight
            duration: 500
        }

        PropertyAnimation {
            id:animateDateCtrlOpacity
            target: dateCtrl
            properties: "opacity"
            to: 1.0
            duration: 500
        }
    }

    ParallelAnimation
    {
        id:animateDateCtrlHide
        PropertyAnimation {
            target: dateCtrl
            properties: "y"
            to: parent.height
            duration: 500
        }
        PropertyAnimation {
            target: dateCtrl
            properties: "opacity"
            to: 0
            duration: 500
        }
    }

    function getMonth()
    {
        switch(dateCtrl.spinner_Month.model.get(dateCtrl.spinner_Month.currentIndex).val)
        {
        case "January": return "01";
        case "February": return "02";
        case "March": return "03";
        case "April": return "04";
        case "May": return "05";
        case "June": return "06";
        case "July": return "07";
        case "August": return "08";
        case "September": return "09";
        case "October": return "10";
        case "November": return "11";
        case "December": return "12";
        }
    }

    function setCurrentDateSelection()
    {
        if(currentDateText.input==="")
        {
            dateCtrl.spinner_Day.currentIndex = 0
            dateCtrl.spinner_Month.currentIndex = 0
            dateCtrl.spinner_Year.currentIndex = 0
            return;
        }

        var dateText = currentDateText.input.split(".")
        dateCtrl.spinner_Day.currentIndex=parseInt(dateText[0])-1
        dateCtrl.spinner_Month.currentIndex=parseInt(dateText[1])-1
        dateCtrl.spinner_Year.currentIndex=parseInt(dateText[2])-dateCtrl.spinner_Year.model.getYear(0)
    }

    function setCurrentTimeSelection()
    {
        if(currentTimeText.input=="")
        {
            timeCtrl.spinner_Hour.currentIndex = 0
            timeCtrl.spinner_Minutes.currentIndex = 0
            timeCtrl.spinner_AMPM.currentIndex = 0
            return;
        }

        var hour_min_ampm = currentTimeText.input.split(":")

        timeCtrl.spinner_Hour.currentIndex=parseInt(hour_min_ampm[0])-1
        var min_ampm = hour_min_ampm[1].split(" ")

        timeCtrl.spinner_Minutes.currentIndex=parseInt(min_ampm[0])
        if(min_ampm[1]==="AM")
            timeCtrl.spinner_AMPM.currentIndex = 0
        else
            timeCtrl.spinner_AMPM.currentIndex = 1
    }

    function showTimeCtrl()
    {
        hideDateCtrl()
        if(!timeCtrlVisible)
        {
            timeCtrlVisible = true
            animateTimeCtrlShow.start()
        }
    }

    function hideTimeCtrl()
    {
        if(timeCtrlVisible)
        {
            timeCtrlVisible = false
            animateTimeCtrlHide.start()
        }
    }

    function showDateCtrl()
    {
        hideTimeCtrl()
        if(!dateCtrlVisible)
        {
            dateCtrlVisible = true
            animateDateCtrlShow.start()
        }
    }

    function hideDateCtrl()
    {
        if(dateCtrlVisible)
        {
            dateCtrlVisible = false
            animateDateCtrlHide.start()
        }
    }

    onPotraitModeChanged:
    {
        if(dateCtrlVisible)
            hideDateCtrl()
        if(timeCtrlVisible)
            hideTimeCtrl()
    }
	//RQ: 1142840 - 'Set clock on PLC' button is moved above after orientation change
    onHeightChanged: {

        flickDateTimeCtl.height = parent.height - headline.height
        defColHeight = currentDateText.height + currentTimeText.height +
                       readTime.height + readMobiletime.height + setPLCtime.height

        if(flickDateTimeCtl.height < defColHeight )
            col.height = defColHeight
        else
            col.height = flickDateTimeCtl.height

        flickDateTimeCtl.contentHeight = col.height

        setPLCtime.anchors.bottom = col.bottom
        setPLCtime.anchors.bottomMargin = def.borderDistanceBottom;
        setPLCtime.anchors.horizontalCenter = col.horizontalCenter

    }

    function getPlcMode()
    {
        backBtnClicked = true
        conController.gotDeviceState.connect(gotDeviceStateOnBackButtonOperation)
        conController.getDeviceState()
    }

    function gotDeviceStateOnPageLoaded(nDeviceState)
    {
        conController.gotDeviceState.disconnect(gotDeviceStateOnPageLoaded)

        deviceState = nDeviceState

        if(deviceState == 0 ) //ConnectionController.enRunState
        {
            readOnLoad = true
            changemodeMsg.text = stringStore.runToStopMsg
            changemodeMsg.open()
        }
        else if(deviceState == 1) //ConnectionController.enStopState
        {
            readDateTime()
        }
        else
        {
            //Netwrok access error
        }
    }

    function gotDeviceStateOnReadClockBtnClick(nDeviceState)
    {
        conController.gotDeviceState.disconnect(gotDeviceStateOnReadClockBtnClick)

        deviceState = nDeviceState;

        if(deviceState == 0 ) //ConnectionController.enRunState
        {
            readOnReadClockBtnClick = true
            changemodeMsg.text = stringStore.runToStopMsg
            changemodeMsg.open()
        }
        else
        {
            readDateTime()
        }
    }

    function gotDeviceStateOnSetClockBtnClick(nDeviceState)
    {
        conController.gotDeviceState.disconnect(gotDeviceStateOnSetClockBtnClick)

        deviceState = nDeviceState;

        if(deviceState == 0 ) //ConnectionController.enRunState
        {
            setClockOnSetClockBtnClick = true
            changemodeMsg.text = stringStore.runToStopMsg
            busyIndicator.visible = false
            changemodeMsg.open()
        }
        else
        {
            //Set Time
            current_dt = currentDateText.input
            current_dt = current_dt + " " + currentTimeText.input
            var result = conController.setClock(current_dt)
            if(def.isOBA7())
                onClockStateChange(result)
        }
    }

    function gotDeviceStateOnBackButtonOperation(nDeviceState)
    {
        conController.gotDeviceState.disconnect(gotDeviceStateOnBackButtonOperation)
        deviceState = nDeviceState;

        if(deviceState == 1)
        {
            changemodeMsg.text = stringStore.stopToRunMsg
            changemodeMsg.open()
        }
        else
        {
            backBtnClicked = false
            stackview.popPage();
        }
    }

    function onSetPLCModeCompleted()
    {
        conController.setPLCModeCompleted.disconnect(onSetPLCModeCompleted)

        if(readOnLoad == true || readOnReadClockBtnClick == true)
        {
            readOnLoad = false
            readOnReadClockBtnClick = false

            readDateTime()
        }
        else if(setClockOnSetClockBtnClick == true)
        {
            busyIndicator.visible = true
            //Set Time
            current_dt = currentDateText.input
            current_dt = current_dt + " " + currentTimeText.input
            var result = conController.setClock(current_dt)
            if(def.isOBA7())
                onClockStateChange(result)
        }
    }

    function readDateTime()
    {
        if(def.isOBA8())
        {
            conController.gotClock.connect(onReadClock)
            conController.readClock()
        }
        else if(def.isOBA7())
        {
            date = conController.readClock()
            dateTime = date.split(";")
            currentDateText.input = dateTime[0]
            currentTimeText.input = dateTime[1]
            plcCurrentDateTimeVal = date
        }
    }

    function onNavigateBack()
    {
        conController.setPLCModeCompleted.disconnect(onNavigateBack)
        stackview.popPage()
    }

    function onReadClock(timeValue)
    {
        conController.gotClock.disconnect(onReadClock)

        dateTime = timeValue.split(";")
        currentDateText.input = dateTime[0]
        currentTimeText.input = dateTime[1]
        plcCurrentDateTimeVal = timeValue
    }

    function checkDateRange(day,month,year)
    {
        var monthInt = parseInt(month)
        var dayInt = parseInt(day)
        var yearInt = parseInt(year)
        var result = false

        if(monthInt === 2)
        {

            var isLeapYear = (((yearInt % 4 == 0) && (yearInt % 100 != 0)) || (yearInt % 400 == 0))

            if(isLeapYear)
            {
                if(29 >= dayInt)
                {
                    result = true;
                }
            }
            else
            {
                if(28 >= dayInt)
                {
                    result = true;
                }
            }
        }
        else
        {
            if(monthDays[monthInt - 1] >= dayInt)
            {
                result = true;
            }
        }

        return result
    }

    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible=false
        stackview.replace(screen_overview)
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
    }

    onOpacityChanged:
    {
        if(opacity === 1)
        {
            conController.clockSetSuccesfull.connect(onClockStateChange)
            conController.connectionError.connect(onConnectionError)
        }
        else
        {
            conController.clockSetSuccesfull.disconnect(onClockStateChange)
            conController.connectionError.disconnect(onConnectionError)
        }
    }

    function backButtonOperation()
    {
        if(currentDateText.input !== "" || currentTimeText.input !== "")
            currentDispVal = currentDateText.input+";"+currentTimeText.input;
        if(currentDispVal !== plcCurrentDateTimeVal)
        {
            goBackWithoutChanges.open()
            return;
        }
        getPlcMode()
    }

    function onClockStateChange(success)
    {
        busyIndicator.visible = false
        if(!success)
        {
            datetimeMsg.title = stringStore.setClock
            datetimeMsg.text = stringStore.dateSetStateErrorMsg
            datetimeMsg.open()
        }
        else
        {
            plcCurrentDateTimeVal = currentDateText.input+";"+currentTimeText.input
            datetimeMsg.title = stringStore.setClock
            datetimeMsg.text = stringStore.dateSetStateSuccessMsg
            datetimeMsg.open()
        }
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in DateTime settings" )
        event.accepted = true;
        if (event.key === Qt.Key_Back && changemodeMsg.visible === false && goBackWithoutChanges.visible === false)
        {
            backButtonOperation()
        }
    }
}
