
import QtQuick 2.2
import QtCharts 2.0
import DataStoreComponents 1.0
import TrendDataSourceComponents 1.0
import LoggerComponents 1.0
import "." as S7App_Components

ChartView {
    id:trendView
    width:parent.width
    height:parent.height
    //title: "Live Tag Values"
    antialiasing: true
    backgroundColor: "transparent"
    legend.visible: false
    margins.top:0
    margins.left:0
    margins.bottom:0
    margins.right:0

    property var refreshFreq:0
    property bool isTrendInitialized:false
    property bool isTrendingStarted:false
    property int bufferSize:150000 //time in miliseconds
    property int yAxisMinima:0
    property int yAxisMaxima:100
    property bool isTrendRunning:true
    property int valueCount:0
    property var dataModel:0
    property bool stopTrendUpdate:false
    property bool updateTrend:false
    property var currTime

    ValueAxis {
        id: valueAxisYRight
        min: yAxisMinima
        max: yAxisMaxima
        color:def.trendViewAxisColor
        labelFormat:"%.0f"
        labelsColor:def.trendViewAxisColor
        labelsFont.family: def.fontDefault
        Component.onCompleted: {
            TrendDataSource.setAxisGridLineColor(valueAxisYRight,def.trendViewAxisColor)
        }
    }

    LineSeries {
        id: defaultSeries
        name:"default"
        axisX: dtAxisX
        axisYRight: valueAxisYRight
        color:Qt.rgba(0,0,0,0)
    }

    DateTimeAxis {
        id: dtAxisX
        tickCount: 5
        format:"hh:mm:ss"
        color:def.trendViewAxisColor
        labelsColor:def.trendViewAxisColor
        labelsFont.family: def.fontDefault
        Component.onCompleted: {
            TrendDataSource.setAxisGridLineColor(dtAxisX,def.trendViewAxisColor)
        }
    }

    MouseArea{
        id:trendMouseArea
        anchors.fill:parent
        property int currX: 0
        property int startX:0
        property int relX: 0
        property int prevX:0
        property bool startTrend:false
        onClicked:{

        }
        onPressed:{
            if(isTrendRunning)
            {
                Logger.consoleLog(def.logInfo,"start dragging" + mouseX )
                startX = mouseX
                isTrendRunning = false

            }
            prevX = mouseX
            currX = mouseX

            Logger.consoleLog(def.logInfo,"startX " + startX + "currX " + currX + "relX "+ relX  + "prevX " + prevX + " in Pressed")

        }
        onPositionChanged: {
            stopTrendUpdate = true
            if(mouseX > currX)
            {
                Logger.consoleLog(def.logInfo,"Trend dragging Left" + mouseX )
                currX = mouseX
                if(currX !== prevX)
                {
                    relX += currX - prevX
                    Logger.consoleLog(def.logInfo,"currX" + currX + "relX "+ relX + "prevX " + prevX)
                    startTrend = false
                }
                if(relX < 3200)
                {
                    var pointX = Math.ceil(mouseX - prevX)
                    trendView.scrollLeft(pointX)
                    Logger.consoleLog(def.logInfo,"currX" + currX + "relX "+ relX + "prevX " + prevX)
                }
                else
                {
                    relX = 3200
                }
                prevX = currX
                startTrend = false

            }
            else if (mouseX < currX)
            {
                Logger.consoleLog(def.logInfo,"Trend dragging Right" + mouseX )
                currX = mouseX
                if(currX !== prevX)
                {
                    relX -= prevX - currX
                    Logger.consoleLog(def.logInfo,"currX" + currX + "relX "+ relX  + "prevX " + prevX)
                }
                if(relX > 0)
                {
                    var pointX = Math.floor(prevX - mouseX)
                    trendView.scrollRight(pointX)
                    Logger.consoleLog(def.logInfo,"currX" + currX + "relX "+ relX + "prevX " + prevX)
                }
                else if(relX <= 0)
                {
                    startTrend = true
                    relX = 0
                    Logger.consoleLog(def.logInfo,"currX" + currX + "relX "+ relX + "prevX " + prevX)
                }
                prevX = currX
            }

        }
        onReleased:{
            if(startTrend)
            {
                if(!isTrendRunning)
                {
                    Logger.consoleLog(def.logInfo,"Update trend view" + currX  + "  " + mouseX)
                    stopTrendUpdate = false
                    isTrendRunning = true
                }
            }
        }


    }

    function createTrend(index,color)
    {
        TrendDataSource.setFullScreenTrendColor(color)
        createTagSeries(index,color)
    }

    function createTagSeries(index,color)
    {
        if(isTrendInitialized)
        {
            isTrendInitialized = false
            var size = bufferSize/(refreshFreq * 1000)
            TrendDataSource.setBufferSize(size)
            trendView.removeSeries(trendView.series("default"))
        }

        var trend = trendView.createSeries(ChartView.SeriesTypeLine, "" + index,dtAxisX,valueAxisYRight)
        trend.color = color
        trend.width = def.trendViewLineWidth
    }

    function updateTagSeries(index,value)
    {
        var trendIndex = TrendDataSource.getActualTrendIndex(index)
        var trend = trendView.series(trendIndex)
        Logger.consoleLog(def.logCritical,"criticial index ============== " + trendIndex + " color  " + trend.color + " value " + value)
        if(trend !== null)
        {
            updateXAxis()

            updateYAxisRange(value)

            if(valueCount > (1200+ (1200 * 0.1)))
            {
                var point = trend.at(0)
                trend.remove(point.x,point.y)
                trend.append(dtAxisX.max,value)
            }
            else
            {
                trend.append(dtAxisX.max,value)
            }
        }
    }

    function removeTagSeries(index)
    {
        var trendIndex = TrendDataSource.getActualTrendIndex(index)
        if(trendIndex >= 0)
        {
            Logger.consoleLog(def.logCritical,"Current Index for remove   ============== " + index + " trend index  " + trendIndex)
            trendView.removeSeries(trendView.series(trendIndex))
            TrendDataSource.removeFullScreenTrendColor(trendIndex);
            valueAxisYRight.max = yAxisMaxima
            valueAxisYRight.min = yAxisMinima
            Logger.consoleLog(def.logCritical,"Trend cout after delete   ============== " + trendView.count )
        }
    }

    function removeAllTagSeries()
    {
        valueCount = 0
        Logger.consoleLog(def.logInfo,"Latest trend count before delete ========= " + trendView.count)
        trendView.removeAllSeries()
        TrendDataSource.removeAllValue()
    }

    function clearTrendView()
    {
        valueCount = 0
        trendMouseArea.currX = 0
        trendMouseArea.prevX = 0
        trendMouseArea.startX = 0
        trendMouseArea.relX = 0
        trendView.removeAllSeries()
        stopTrendUpdate = false
        trendMouseArea.startTrend = false
        initializeTrendView()
    }

    function updateXAxisRange()
    {
        dtAxisX.max = new Date()
        var startTime = dtAxisX.max

        startTime.setTime(startTime.getTime() - refreshFreq * bufferSize)
        dtAxisX.min = startTime
    }

    function updateXAxis()
    {
        dtAxisX.max = currTime
        var startTime = dtAxisX.max

        startTime.setTime(startTime.getTime() - refreshFreq * bufferSize)
        dtAxisX.min = startTime
    }

    function updateYAxisRange(value)
    {
        if(value !== undefined)
        {
            if(valueAxisYRight.max <= value)
            {
                Logger.consoleLog(def.logInfo,"In High tag Seies ")
                valueAxisYRight.max = value
            }
            else
            {
                if(!isTrendingStarted)
                {
                    valueAxisYRight.min = value
                    isTrendingStarted = true
                }
                else if(valueAxisYRight.min > value)
                {
                    Logger.consoleLog(def.logInfo,"In Low tag Seies " + valueAxisYRight.min  + "  " + value)
                    valueAxisYRight.min = value
                }
            }
        }
    }

    function getTrendColor(index)
    {
        if(index >= 0)
        {
            Logger.consoleLog(def.logInfo,"Current index " + index)
            return TrendDataSource.getFullScreenTrendColor(index)
        }
    }

    function onFrequencyChanged(freq)
    {
        refreshFreq = freq
        updateXAxisRange()
    }

    function onDataUpdate()
    {
        if(screen_trendView_FullScreen.opacity == 1)
        {
            var result;
            result = dataModel.getUpdatedData()
            currTime = new Date()
            TrendDataSource.setUpdatedTrendValue(result,currTime.getTime(),true)

            if(!stopTrendUpdate)
            {
                var trendIndex;
                if(updateTrend)
                {
                    busyIndicator.visible = true
                    valueCount = TrendDataSource.getTrendValueCount();
                    for(var j = 0 ; j < dataModel.getSelectedDataElementCount() ; j++)
                    {
                        trendIndex = TrendDataSource.getTrendIndex(j)
                        var actTrendIndex = TrendDataSource.getActualTrendIndex(trendIndex)
                        var trend = trendView.series(actTrendIndex)

                        Logger.consoleLog(def.logInfo,"Updating the trend in one go ============== ")
                        TrendDataSource.updateTrend(trend,j)
                    }
                    updateTrend = false
                }
                else
                {
                    valueCount++
                    for(var j = 0 ; j < dataModel.getSelectedDataElementCount() ; j++)
                    {
                        var trendvalue;
                        trendvalue = TrendDataSource.getUpdatedTrendValue(j)
                        trendIndex = TrendDataSource.getTrendIndex(j)

                        if(trendvalue !== -1)
                        {
                            Logger.consoleLog(def.logInfo,"Updating the trend value ============== " + trendvalue + " at index  ============== " + trendIndex )
                            updateTagSeries(trendIndex,trendvalue)
                            busyIndicator.visible = false
                        }
                    }
                }
            }
        }

    }

    function onAppStatusChanged()
    {
        if(appController.appStaus !== def.appActive)
        {
            stopTrendUpdate = false
            dataModel.removeAllArchiveData()
            clearTrendView()
            screen_trendView.trendViewObj.reStartTrendView()
            if(screen_trendView_FullScreen.dataController!=0)
            screen_trendView_FullScreen.dataController.stopRefreshing()
            Logger.consoleLog(def.logInfo,"Current app status ============= " +  appController.appStaus)
        }
    }

    Component.onCompleted: {
        refresh_freq_settings.frequencyChanged.connect(onFrequencyChanged)
    }

    Component.onDestruction: {
        trendView.removeAllSeries()
        refresh_freq_settings.frequencyChanged.disconnect(onFrequencyChanged)
    }

    function initializeTrendView()
    {
        isTrendInitialized = true
        refreshFreq = DataStore.getSettings("rf_seconds")
        updateXAxisRange()
        setAxisRange(yAxisMinima,yAxisMaxima)
    }

    function setDataModel(model)
    {
        dataModel = model
    }

    function setAxisRange(minima,maxima)
    {
        valueAxisYRight.min = minima
        valueAxisYRight.max = maxima
        yAxisMinima = minima
        yAxisMaxima = maxima

        Logger.consoleLog(def.logInfo," Full Screen Updating the trend axis range  ============== " + minima +  maxima)
    }

    function setTickCount(count)
    {
        valueAxisYRight.tickCount = count
    }

    onStopTrendUpdateChanged: {
        if(stopTrendUpdate == false)
            updateTrend = true
        else
            updateTrend = false

    }

}
