import QtQuick 2.0


import "." as S7App_Components


Rectangle {


    property alias headline_text: hl_text.text
    property alias backbutton_visible: back_button.visible
    property alias editbutton_visible: edit_button.visible
    property alias addbutton_visible: add_button.visible
    property alias savebutton_visible: save_button.visible
    property alias duplicate_visible: duplicate_button.visible
    property alias delete_visible: delete_button.visible
    property alias headline_text_fullScreen_trend: hl_text_trend_full_screen.text
    property alias backbutton_fullScreen_trend_visible: back_button_trend_full_screen.visible
    property alias share_visible: share_button.visible

    property alias ipvisible: textIP.visible
    property alias appNameVisible: textAppName.visible


    signal buttonEditClicked()
    signal buttonAddClicked()
    signal buttonBackClicked()
    signal buttonSaveClicked()
    signal buttonDuplicateClicked()
    signal buttonDeleteClicked()
    signal buttonBackFullScreenTrendViewClicked()
    signal buttonShareClicked()

    width: parent.width
    height: def.headlineHeight
    color: def.colHeadlineBackground
    z: 20

    Rectangle{  // fill background
        width: parent.width
        height: parent.height
        color: def.colHeadlineBackground
    }


    DefaultText {
        id:textAppName
        color: def.colFontTitle
        text: def.isS7App ? stringStore.appNameS7: stringStore.appNameLogo
        anchors.right: parent.right
        anchors.rightMargin: def.borderDistanceRight
        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceTitleToTop
        font.pixelSize:  def.titleFontSize
        visible:true

    }

    DefaultText {
        id: textIP
        color: def.colFontTitle
        text: "IP: "+ mainObject.currentIP
        //text: "size: "+ font.pixelSize + " - hl-size: " + hl_text.font.pixelSize + " - Height: " + parent.height
        anchors.top: parent.top
        anchors.topMargin: def.borderDistanceTop
        anchors.left: parent.left
        anchors.leftMargin: def.borderDistanceLeft
        font.pixelSize:  def.titleFontSize
        visible: false
    }

    DefaultText {
        id: hl_text
        color: def.colFontHeadline
        text: ""
        font.pixelSize: def.headlineFontSize
        font.bold: true
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
    }

    Headline_Button {
        id: edit_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.right: parent.right
        anchors.rightMargin: def.borderDistanceRight

        button_imagesource: "pictures/ICO_APP_Edit.svg"
        buttonpressed_image: "pictures/ICO_APP_Edit_pressed.svg"
        onButtonClicked: {
            buttonEditClicked()
        }
    }

    Headline_Button {
        id: add_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.right: edit_button.left
        anchors.rightMargin: def.headlineDistanceBetweenTwoButtons

        button_imagesource: "pictures/ICO_APP_Add.svg"
        buttonpressed_image: "pictures/ICO_APP_Add_pressed.svg"
        onButtonClicked: {
            buttonAddClicked()
        }
    }

    Headline_Button {
        id: back_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.left: parent.left
        anchors.leftMargin: def.borderDistanceLeft

        button_imagesource: "pictures/ICO_APP_Back.svg"
        buttonpressed_image: "pictures/ICO_APP_Back_pressed.svg"

        onButtonClicked: {
            buttonBackClicked()
        }
    }

    Headline_Button {
        id: save_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.right: parent.right
        anchors.rightMargin: def.borderDistanceRight

        button_imagesource: "pictures/ICO_APP_Save.svg"
        buttonpressed_image: "pictures/ICO_APP_Save_pressed.svg"
        onButtonClicked: {
            visible = false
            Qt.inputMethod.commit()
            buttonSaveClicked()
        }
    }


    Headline_Button {
        id: duplicate_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.right: delete_button.left
        anchors.rightMargin: def.headlineDistanceBetweenTwoButtons

        button_imagesource: "pictures/ICO_APP_CopyPage.svg"
        buttonpressed_image: "pictures/ICO_APP_CopyPage_pressed.svg"

        onButtonClicked: {
            buttonDuplicateClicked()
        }
    }
    Headline_Button {
        id: delete_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.right: parent.right
        anchors.rightMargin: def.borderDistanceRight

        button_imagesource: "pictures/ICO_APP_Remove.svg"
        buttonpressed_image: "pictures/ICO_APP_Remove_pressed.svg"
        onButtonClicked: {
            buttonDeleteClicked()
        }
    }

    Headline_Button {
        id: share_button
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.top: parent.top
        anchors.topMargin: def.headlineDistanceObjToTop
        anchors.right: parent.right
        anchors.rightMargin: def.borderDistanceRight

        button_imagesource: "pictures/ICO_APP_Share.svg"
        buttonpressed_image: "pictures/ICO_APP_Share_pressed.svg"
        onButtonClicked: {
            buttonShareClicked()
        }
    }


    DefaultText {
        id: hl_text_trend_full_screen
        color: def.colFontHeadline
        text: ""
        font.pixelSize: def.headlineFontSize
        font.bold: true
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        visible:true
    }

    Headline_Button {
        id: back_button_trend_full_screen
        width: def.headlineButtonSize
        height: width
        visible: false

        anchors.verticalCenter: parent.verticalCenter
        anchors.left: parent.left
        anchors.leftMargin: def.borderDistanceLeft

        button_imagesource: "pictures/ICO_APP_Back.svg"
        buttonpressed_image: "pictures/ICO_APP_Back_pressed.svg"
        onButtonClicked: {
            buttonBackFullScreenTrendViewClicked()
        }
    }

    Seperator{
        anchors.bottom: parent.bottom
        withShadow: true
    }



}
