import QtQuick 2.0
import DataStoreComponents 1.0
import LoggerComponents 1.0

import "." as S7App_Components


Item {

    //   property alias settingsModel:settingsListModel
    property string arrowText:""
    property variant langSettingsComp: null
    property variant langSettingsObj: null

    property variant rFrequencySettingsComp: null
    property variant rFrequencySettingsObj: null

    Headline {
        id: headline
        width: parent.width
        z:50

        headline_text: stringStore.langSelectionHeading
        backbutton_visible: true
        savebutton_visible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Headline Settings clicked!")
            backButtonOperation()

        }
        onButtonSaveClicked: {
            if (german.isSelected)
            {
                Logger.consoleLog(def.logInfo,"App Language Deutsch")
                DataStore.saveSettings("App_Language","Deutsch")
            }
            else if (english.isSelected)
            {
                Logger.consoleLog(def.logInfo,"App Language English")
                DataStore.saveSettings("App_Language", "English")
            }

            langSelector.selectLanguage(DataStore.getSettings("App_Language"));

            stackview.popPage()
        }
    }

    ModalDialog{
        id: langSettingsGoBackMsg
        title: stringStore.langSelectionHeading
        text: stringStore.goBackAlertMessage
        button_Yes: true
        button_No: true

        onYesButtonClicked: {
            stackview.popPage()
        }
    }

    Flickable{
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height

        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column{
            id: col
            width: parent.width

            CaptionListEntry{
                id: english
                caption: stringStore.langEnglish
                withRadioButton: true
                isSelected: false
                onFieldClicked: {
                    isSelected = true;
                    german.isSelected = false;
                }

            }

            ListSeperator{}

            CaptionListEntry{
                id: german
                caption: stringStore.langGerman
                withRadioButton: true
                isSelected: false
                onFieldClicked: {
                    isSelected = true;
                    english.isSelected = false;
                }
            }


        }

    }
    property string strLang : "";

    Component.onCompleted: {
        langSelector.selectLanguage(DataStore.getSettings("App_Language"));
    }
    onOpacityChanged: {

        if(opacity === 1)
        {
            strLang = DataStore.getSettings("App_Language")

            Logger.consoleLog(def.logInfo,"current language setting: " + strLang)
            if(strLang === "Deutsch")
            {
                german.isSelected = true
                english.isSelected = false
            }
            else
            {
                strLang = "English"
                english.isSelected = true
                german.isSelected = false
            }

            headline.savebutton_visible = true
        }
    }

    function backButtonOperation()
    {
        strLang = DataStore.getSettings("App_Language")

        Logger.consoleLog(def.logInfo,"current language setting: " + strLang)

        if((strLang === "Deutsch" && english.isSelected === true) ||
           (strLang === "English" && german.isSelected === true))
        {
            langSettingsGoBackMsg.open();
            return;
        }
        stackview.popPage();
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Language settings" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
    }


}


