import QtQuick 2.0
import LoggerComponents 1.0

Rectangle {


    property alias title:title.text
    property alias text_Component:messageText
    property alias text:messageText.text
    property alias textWidth:messageText.width
    property alias button_OK: okButton.visible
    property alias button_Cancel: cancelButton.visible
    property alias button_Yes: yesButton.visible
    property alias button_No: noButton.visible
    property alias button_Accept: acceptButton.visible
    property alias button_Decline: declineButton.visible
    property alias dialogWidth: field.width
    property alias useScrollable: scrollable.visible
    property alias inputfield_text:pwdText.input
    property alias inputfield_visible:pwdText.visible
    property alias text_scrollable: scrollText.text

    signal okButtonClicked()
    signal cancelButtonClicked()
    signal yesButtonClicked()
    signal noButtonClicked()

    signal acceptButtonClicked()
    signal declineButtonClicked()

    function open() {
        if(field.width === def.dialogWidth)
        {
            field.width = calcWidth()
            title.elide = Text.ElideRight
            title.width = field.width
        }
        visible = true;
        focus = true;
        Qt.inputMethod.hide()
    }

 
    id: diag
    anchors.fill: parent
    z:50
    color: "transparent"
    visible: false

    Rectangle{
        id: background
        anchors.fill: parent
        color: "black"
        opacity: 0.4

        MouseArea{
            anchors.fill: parent
            onClicked: {  }
        }
    }


    Rectangle{
        id: field
        width: def.dialogWidth
        height: content.height
        anchors.centerIn: diag
        color: def.colDialog
        opacity: 1
        visible: background.visible
    }

    Column{
        id: content
        anchors.centerIn: field
        width: field.width
        visible: background.visible

        Rectangle{
            width: parent.width
            height: def.borderDistanceTop
            color: "transparent"
        }

        Text{
            id: title
            anchors.left: parent.left
            anchors.leftMargin: def.borderDistanceRight
            font.pixelSize: def.dialogTitleFontSize
            color: def.colDiagFont
            width : def.dialogWidth
            anchors.horizontalCenter: parent.horizontalCenter
        }

        Rectangle{
            width: parent.width
            height: def.borderDistanceTop
            color: "transparent"
        }

        ListSeperator{
            visible: (title.text === "") ? false :true
        }

        Rectangle{
            width: parent.width
            height: def.borderDistanceTop
            color: "transparent"
        }


        Rectangle{
            id: scrollable
            height: def.dialogMaxHeight
            width: parent.width-2*def.borderDistanceLeft
            anchors.horizontalCenter: parent.horizontalCenter
            color: "transparent"
            visible: false

            Flickable{
                width: parent.width
                height: parent.height

                clip:true
                
                contentHeight: scrollText.height
                flickableDirection: Flickable.VerticalFlick
                boundsBehavior: Flickable.DragOverBounds
                Text{
                    id: scrollText
                    width: parent.width
                    onLinkActivated: Qt.openUrlExternally(link)
                    font.pixelSize: def.fontSizeNormalText
                    wrapMode: Text.Wrap
                    color: def.colDiagFont
                }
            }
        }
        InputField{
            id: pwdText
            width: parent.width-2*def.borderDistanceLeft
            anchors.horizontalCenter: parent.horizontalCenter
            input:""
            visible: false
            echoMode: TextInput.Password
            label: stringStore.authentication_password
            Keys.onPressed: {
                if (event.key === Qt.Key_Return)
                Qt.inputMethod.hide()
              }
           }

        Text{
            id: messageText
            width: parent.width-2*def.borderDistanceLeft
            anchors.horizontalCenter: parent.horizontalCenter
            font.pixelSize: def.fontSizeNormalText
            onLinkActivated: Qt.openUrlExternally(link)
            wrapMode: Text.Wrap
            color: def.colDiagFont
            visible: !scrollable.visible
        }

        Rectangle{
            width: parent.width
            height: 2*def.borderDistanceTop
            color: "transparent"
        }


        Rectangle{
            width: parent.width
            height: def.dialogTitleHeigth
            color: "transparent"

            Button{
                id: okButton
                isTextbutton: true
                visible: false
                anchors.right: parent.right
                anchors.rightMargin: def.borderDistanceRight
                button_Text: stringStore.ok
                onButtonClicked: {
                    diag.visible = false
                    diag.focus = false
                    stackview.currentItem.forceActiveFocus()
                    okButtonClicked();
                }
            }

            Button{
                id: cancelButton
                isTextbutton: true
                visible: false
                anchors.left: parent.left
                anchors.leftMargin: def.borderDistanceLeft
                button_Text: stringStore.cancel
                onButtonClicked: {
                    diag.visible = false
                    diag.focus = false
                    stackview.currentItem.forceActiveFocus()
                    cancelButtonClicked();
                }
            }

            Button{
                id: yesButton
                visible: false
                isTextbutton: true
                anchors.right: parent.right
                anchors.rightMargin: def.borderDistanceRight
                button_Text: stringStore.yes
                onButtonClicked: {
                    diag.visible = false
                    diag.focus = false
                    stackview.currentItem.forceActiveFocus()
                    yesButtonClicked();
                }
            }

            Button{
                id: noButton
                visible: false
                isTextbutton: true
                anchors.left: parent.left
                anchors.leftMargin: def.borderDistanceLeft
                button_Text: stringStore.no
                onButtonClicked: {
                    diag.visible = false
                    diag.focus = false
                    stackview.currentItem.forceActiveFocus()
                    noButtonClicked();
                }
            }

            Button{
                id: acceptButton
                isTextbutton: true
                visible: false
                anchors.right: parent.right
                anchors.rightMargin: def.borderDistanceRight
                button_Text: stringStore.acceptBtn
                onButtonClicked: {
                    diag.visible = false
                    acceptButtonClicked();
                }
            }

            Button{
                id: declineButton
                visible: false
                isTextbutton: true
                anchors.left: parent.left
                anchors.leftMargin: def.borderDistanceLeft
                button_Text: stringStore.declineBtn
                onButtonClicked: {
                    diag.visible = false
                    declineButtonClicked();
                }
            }

        }
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypress: in modal dialog ignored" )
        event.accepted = true;
    }

    function calcWidth()
    {
        Logger.consoleLog(def.logCritical,"title.contentWidth : "+ title.contentWidth +" & def.dialogWidth : "
                          + def.dialogWidth +"& def.deviceWidth : "+ def.deviceWidth)
        if(title.contentWidth > def.dialogWidth)
        {
            return def.deviceWidth - 2*def.borderDistanceLeft
        }
        return def.dialogWidth

    }
}

