import QtQuick 2.0
import LoggerComponents 1.0

Item {

    property string itemTitle: ""
    property alias title: col.itemTitle
    property string langSelected : stringStore.info_lang

    Headline {
        id: headline
        width: parent.width
        z: 50

        headline_text: stringStore.appInfoTitle
        backbutton_visible: true
        ipvisible: false

        onButtonBackClicked: {
            backButtonOperation()
        }
    }

    Flickable{
        id:flickCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height

        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column{
            id: col
            width: parent.width
            property string itemTitle: ""

            // Imprint
            CaptionListEntry{
                caption: stringStore.info_Imprint
                onFieldClicked: {
                    title = caption
                    Qt.openUrlExternally(stringStore.info_ImprintUrl);
                }
            }

            ListSeperator{}

            // Privacy Policy
            CaptionListEntry{
                caption: stringStore.info_PrivacyPolicy
                onFieldClicked: {
                    title = caption
                    Qt.openUrlExternally(stringStore.info_PrivacyPolicyUrl);
                }
            }

            ListSeperator{}

            // Terms Of Use
            CaptionListEntry{
                caption: stringStore.info_TermsOfUse
                onFieldClicked: {
                    title = caption
                    screen_appInfoContent.textFormat = Text.AutoText
                    screen_appInfoContent.text = infoModel.getInfoFile(langSelected, 0)
                    stackview.pushPage(screen_appInfoContent);
                }
            }

            ListSeperator{}

            // Security
            CaptionListEntry{
                caption: stringStore.info_Security
                onFieldClicked: {
                    title = caption
                    screen_appInfoContent.textFormat = Text.RichText
                    screen_appInfoContent.text = infoModel.getInfoFile(langSelected, 1)
                    stackview.pushPage(screen_appInfoContent);
                }
            }

            ListSeperator{}

            // Changelog
            CaptionListEntry{
                caption: stringStore.info_Changelog
                onFieldClicked: {
                    title = caption
                     screen_appInfoContent.textFormat = Text.RichText
                    screen_appInfoContent.text = infoModel.getInfoFile(langSelected, 2)
                    stackview.pushPage(screen_appInfoContent);
                }
            }

            ListSeperator{}

            // License
            CaptionListEntry{
                caption: stringStore.info_License
                onFieldClicked: {
                    title = caption
                    stackview.pushPage(screen_appInfoLicense);
                }
            }

            ListSeperator{}
            CaptionListEntry{
                caption: stringStore.info_Feedback
                onFieldClicked: {
                    title = caption
                    Qt.openUrlExternally(stringStore.info_FeedbackUrl);
                }
            }
            ListSeperator{}
        }
    }

    function backButtonOperation()
    {
        stackview.popPage();
    }

    Keys.onPressed: {
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
    }

    onVisibleChanged: {
        if (screen_appInfoContent){
            screen_appInfoContent.isLicenseViewShown = false;
        }
    }

    onOpacityChanged: {
        if(opacity == 0)
        {
            flickCtl.contentY = 0
        }
    }
}






