import QtQuick 2.0
import DataStoreComponents 1.0
import LoggerComponents 1.0

Item {

    property int screenMode: 0    
    property alias connListObj:conn_list
    property bool listElementSelected:false
    property bool clearSelectedItem:false
    property real scrollUpLimit:def.headlineHeight+(def.listviewEntryHeight * 1.5)
    property real scrollDownLimit:mainObject.height - (def.listviewEntryHeight * 1.5)
    property alias conFailedMsg:msg

    ModalDialog {
        id: msg
        title: stringStore.connectionFailedMsgTitle
        text: stringStore.connectionFailedMsg
        button_OK: true

        function openWithErrorMessage(strErrorMsg)
        {
            if(strErrorMsg === "LoginFailed")
            {
                msg.title=stringStore.autFailedMsgTitle
                msg.text=stringStore.autFailedMsgError
            }
            else if(strErrorMsg === "S7_INTERNALSERVER_ERROR")
            {
                msg.text = stringStore.firmwareupdateMsg
            }
            else
            {
                msg.title = strErrorMsg === "" ?stringStore.connectionFailedMsgTitle:stringStore.connectionFailedMsgTitle_en
                msg.text= strErrorMsg === "" ?  stringStore.connectionFailedMsg : strErrorMsg
            }
            msg.open()
        }
    }


    Column {
        anchors.fill:parent

        Headline {
            id: headline
            z: 2

            headline_text: (screenMode === 0) ? stringStore.deviceOverview: stringStore.deviceOverviewEdit
            addbutton_visible: true
            editbutton_visible: true
            backbutton_visible: false
            delete_visible:false

            onButtonEditClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDeviceOverview: Button Edit from Headline clicked!")

                //in edit mode
                screenMode = 1
                //infobutton.infoButtonState = 1
            }
            onButtonAddClicked: {
                if(def.isS7App)
                    screen_editConnection.plcType = -1
                else
                    screen_editConnection.plcType = 0
                Logger.consoleLog(def.logInfo,"ScreenDeviceOverview: Button Add from Headline clicked!")
                stackview.pushPage(screen_editConnection)
            }
            onButtonBackClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDeviceOverview: Button Back from Headline clicked!")
                backButtonOperation()
            }

            onButtonDeleteClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDeviceOverview: Button Delete from Headline clicked!" + screenMode)

                if(screenMode == 2)
                {
                    screenMode = 1
                }
                if(screenMode == 1)
                {
                   connectionModel.removeSelectedElements()
                }
            }
        }

       Rectangle{
           id: shadowSpace
            height: def.headlineShadowHeight
            width: parent.width
            color: "transparent"
        }

        ConnectionList {
            id: conn_list
            width: parent.width
            height: parent.height-bottomline.height-headline.height - shadowSpace.height
        }

        Buttonline {
            id: bottomline
            width: parent.width
            buttonSettings_visible: true
            buttonInformation_visible:true

            onButtonSettingsClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDeviceOverview: Button Settings from Buttonline clicked!")
                stackview.pushPage(screen_settings)
            }
            onButtonInformationClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDeviceOverview: Button Information from Buttonline clicked!")
                stackview.pushPage(screen_appInfo)
            }
            onVisibleChanged: {
                if(visible)
                {
                    height = def.buttonlineHeight
                }
                else
                {
                    height = 0
                }
            }
        }
    }

    Component.onCompleted: {
        if( DataStore.getSettings("ShowWhatIsNewDialog") === 0){
            whatisnewDialog.open();
        }
    }

    ModalDialog {
        id: whatisnewDialog
        dialogWidth: def.dialogWidth + 2
        title: stringStore.whatisnewDialogTitle
        text: infoModel.getWhatIsNewFile(stringStore.info_lang)
        button_OK: true
        text_Component.textFormat: Text.RichText

        onOkButtonClicked: {
            DataStore.saveSettings("ShowWhatIsNewDialog", 1);
        }
        onTextWidthChanged: {
            whatisnewDialog.text_Component.doLayout()
        }
    }

    onScreenModeChanged: {

        if(screenMode === 1)  // editing mode
        {
            headline.addbutton_visible = false
            headline.editbutton_visible = false
            headline.delete_visible = true
            headline.backbutton_visible = true
            bottomline.visible = false
        }
        else if(screenMode === 0) // normal view mode
        {
            headline.addbutton_visible = true
            headline.editbutton_visible = true
            headline.delete_visible = false
            headline.backbutton_visible = false
            bottomline.visible = true
        }
    }

    function backButtonOperation()
    {
        //in edit mode
        screen_overview.screenMode = 0
        //infobutton.infoButtonState = 1
    }

    function initializeConnList()
    {
        connectionModel.initSavedConnection();
    }

    function updateConnectionList()
    {
        if(connectionModel.getSize() > 0)
        {           
            connectionModel.updateConnectionModel()
        }
    }

    function clearConnectionList()
    {
        if(connectionModel.getSize() > 0)
        {
            connectionModel.clearConnectionModel()
        }
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed: ScreenDeviceOverview" )
        if (msg.visible || whatisnewDialog.visible || busyIndicator.visible)
        {
            event.accepted = true; // no keys accepted with active dialog!
        }
    }
}
