import QtQuick 2.0
import QtGraphicalEffects 1.0

import LoggerComponents 1.0

import "." as S7App_Components

Item {

    property alias ack_visible : acknowledge.visible
    property alias timerInDiag_obj :timerInDiagnostics

    Column {
        anchors.fill:parent

        Headline {
            id: headline
            width: parent.width

            headline_text: def.isS7App ? stringStore.diagnosticsS7Headline : stringStore.diagnosticsHeadline
            backbutton_visible: true
            share_visible:def.isS7App ? true : false
            ipvisible: true

            onButtonBackClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDiagnostics: Button Back from Headline clicked!")
                backButtonOperation()
            }
            onButtonShareClicked: {
                Logger.consoleLog(def.logInfo,"ScreenDiagnostics: Button Share from Headline clicked!")
                diagnosticbuffController.getAllDiagnosticsInfo()

            }
        }

        Rectangle{
            id: shadowSpace
            height: def.headlineShadowHeight
            width: parent.width
            color: "transparent"
        }

        DiagnosticList {
            id: diag_list
            //anchors.top : headline.bottom
            width: parent.width
            height: (parent.height - (def.isS7App ? 0 : bottomline.height) - headline.height - shadowSpace.height)
        }
        Rectangle
        {
            id:newStories
            height: headline.height/2
            width: headline.width
            anchors.top: headline.bottom
            color:"RED"
            //opacity: 0.8
            visible: false
            // z : 1
            MouseArea
            {
                anchors.fill: parent
                onClicked:
                {
                    diag_list.diag_listview.positionViewAtIndex(0,ListView.Beginning)
                    newStories.visible = false
                    diag_list.anchors.top = headline.bottom
                }
            }
            Text {
                id: name
                text: stringStore.diagnosticsNewAlerts
                color: "white"
                anchors.centerIn: parent
                font.pixelSize: headline.height*0.25
                font.bold: true
            }

        }

        Rectangle{
            id: bottomline
            width: parent.width
            height: def.buttonlineHeight

            LinearGradient {
                    anchors.fill: parent
                    start: Qt.point(0, 0)
                    end: Qt.point(parent.width, 0)
                    gradient: Gradient {
                        GradientStop { position: 0.0; color: def.backgroundgradientleft }
                        GradientStop { position: 1.0; color: def.backgroundgradientright}
                    }
                }

            Button{
                id:acknowledge
                width: def.standardButtonWidthExtendedPlus
                anchors.centerIn: parent
                isTextbutton: true
                button_Text: stringStore.acknOnPLC
                visible:def.isS7App ? false : true
                onButtonClicked: {
                    Logger.consoleLog(def.logCritical,"buttonclick:in Diagnostic" )
                    diagnosticbuffController.clearDiagnosticsInfo(isSelectedItems())

                }
            }
        }
    }


    function isSelectedItems()
    {
        //only for logo 0ba7 we need to clear only selected items otherwise all errors
        if( def.isOBA7() )
        {
            return true;
        }
        return false;
    }

    function backButtonOperation()
    {
        diagnosticbuffController.removeDiagnosticsInfo()
        stackview.pop();
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Diagnostic" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            if(busyIndicator.visible == false)
            backButtonOperation()
        }
    }


    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible = false
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        diagnosticbuffController.removeDiagnosticsInfo()
        stackview.replace(screen_overview)
    }

    onOpacityChanged:
    {
        if(opacity === 1)
        {
            conController.connectionError.connect(onConnectionError)
            if(def.isS7App)
            {
                diagnosticbuffController.timeStampFound.connect(onTimeStampFound)
                timerInDiagnostics.running = true
                diag_list.listAtBegining.connect(listAtBeginning)
                appController.appStatusChanged.connect(onAppstatusChanged)
                conController.badRequest.connect(onBadRequest)
                screen_authentication.authentication.startRefreshing.connect(onAuthenticationSucceded)
            }
        }
        else
        {
            conController.connectionError.disconnect(onConnectionError)
            if(def.isS7App)
            {
                diagnosticbuffController.timeStampFound.disconnect(onTimeStampFound)
                timerInDiagnostics.running = false
                diag_list.listAtBegining.disconnect(listAtBeginning)
                appController.appStatusChanged.disconnect(onAppstatusChanged)
                conController.badRequest.disconnect(onBadRequest)
                screen_authentication.authentication.startRefreshing.disconnect(onAuthenticationSucceded)
            }
        }
    }

    function onTimeStampFound()
    {
        Logger.consoleLog(def.logCritical,"**signal recieved in onTimeStampFound**")
        if(diag_list.isAtBegin)
        {
            newStories.visible =false
        }
        else
        {
            newStories.visible = true
            diag_list.anchors.top = newStories.bottom
        }
    }
    function listAtBeginning()
    {
        Logger.consoleLog(def.logCritical,"**signal recieved in DiaglistAtBeginning**")
        newStories.visible = false
        diag_list.anchors.top = headline.bottom
    }
    function onBadRequest(strErrorMsg)
    {
        busyIndicator.visible = false
        badRequestDialog.open()
    }
    ModalDialog
    {
        id: badRequestDialog
        title: stringStore.diagnosticsS7Headline
        text: stringStore.badRequestInDiagnostics//"Bad request"
        button_OK: true
        onOkButtonClicked:
        {
            timerInDiagnostics.running = true
        }
    }

    Timer{
        id: timerInDiagnostics
        interval: 5000 ; running: false; repeat:true
        onTriggered: {
            Logger.consoleLog(def.logCritical,"**Timer is running in Diagnostics**")
            diagnosticbuffController.getDiagnosticsInfo()
        }
    }

    function onAppstatusChanged()
    {

        if(appController.appStaus !== def.appActive)
        {
            timerInDiagnostics.running = false
            Logger.consoleLog(def.logCritical,"**Timer stopped in Diagnostics**"+appController.appStaus)
        }
    }

    function onAuthenticationSucceded()
    {
        Logger.consoleLog(def.logCritical,"**Timer started in Diagnostics**"+appController.appStaus)
        timerInDiagnostics.running = true
    }
}
