import QtQuick 2.0
import LoggerComponents 1.0

Rectangle {
    width: parent.width
    height: parent.height
    color: "transparent"

    Headline {
        id: headline
        z:2

        headline_text: stringStore.diagnosticsDetailHeadline
        backbutton_visible: true
        ipvisible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Headline diagnostic details clicked!")
            backButtonOperation()   
        }
    }

    Flickable{
        id:flickCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height
        contentHeight: col.height + def.borderDistanceBottom
        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column
        {
            id: col
            width: parent.width

            spacing: 1.2 * def.borderDistanceTop

            Item{
                width: parent.width
                height: time.height

                DefaultText{
                    id: time
                    text: ""
                    bold: true
                    anchors{left:parent.left; leftMargin:def.borderDistanceLeft; top: parent.top; topMargin: 1.2*def.borderDistanceTop}
                }

                DefaultText{
                    id: date
                    text: ""
                    bold: true
                    anchors{right:parent.right; rightMargin:def.borderDistanceRight; top: parent.top; topMargin: 1.2*def.borderDistanceTop}
                }
            }

            DefaultText{
                id: logNumber
                width: parent.width
                wrapMode: Text.WordWrap

                text: ""
                bold: true
                anchors{left:parent.left; leftMargin: def.borderDistanceLeft;horizontalCenter: parent.horizontalCenter}
            }

            DefaultText{
                id: eventID
                text: ""
                bold: true
                anchors{left:parent.left; leftMargin: def.borderDistanceLeft;}
            }

            ListSeperator{ width: parent.width;anchors {right:parent.right; rightMargin:def.borderDistanceRight;}}

            DefaultText{
                id: detailedMessageText
                width: parent.width
                wrapMode: Text.WordWrap

                text: ""
                anchors{left:parent.left; leftMargin: def.borderDistanceLeft;right:parent.right; rightMargin:def.borderDistanceRight;}

            }

        }
    }

    onOpacityChanged: {
        if(opacity === 1)
        {
            var currDetailInfoIndex = diagnosticBufferModel.getDetailInfoIndex()
            time.text = diagnosticBufferModel.getErrorTime(currDetailInfoIndex)
            date.text = diagnosticBufferModel.getErrorDate(currDetailInfoIndex)
            logNumber.text = diagnosticBufferModel.getErrorID(currDetailInfoIndex)
            eventID.text = diagnosticBufferModel.getEventID(currDetailInfoIndex)
            detailedMessageText.text = diagnosticBufferModel.getEventDetailInfo(currDetailInfoIndex)
	    busyIndicator.visible = false
        }
        else
        {
            flickCtl.contentY = 0
        }
    }

    function backButtonOperation()
    {
        stackview.pop();
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Diagnostic details" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            busyIndicator.visible = false
            backButtonOperation()
        }
    }

}
