import QtQuick 2.0
import QtQuick.Window 2.1
import LoggerComponents 1.0

import "." as S7App_Components

Rectangle {
    width: parent.width
    height: parent.height
    property alias stationNameLabel:stationName.input
    property alias ipAddressLabel:ipAddress.input
    property alias userNameLabel:username.input
    property alias passwordLabel:password.input
    property int screenMode:screen_overview.screenMode
    property int plcType:-1
    property string sslInfo:""

    color: "transparent"

    ModalDialog {
        id: editVersionChange
        title: stringStore.editConnectionPLCUpdateTitle
        text: stringStore.editConnectionPLCUpdateMsg
        button_Yes: true
        button_No: true

        onYesButtonClicked: {
            connectionModel.removeConnConfig(mainObject.currentIndex)
            updateConnectionInfo()
        }

        onNoButtonClicked: {
            plcType = 1
            headline.savebutton_visible = true
        }
    }

    ModalDialog {
        id: editConDupMsg
        title: stringStore.editConnectionMsgTitle
        text: stringStore.editConnectionDupMsg
        button_OK: true
    }

    ModalDialog{
        id: editConGoBackMsg
        title: stringStore.editConnectionMsgTitle
        text: stringStore.goBackAlertMessage
        button_Yes: true
        button_No: true

        onYesButtonClicked: {
            stackview.popPage()
        }
    }

    ModalDialog {
        id: sslCertDeleteMsg
        title: stringStore.sslHeader
        button_OK: true
    }

    ModalDialog{
        id: invalidIPaddress
        title: stringStore.editConnectionMsgTitle
        text: stringStore.invalidIP
        button_OK: true
        onOkButtonClicked:
        {
            headline.savebutton_visible = true
        }
    }

    Headline{
        id: headline
        z: 2
        headline_text: (screenMode === 0) ? stringStore.createConnection : stringStore.editConnection
 	backbutton_visible: true
        savebutton_visible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Headline clicked!")
            Logger.consoleLog(def.logInfo,"Button Infos for " + stationName.input + " clicked!")
            backButtonOperation()

        }

        onButtonSaveClicked: {
            Logger.consoleLog(def.logInfo,"Button Add from Headline clicked!")
            Logger.consoleLog(def.logCritical,"Button Infos for " + stationName.input + " clicked! " + plcType)

            if(def.isLogoApp)
            {
                if(connectionModel.getSize() === 0 || screenMode === 0)
                {
                    updateConnectionInfo()
                }
                else if(plcType === connectionModel.getPlcType(mainObject.currentIndex))
                {
                    updateConnectionInfo()
                }
                else if(plcType < connectionModel.getPlcType(mainObject.currentIndex))
                {
                    editVersionChange.open()
                }
                else if(plcType > connectionModel.getPlcType(mainObject.currentIndex))
                {
                    Logger.consoleLog(def.logCritical,"formatting the data as per the password")
                    connectionModel.reFormatConnConfig(mainObject.currentIndex,password.input)
                    Logger.consoleLog(def.logCritical,"After formatting the data as per the password")
                    updateConnectionInfo()
                }
            }
            else
            {
                updateConnectionInfo()
            }

        }
    }

    Flickable{
        id:flickCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height
        contentHeight: col.height + def.borderDistanceBottom
        contentWidth: col.width
        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column
        {
            id: col
            width: parent.width

            Rectangle{
                width: col.width
                height: def.inputFieldHeight
                color: "transparent"

                Image {
                    id: deviceImage
                    height: def.editConnectionDeviceImageHeight
                    width: height*1.2
                    visible: true
                    fillMode: Image.PreserveAspectFit
                    horizontalAlignment: Image.AlignLeft
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.left: parent.left
                    anchors.leftMargin: def.borderDistanceLeft
                    source: (def.isS7App)?"pictures/S7-Default.svg":"pictures/LOGO_0BA7.svg"
                }


                DefaultText {
                    id:textType
                    visible:def.isLogoApp
                    text: stringStore.type
                    anchors.bottom: radioPLc1.top
                    anchors.bottomMargin: def.borderDistanceTop
                    anchors.left: deviceImage.right
                    anchors.leftMargin: def.borderDistanceLeft*2
                    font.pixelSize: def.fontSizeCaption
                 }
                RadioButton{
                    id: radioPLc1
                    anchors.left: textType.left
                    anchors.bottom: deviceImage.bottom
                    visible:def.isLogoApp
                    onRadioButtonClicked: {
                        plcType = 0;

                    }
                }

                DefaultText {
                    id: radiotext1
                    text: "0BA7"
                    font.pixelSize: def.fontSizeCaption
                    anchors.verticalCenter: radioPLc1.verticalCenter
                    anchors.left: radioPLc1.right
                    visible:def.isLogoApp
                    anchors.leftMargin: def.borderDistanceLeft
                }


                RadioButton{
                    id: radioPLc2
                    anchors.left: radiotext1.right
                    anchors.leftMargin: def.borderDistanceLeft
                    anchors.bottom: deviceImage.bottom
                    visible:def.isLogoApp
                    onRadioButtonClicked: {
                        plcType = 1;
                    }
                }

                DefaultText {
                    text: "0BA8"
                    font.pixelSize: def.fontSizeCaption
                    anchors.verticalCenter: radioPLc1.verticalCenter
                    anchors.left: radioPLc2.right
                    visible:def.isLogoApp
                    anchors.leftMargin: def.borderDistanceLeft
                }
            }
            InputField{
                id: stationName
                label: stringStore.name
                input: ""
            }

            InputField{
                id: ipAddress
                label: stringStore.adress
                input: ""
            }

            InputField{
                id: username
                label: stringStore.conUsername
                input: (def.isS7App)?"":"App User"
                visible: isUserFieldVisible()
            }

            InputField{
                id: password
                label: stringStore.conPassword
                input: ""
                echoMode: TextInput.Password
                visible: isUserFieldVisible()
            }

            Rectangle{
                width: col.width
                height: def.standardButtonHeight
                color: "transparent"
                visible: ((def.isS7App) && (screenMode === 1))
                anchors.left:password.left
                anchors.right: password.right
                Button{
                    id: delCertButton
                    width: def.standardButtonWidth *1.5
                    anchors.left: parent.left
                    isTextbutton: true                    
                    anchors.leftMargin: def.borderDistanceLeft
                    button_Text: stringStore.sslDelCertBtn
                    onButtonClicked: {
                        Qt.inputMethod.hide()
                        sslInfo = (screenMode === 0)?"":conController.getSSLCertificate(mainObject.currentIndex)
                        if(sslInfo !== "")
                        {
                            conController.deleteSSLCertificate(mainObject.currentIndex);
                            sslCertDeleteMsg.text = stringStore.sslCertDeletedMsg
                        }
                        else
                        {
                            sslCertDeleteMsg.text = stringStore.sslNoCertAvailableMgs
                        }
                        sslCertDeleteMsg.open()
                    }
                }

                Button{
                    id: showCertButton
                    isTextbutton: true
                    width: def.standardButtonWidth *1.5                   
                    anchors.right:  parent.right
                    anchors.rightMargin: def.borderDistanceLeft
                    button_Text: stringStore.sslShowCertBtn
                    onButtonClicked: {
                        Qt.inputMethod.hide()
                        sslInfo = (screenMode === 0)?"":conController.getSSLCertificate(mainObject.currentIndex)
                        sslCertDialog.text_scrollable = (sslInfo==="") ? stringStore.sslNoCertAvailable : sslInfo;
                        sslCertDialog.button_Accept = false;
                        sslCertDialog.button_Decline = false;
                        sslCertDialog.button_OK = true;
                        sslCertDialog.open()                       
                    }
                }
            }

        }

    }

    onScreenModeChanged:
    {
        if(screenMode === 0)
        {
            ipAddress.input = ""
            stationName.input = ""
            username.input = (def.isS7App)?"":"App User"
            password.input = ""

        }
    }

    function isUserFieldVisible()
    {
        if( (!def.isLogoApp) || (plcType == 1) )
        {
            return true;
        }
        return false;
    }

    function backButtonOperation()
    {
        editConGoBackMsg.open()
    }

    function updateConnectionInfo()
    {
        var retVal = conController.validateAndRemoveLeadingZeros(ipAddress.input)
        Logger.consoleLog(def.logInfo,"ipaddresstext is "+ ipAddress.input)
        if(retVal === "invalid")
        {
            Logger.consoleLog(def.logInfo,"Invalid address")
            ipAddress.input = ""
            invalidIPaddress.open()
            return;
        }
        ipAddress.input = retVal

        Logger.consoleLog(def.logInfo,"Station Name = " + stationName.input + " ip Address =  " + ipAddress.input)

        if(def.isLogoApp)
        {
            if(plcType == 0)
            {
                username.input = ""
                password.input = ""
            }
        }

        if(screenMode === 0)
        {
            var str = connectionModel.appendElement(stationName.input,
                                  ipAddress.input,
                                  "unknown",username.input,password.input,plcType,connectionModel.getSize())

            if(str === false)
            {
                editConDupMsg.open()
            }
            else
            {
                stackview.popPage()
            }

        }
        else if(screenMode === 1)
        {

            Logger.consoleLog(def.logInfo,"Index at save point = "+mainObject.currentIndex)
            connectionModel.verifyingIpChangeFinished.connect(onVerifyingIpChangeFinished)
            busyIndicator.visible = true
            connectionModel.updateElement(stationName.input,
                                          ipAddress.input,"unknown",username.input,password.input,
                                          plcType,mainObject.currentIndex,def.isS7App)

        }
        if(def.isLogoApp)
        {
            if(plcType == 0)
            {
                username.input = "App User"
                password.input = ""
            }
        }
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in edit connection" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
        if(busyIndicator.visible == false)
            backButtonOperation()	
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }   
    onOpacityChanged: {
        if(opacity == 1)
        {
            headline.savebutton_visible = true
            if(screenMode === 0)
            {
                ipAddress.input = ""
                stationName.input = ""
                username.input = (def.isS7App)?"":"App User"
                password.input = ""
            }
        }
        else if(opacity == 0)
        {
            flickCtl.contentY = 0
	    if(busyIndicator.visible == true)
	    	busyIndicator.visible = false
	    connectionModel.verifyingIpChangeFinished.disconnect(onVerifyingIpChangeFinished)
        }
    }
    
    onPlcTypeChanged:
    {
        Logger.consoleLog(def.logInfo,"----------------------------did-----------------")
        if(def.isS7App)
        {
            if(plcType == 0)
                deviceImage.source = "pictures/S7-1200.svg"
            else if(plcType == 1)
                deviceImage.source = "pictures/S7-1500.svg"
            else
                deviceImage.source = "pictures/S7-Default.svg"
        }
        else
        {
            if(plcType == 0)
            {
                radioPLc1.isSelected = true;
                radioPLc2.isSelected = false;
                deviceImage.source = "pictures/LOGO_0BA7.svg"

            }
            else
            {
                radioPLc1.isSelected = false;
                radioPLc2.isSelected = true;
                deviceImage.source = "pictures/LOGO_0BA8.svg"

            }
        }
    }
    function onVerifyingIpChangeFinished()
    {
        Logger.consoleLog(def.logInfo,"onVerifyingIpChangeFinished")
        busyIndicator.visible = false
        screen_overview.screenMode = 0
        stackview.popPage()
        connectionModel.verifyingIpChangeFinished.disconnect(onVerifyingIpChangeFinished)
    }

}
