import QtQuick 2.0
import DataStoreComponents 1.0
import LoggerComponents 1.0

import "." as S7App_Components
Rectangle
{
    id: refreshsettings

    //    property alias rf_seconds:freqSettingsModel.rf_secs
    width: parent.width
    height: parent.height

    color: "transparent"
    property int initialIndex:0
    property bool atStartup:false

    signal frequencyChanged(var freq)

    Headline {
        id: headline
        width: parent.width

        headline_text: stringStore.refrFrequencyTitle
        backbutton_visible: true
        savebutton_visible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Refresh Frequency Settings clicked!")
            backButtonOperation()
        }

        onButtonSaveClicked: {
            Logger.consoleLog(def.logInfo,"Button Save from Refresh Frequency Settings clicked!")

            initialIndex = freqSettingsModel.currentIndex
            DataStore.saveSettings("rf_seconds", freqSettingsModel.choosenSecs )
            frequencyChanged(freqSettingsModel.choosenSecs)
            stackview.popPage()
        }
    }

    ModalDialog{
        id: freqSettingsGoBackMsg
        title: stringStore.refrFrequencyTitle
        text: stringStore.goBackAlertMessage
        button_Yes: true
        button_No: true

        onYesButtonClicked: {
            stackview.popPage()
            freqSettingsModel.setProperty(freqSettingsModel.currentIndex, "isTicked", false)
            freqSettingsModel.currentIndex=initialIndex

        }
    }

    ListView {
        width: parent.width
        height: parent.height-headline.height
        anchors{top: parent.top; topMargin: headline.height}
        model: freqSettingsModel
        delegate: rfSettingsDelegate
    }

    Component {
        id: rfSettingsDelegate

        CaptionListEntry{
            caption: {
                secs + " " + stringStore.secs
                if (secs == "1")
                    secs + " " + stringStore.sec
            }
            withRadioButton: true
            isSelected: isTicked

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    Logger.consoleLog(def.logInfo,"clicked: " + secs + " Seconds")
                    freqSettingsModel.currentIndex = index
                    freqSettingsModel.setProperty(freqSettingsModel.prevIndex, "isTicked", false)
                    freqSettingsModel.setProperty(index, "isTicked", true)
                    freqSettingsModel.prevIndex = index
                    freqSettingsModel.choosenSecs = secs
                    headline.savebutton_visible = true
                }
            }

            ListSeperator{
                anchors.bottom:  parent.bottom
            }

        }
    }

    function setSelectedIndex()
    {
        Logger.consoleLog(def.logInfo,"Selected freq ")
        var secs = DataStore.getSettings("rf_seconds") ;
        if(secs.toString() === "0" )
        {
            DataStore.saveSettings("rf_seconds", "1" )
            freqSettingsModel.setProperty(1, "isTicked", true)
        }
        else
        {
            freqSettingsModel.setProperty(1, "isTicked", false)
            for(var count = 0;count < freqSettingsModel.count; count++)
            {
                if(secs.toString() === freqSettingsModel.get(count).secs)
                {
                    freqSettingsModel.prevIndex = count
                    Logger.consoleLog(def.logInfo,"Selected freq " + count + " Seconds" + secs )
                    freqSettingsModel.currentIndex = count
                    freqSettingsModel.setProperty(count, "isTicked", true)
                    initialIndex = count
                }
            }
        }
    }

    ListModel {
        id : freqSettingsModel

        property int prevIndex: 1
        property int currentIndex: -1
        property real choosenSecs: 0

        ListElement {
            secs : "0.5"
            isTicked : false;
        }
        ListElement {
            secs : "1"
            isTicked : false;
        }
        ListElement {
            secs : "1.5"
            isTicked : false;
        }
        ListElement {
            secs : "2"
            isTicked : false;
        }
        ListElement {
            secs : "2.5"
            isTicked : false;
        }
        ListElement {
            secs : "3"
            isTicked : false;
        }
        ListElement {
            secs : "5"
            isTicked : false;
        }
        ListElement {
            secs : "10"
            isTicked : false;
        }
        ListElement {
            secs : "15"
            isTicked : false;
        }
        ListElement {
            secs : "20"
            isTicked : false;
        }
        ListElement {
            secs : "25"
            isTicked : false;
        }
        ListElement {
            secs : "30"
            isTicked : false;
        }
    }

    onOpacityChanged: {
        if(opacity == 1)
        {
            headline.savebutton_visible = true
            if(!atStartup)
            {
                setSelectedIndex()
                atStartup = true
            }
            else
            {
                freqSettingsModel.setProperty(initialIndex, "isTicked", true)
                freqSettingsModel.prevIndex = initialIndex
            }
        }
    }

    function backButtonOperation()
    {
        Logger.consoleLog(def.logInfo,"initial index= "+initialIndex+" Current index = "+freqSettingsModel.currentIndex)
        if(initialIndex != freqSettingsModel.currentIndex)
        {
            freqSettingsGoBackMsg.open()
            return;
        }
        stackview.popPage()
        freqSettingsModel.setProperty(freqSettingsModel.currentIndex, "isTicked", false)
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Tag List" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
    }
    Component.onCompleted: {
        setSelectedIndex()
    }
}
