import QtQuick 2.0
import DataStoreComponents 1.0
import QtQuick.Controls 1.2
import LoggerComponents 1.0
import QtGraphicalEffects 1.0


import "." as S7App_Components
Item {

    property int mode: -1
    // 0: inputs, 1: outputs, 2: flags, 3: shift register bits

    property var headlines: [   stringStore.io_Inputs,         stringStore.io_Ouputs,
                                stringStore.io_Flags,          stringStore.io_ShiftRegBits,
                                stringStore.io_CursorKeys,     stringStore.io_AnalogInputs,
                                stringStore.io_AnalogOutput,   stringStore.io_AnalogFlags,
                                stringStore.io_FuntionKeys                                 ]

    // Based on the number of bits for each mode mapped to the headlines array declared above
    property var columncounts: [24,16, 27, 32, 4, 8, 2, 16, 4]


    Headline {
        id: headline
        width: parent.width
        z:50

        headline_text: headlines[mode]
        backbutton_visible: true
        ipvisible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Headline IO State clicked!")
            backButtonOperation()
        }
    }

    Component{
        id: delegateHeader
        Rectangle{

            height: def.ioTableFieldHeight
            color: "transparent" //def.backgroundgradientright
            DefaultText{
                anchors.centerIn: parent
                font.pixelSize: def.ioTableFontSize
                font.bold: true
                text: styleData.value
            }
            Rectangle{
                width: styleData.column === 0 ? parent.width-def.borderDistanceLeft : parent.width
                height: 1
                anchors.bottom: parent.bottom
                anchors.right: parent.right
                color: def.ioTableLineColor
            }
            Rectangle{
                width: 1
                height: parent.height
                anchors.right: parent.right
                color: def.ioTableLineColor
                visible: styleData.column === 0 ? true : false
            }
        }
    }

    Component{
        id: delegateRow
        Rectangle{
            color: "transparent"
            height: def.ioTableFieldHeight
        }
    }

    Component{
        id: delegateItem
        Rectangle{
            height: def.ioTableFieldHeight
            width: parent.width
            color: "transparent"
            DefaultText{
                anchors.centerIn: parent
                font.pixelSize: def.ioTableFontSize
                font.bold: styleData.column === 0 ? true : false
                text: styleData.value
            }
            Rectangle{
                width: styleData.column === 0 ? parent.width-def.borderDistanceLeft : parent.width
                height: 1
                anchors.bottom: parent.bottom
                anchors.right: parent.right
                color: def.ioTableLineColor
            }
            Rectangle{
                width: 1
                height: parent.height
                anchors.right: parent.right
                color: def.ioTableLineColor
                visible: styleData.column === 0 ? true : false
            }
        }
    }

    Flickable{
        visible: mode >=0 ? true : false
        width: parent.width
        height: parent.height - headline.height - bottomline.height
        anchors.top: headline.bottom
        contentHeight: tablesrb.height
        contentWidth: tablesrb.width  + def.borderDistanceRight
        boundsBehavior: Flickable.DragOverBounds

        TableView {
            id: tablesrb
            anchors.top: parent.top
            height : mainObject.height - headline.height - bottomline.height
            width: tbcol0.width+tbcol1.width+tbcol2.width+tbcol3.width+tbcol4.width+
                   tbcol5.width+tbcol6.width+tbcol7.width+tbcol8.width+tbcol9.width+
                   tbcol10.width+tbcol11.width+tbcol12.width+tbcol13.width+tbcol14.width+
                   tbcol15.width+tbcol16.width+tbcol17.width+tbcol18.width+tbcol19.width+
                   tbcol20.width+tbcol21.width+tbcol22.width+tbcol23.width+tbcol24.width+
                   tbcol25.width+tbcol26.width+tbcol27.width+tbcol28.width+tbcol29.width+
                   tbcol30.width+tbcol31.width+tbcol32.width+tbcol33.width+tbcol34.width+
                   tbcol35.width+tbcol36.width+tbcol37.width+tbcol38.width+tbcol39.width+
                   tbcol40.width+tbcol41.width+tbcol42.width+tbcol43.width+tbcol44.width+
                   tbcol45.width+tbcol46.width+tbcol47.width+tbcol48.width+tbcol49.width+
                   tbcol50.width+tbcol51.width+tbcol52.width+tbcol53.width+tbcol54.width+
                   tbcol55.width+tbcol56.width+tbcol57.width+tbcol58.width+tbcol59.width+
                   tbcol60.width+tbcol61.width+tbcol62.width+tbcol63.width+tbcol64.width

            backgroundVisible: false
            frameVisible: false

            __horizontalScrollBar.visible: false
            __verticalScrollBar.visible: false
            __listView.clip: true
	    
            TableViewColumn{ id: tbcol0; role: "time"  ; title: stringStore.io_tableTitle ; width: def.ioTableTimecolumnWidth;movable:false }
            TableViewColumn{ id: tbcol1; role: "i1"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol2; role: "i2"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol3; role: "i3"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol4; role: "i4"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol5; role: "i5"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol6; role: "i6"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol7; role: "i7"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol8; role: "i8"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol9; role: "i9"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol10; role: "i10" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol11; role: "i11" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol12; role: "i12" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol13; role: "i13" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol14; role: "i14" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol15; role: "i15" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol16; role: "i16" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol17; role: "i17" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol18; role: "i18" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol19; role: "i19" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol20; role: "i20" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol21; role: "i21" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol22; role: "i22" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol23; role: "i23" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol24; role: "i24" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol25; role: "i25" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol26; role: "i26" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol27; role: "i27" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol28; role: "i28" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol29; role: "i29" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol30; role: "i30" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol31; role: "i31" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol32; role: "i32" ;  width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol33; role: "i33"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol34; role: "i34"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol35; role: "i35" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol36; role: "i36"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol37; role: "i37"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol38; role: "i38"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol39; role: "i39"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol40; role: "i40"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol41; role: "i41"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol42; role: "i42"  ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol43; role: "i43" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol44; role: "i44" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol45; role: "i45" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol46; role: "i46" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol47; role: "i47" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol48; role: "i48" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol49; role: "i49" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol50; role: "i50" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol51; role: "i51" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol52; role: "i52" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol53; role: "i53" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol54; role: "i54" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol55; role: "i55" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol56; role: "i56" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol57; role: "i57" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol58; role: "i58" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol59; role: "i59" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol60; role: "i60" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol61; role: "i61" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol62; role: "i62" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol63; role: "i63" ; width: def.ioTableColumnWidth }
            TableViewColumn{ id: tbcol64; role: "i64" ; width: def.ioTableColumnWidth }

            model: ioStatusTableModel
            headerDelegate: delegateHeader
            rowDelegate: delegateRow
            itemDelegate: delegateItem
        }
    }

    Buttonline {
        id: bottomline
        width: parent.width
        anchors.bottom: parent.bottom

        buttonMonitor_visible: true

        onButtonMonitorClicked: {
            Logger.consoleLog(def.logInfo,"ScreenIOState: Button Monitor from Buttonline clicked!")
            screen_trendView.readMode = mode
            ioStatusListModel.removeAll()
            ioStatusController.setIOStatusModel(ioStatusListModel)
            ioStatusController.setIOMode(mode)
            ioStatusListModel.setIOConfiguration(connectionModel.getPlcType(mainObject.currentIndex))
            ioStatusListModel.createIOList()
            screen_trendView.initializeTrendData(ioStatusController,ioStatusListModel,trendViewIOListItem)
            screen_trendView_FullScreen.initializeFullScreenTrendData(ioStatusController,ioStatusListModel,dataGridItem)
	    screen_trendView.trendViewObj.initializeTrendView()
            if(mode === 5 || mode === 6 || mode === 7) // for Analog Input,Output & Flag modes
            {
                screen_trendView.trendViewObj.setAxisRange(0,100)
                screen_trendView.trendViewObj.setTickCount(5)
            }
            else
            {
                screen_trendView.trendViewObj.setAxisRange(0,1)    // for Digital modes
                screen_trendView.trendViewObj.setTickCount(2)
            }
            stackview.pushPage(screen_trendView)
        }
    }

    function onConnectionError(strErrorMsg)
    {
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        ioStatusController.stopIOStatusProcessing()
        stackview.replace(screen_overview)
    }

    function backButtonOperation()
    {
        ioStatusController.stopIOStatusProcessing()
        ioStatusListModel.removeAll()
        stackview.popPage();
    }

    function updateTable()
    {
        ioStatusTableModel.setIOConfiguration(connectionModel.getPlcType(mainObject.currentIndex))
        var colFirst = tablesrb.getColumn(0);
        colFirst.width = def.ioTableTimecolumnWidth
        colFirst.title = "Time/Data"
        colFirst.role = "time"
        colFirst.movable = false

        for(var j = 1;j <= 64; j++)
	{
            var colLast = tablesrb.getColumn(j);

            if(j <= ioStatusTableModel.getColumnCount())
            {
                colLast.visible = true
                colLast.width = def.ioTableColumnWidth
            }
            else
            {
                colLast.visible = false
                colLast.width = 0
            }
	}
        for(var i = 1;i <= ioStatusTableModel.getColumnCount(); i++)
        {
            var colCurrent = tablesrb.getColumn(i);

            Logger.consoleLog(def.logInfo,"Column name " + colCurrent.title + " col " + colCurrent)
            Logger.consoleLog(def.logInfo,"Mode Name " + ioStatusTableModel.getColumnName(i))
            colCurrent.title = ioStatusTableModel.getColumnName(i)
            colCurrent.role = "i" + i
        }

        
    }

    Keys.onPressed: {
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
    }

    onOpacityChanged:
    {
        if(opacity === 1)
        {
            busyIndicator.visible = false
            conController.connectionError.connect(onConnectionError)
            appController.appStatusChanged.connect(onAppStatusChanged)
            screen_authentication.authentication.startRefreshing.connect(onAuthenticationSucceded)
            if(screen_trendView.dataController != 0)
            {
                ioStatusController.setIOMode(screen_iostate.mode)
                ioStatusController.setIOStatusModel(ioStatusTableModel)
            }
        }
        else
        {
            conController.connectionError.disconnect(onConnectionError)
            appController.appStatusChanged.disconnect(onAppStatusChanged)
            screen_authentication.authentication.startRefreshing.disconnect(onAuthenticationSucceded)
        }
    }
    function onAppStatusChanged()
    {
        if(appController.appStaus !== def.appActive)
        ioStatusController.stopRefreshing()
    }

    function onAuthenticationSucceded()
    {

        ioStatusController.startRefreshing(screen_iostate.mode)
    }
}


