import QtQuick 2.0
import DataStoreComponents 1.0
import LoggerComponents 1.0

import "." as S7App_Components
Item {

    property int deviceState : -1
    Headline {
        id: headline
        width: parent.width
        z:50

        headline_text: stringStore.io_selTitle
        backbutton_visible: true
        ipvisible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Headline IO Status Selection clicked!")
            backButtonOperation()
        }
    }

    Flickable{
        id:flickCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height

        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column{
            id: col
            width: parent.width

            CaptionListEntry{
                caption: stringStore.io_Inputs
                onFieldClicked: {
                    busyIndicator.visible = true
                    screen_iostate.mode = 0; //IO_STATUS_INPUTS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}

            CaptionListEntry{
                caption: stringStore.io_Ouputs
                onFieldClicked: {
                    busyIndicator.visible = true
                    screen_iostate.mode = 1; //IO_STATUS_OUTPUTS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}

            CaptionListEntry{
                caption: stringStore.io_Flags
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 2; //IO_STATUS_FLAGS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}

            CaptionListEntry{
                caption: stringStore.io_ShiftRegBits
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 3; //IO_STATUS_SHIFTBITS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}

            CaptionListEntry{
                caption: stringStore.io_CursorKeys // "Cursor Keys"
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 4; //IO_STATUS_CURSORKEYS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}
            CaptionListEntry{
                caption: stringStore.io_AnalogInputs
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 5; // IO_STATUS_ANALOGINPUTS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}
            CaptionListEntry{
                caption: stringStore.io_AnalogOutput
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 6; //IO_STATUS_ANALOGOUTPUTS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}

            CaptionListEntry{
                caption: stringStore.io_AnalogFlags
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 7; //IO_STATUS_ANALOGOUTPUTS
                    conController.getDeviceState()
                }
            }

            ListSeperator{}

            CaptionListEntry{
                caption: stringStore.io_FuntionKeys
                onFieldClicked:{
                    busyIndicator.visible = true
                    screen_iostate.mode = 8; //IO_STATUS_ANALOGOUTPUTS
                    conController.getDeviceState()
                }
            }

              ListSeperator{}

        }
    }

    function backButtonOperation()
    {
        ioStatusController.deInit()
        stackview.popPage();
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in settings" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
    }

    function onGotDeviceState(nDeviceState)
    {
        deviceState = nDeviceState;

         Logger.consoleLog(def.logInfo,"DEVICE STATE_INVALID")

        if(deviceState >= 2)
        {
            Logger.consoleLog(def.logInfo,"DEVICE STATE_INVALID")
            conController.disConnectFromHost()
            stackview.replace(screen_overview)
            screen_overview.conFailedMsg.openWithErrorMessage("")
            return;
        }

        if(deviceState == 1 ) //ConnectionController.enStopState
        {
            changemodeMsg.text =  stringStore.stopToRunMsg //"Device is in STOP mode. Do you want to change it to RUN mode?"
            changemodeMsg.open()
        }
        else if(deviceState == 0) //ConnectionController.enRunState
        {
            Logger.consoleLog(def.logInfo,"IO STATUS LIST - DEVICE RUN MODE")

            if(def.isLogoApp)
            {  
                Logger.consoleLog(def.logInfo,"mode for IO = " + screen_iostate.mode)
                ioStatusController.setIOMode(screen_iostate.mode)
                ioStatusController.setIOStatusModel(ioStatusTableModel)
                screen_iostate.updateTable()
                ioStatusController.startIOStatusProcessing(screen_iostate.mode)
                stackview.pushPage(screen_iostate)
            }

        }
    }

    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible = false
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        Logger.consoleLog(def.logInfo,"ON CONNECTION ERROR")
        stackview.replace(screen_overview)
    }
    
    ModalDialog {
        id: changemodeMsg
        title: stringStore.logoChangeModeMsgTitle
        button_Yes: true
        button_No: true
        onYesButtonClicked: {
            if(deviceState === 1)
            {
                conController.setPLCModeCompleted.connect(onSetPLCModeCompleted)
                conController.setPLCMode(0) //change to run mode
            }
        }
    }

    function onSetPLCModeCompleted()
    {
        conController.setPLCModeCompleted.disconnect(onSetPLCModeCompleted)
        conController.getDeviceState()
    }

    onOpacityChanged: {
        if (opacity == 1)
        {
            busyIndicator.visible = false
            conController.gotDeviceState.connect(onGotDeviceState)
            conController.connectionError.connect(onConnectionError)

        }
        else
        {
            flickCtl.contentY = 0
            conController.gotDeviceState.disconnect(onGotDeviceState)
            conController.connectionError.disconnect(onConnectionError)
        }
    }
}




