import QtQuick 2.0
import LoggerComponents 1.0

import "." as S7App_Components

Item {

  //  property alias systemInfo:stationState.text;
    property alias plcInfo : plcInfoHeader.text    
    property alias diag_visible : diagnostics.visible
    property alias runButtonInSysInfo: runButton
    property alias runStateButtonInSysInfo: runStateButton
    property alias timerInSysInfo: timer
    property alias plcInfoHeaderHeight : plcInfoHeader.height
    property alias rectInSystemInfoHeight : rectInSystemInfo.height
    

    property variant setClockComp : null
    property variant setClockObj : null
    property int deviceState : -1
    property bool variablesOrYesClicked : false

    Headline {
        id: headline
        width: parent.width
        z: 50

        headline_text: stringStore.stationInfo
        backbutton_visible: true
        ipvisible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Button Back from Headline SysInfo clicked!")
            backButtonOperation()
        }
    }

    Flickable{
    	id:flickCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height

        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column{
            id: col

            anchors.top: parent.top
            anchors.left: parent.left

            width: parent.width
          //  height: def.standardFieldHeight*5

            Rectangle{      // first field with PLC name and state
                id:rectInSystemInfo
                width: parent.width
                height: def.standardFieldHeight
                color: "transparent"

                DefaultText {
                    id: stationText

                    color: def.colFontGeneral

                    text: mainObject.currentConnection
                    font.pixelSize: def.fontSizeStationName
                    font.bold: true
                    elide:Text.ElideRight
                    width:parent.width/2
                    maximumLineCount: 20

                    anchors.left: parent.left
                    anchors.leftMargin: def.borderDistanceLeft
                    anchors.top: parent.top
                    anchors.topMargin: def.borderDistanceTop

                }

                DefaultText {
                    id: plcInfoHeader

                    color: def.colFontGeneral

                    text: "" //+ mainObject.currentConnection
                    font.pixelSize: def.fontSizeNormalText
                    font.bold: false

                    anchors.left: parent.left
                    anchors.leftMargin: def.borderDistanceLeft
                    anchors.top: stationText.bottom
                    anchors.topMargin: def.borderDistanceTop

                }
                StateIndicator{
                    id: runButton
                    visible: def.isLogoApp
                    anchors.left: parent.left
                    anchors.leftMargin: def.borderDistanceLeft
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: def.borderDistanceTop
                }

                StateButton{
                    id: runStateButton
                    visible:def.isS7App
                    anchors.left: parent.left
                    anchors.leftMargin: def.borderDistanceLeft
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: def.borderDistanceBottom
                    onButtonClicked: {

                        //timer.running = false;

                        if(runStateButton.state == true) // PLC is in RUN mode hence set to STOP mode
                            conController.setPLCMode(1)
                        else
                            conController.setPLCMode(0) // PLC is in STOP mode hence set to RUN mode

                    }
                }

   /*             Button{
                    id: flashButton
                    width: def.standardButtonWidthExtended

                    anchors.right:  parent.right
                    anchors.rightMargin: def.borderDistanceRight
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: def.borderDistanceTop

                    button_Text: qsTr("Webserver") + langSelector.emptyString
                    isTextbutton: true

                    onButtonClicked: { Logger.consoleLog(def.logInfo,"Webserver button clicked!") }

                }*/


            }

            ListSeperator{
               // anchors.horizontalCenter: parent.horizontalCenter
            }


            CaptionListEntry{
                id:ioCaptionEntry
                caption: stringStore.iostatus
                visible: def.isLogoApp
                property bool isIOStatus:false
                onFieldClicked: {
                    Logger.consoleLog(def.logInfo,"IO Status  clicked");
                    busyIndicator.visible = false
                    isIOStatus = true
                    conController.getDeviceState()
                }
            }

            ListSeperator{visible: def.isLogoApp}

            CaptionListEntry{
                caption: stringStore.tags
                visible: def.isLogoApp
                onFieldClicked:{
                    Logger.consoleLog(def.logInfo,"Variables clicked");

                    variablesOrYesClicked = true

                    //check the program mode of the PLC
                    conController.getDeviceState()
                   
                }
            }

            ListSeperator{visible: def.isLogoApp}

            CaptionListEntry{
                id: diagnostics
                visible:  false
                caption: stringStore.diagnostics
                onFieldClicked:{
                    timer.running = false
                    diagnosticbuffController.init()
                    diagnosticbuffController.getDiagnosticsInfo()
                    Logger.consoleLog(def.logInfo,"Diagnostic Monitor clicked");
                }
            }

            ListSeperator{visible: diagnostics.visible}

            CaptionListEntry{
                caption: stringStore.datetimeMsgTitle
                visible: def.isLogoApp
                onFieldClicked:{
                    Logger.consoleLog(def.logInfo,"SET Clock clicked");
		    
                    if(setClockComp != null)
                    {
                    	if(setClockObj != null)
                        	setClockObj.destroy()
                        setClockComp.destroy()
                        setClockComp = null
                    }
			    
                    if(setClockComp == null)
                    {
                        setClockComp = Qt.createComponent("DateTimeSettings.qml")
                        if(setClockComp.status == Component.Ready)
                        {
                            setClockObj = setClockComp.createObject(mainObject, {"width":mainObject.width,
                                                                    "height":mainObject.height,
                                                                    "opacity":0})
                        }
                        else if(setClockComp.status == Component.Error)
                        {
                            Logger.consoleLog(def.logInfo,"DateTime Settings Component status is Error!")
                        }
                    }
                    if(setClockObj != null)
                        stackview.pushPage(setClockObj)
                }
            }

            CaptionListEntry{
                caption: stringStore.functions
                visible: def.isS7App
                withArrows: true
                onFieldClicked: functionlist.visible = !functionlist.visible

            }

            Rectangle{
                id: functionlist
                width: parent.width
                height: def.standardFieldHeight/2 * 2 + 2*def.seperatorLine + 2*def.headlineShadowHeight
                color: def.colDetailsBackground
                visible: false

                Column{
                    id: functionlistColumn
                    width: parent.width
                    height: parent.height
                    Seperator{}
                    Shadow{}

                    FunctionListEntry{
                        caption: stringStore.tags
                        onFctClicked:{
                            Logger.consoleLog(def.logInfo,"Tag List clicked");

                            if(def.isLogoApp)
			    {
			    variablesOrYesClicked = true
                            //check the program mode of the PLC
                            conController.getDeviceState()
			    }
                            else
                            {
                                loadTagListScreen()
                            }
                        }

                    }
                    ListSeperator{}

                    
                    FunctionListEntry{
                        caption: stringStore.diagnosticsS7Headline
                        onFctClicked:{
                            Logger.consoleLog(def.logInfo,"Diagnostic Buffer clicked");
                            busyIndicator.visible = true
                            diagnosticbuffController.init()
                            diagnosticbuffController.getDiagnosticsInfo()
                        }

                    }
//                    ListSeperator{}

//                    FunctionListEntry{
//                        caption: stringStore.dump
//                        onFctClicked:{
//                            Logger.consoleLog(def.logInfo,"Dump clicked");
//                        }
//                    }

//                    ListSeperator{}
//                    FunctionListEntry{
//                        caption: stringStore.datalog
//                        onFctClicked:{
//                            Logger.consoleLog(def.logInfo,"Data Log clicked");
//                            stackview.pushPage(screen_dataLog)
//                        }
//                    }

                    Shadow{
                        rotation: 180
                    }

                    Seperator{
                        withShadow: true
                        rotation: 180
                    }

                }

            }



            ListSeperator{
                anchors.horizontalCenter: parent.horizontalCenter
                visible: !functionlist.visible
            }

            CaptionListEntry{
                caption: stringStore.details
                visible: def.isS7App
                withArrows: true
                onFieldClicked: {
                    detailedInformation.visible = !detailedInformation.visible
                }
            }


            Seperator{
                visible: detailedInformation.visible
                withShadow: false
            }
            Shadow{
                visible: detailedInformation.visible
            }
            // todo: Detailed infos must be filled dynamically.
            // To test the design, I made 2 fix entries. Feel free to change!

           DetailedInfoList{
                id: detailedInformation
                visible: false               

            }

            ListSeperator{
                anchors.horizontalCenter: parent.horizontalCenter
                visible: detailedInformation.visible
            }

            Seperator{
                visible: detailedInformation.visible
                withShadow: true
                rotation: 180
            }

        }
    }

    function backButtonOperation()
    {
        conController.disconnectedFromhost.connect(onDisconnectedFromhost)
        conController.disConnectFromHost()
    }

    function onDisconnectedFromhost()
    {
        conController.disconnectedFromhost.disconnect(onDisconnectedFromhost)
        stackview.popCurrentPage(screen_systemInfo)
        if(stackview.busy)
        {
            stackview.completeTransition()
        }
    }
    function onGotDeviceState(nDeviceState)
    {
        deviceState = nDeviceState;

        if(deviceState >= 2)
        {
            Logger.consoleLog(def.logInfo,"DEVICE STATE_INVALID")
            tagModel.removeAll()
            conController.disConnectFromHost()
            stackview.replace(screen_overview)
            screen_overview.conFailedMsg.openWithErrorMessage("")
            return;
        }

        if(def.isLogoApp)
            runButton.state = deviceState == 0 ? true : false
        else
            runStateButton.state = deviceState == 0 ? true : false

        if(ioCaptionEntry.isIOStatus)
        {
            ioCaptionEntry.isIOStatus = false;
            if(deviceState == 0 ) //ConnectionController.enRunState
            {
                loadIOScreen()
            }
            else if(deviceState == 1) //ConnectionController.enStopState
            {
                if(def.isLogoApp)
                {
                    //"Device is Stop mode. Do you want to change it to RUN mode?"
                    changemodeMsg.text =  stringStore.stopToRunMsg
                    changemodeMsg.open()
                }
                else
                {
                    loadIOScreen()
                }
            }

        }

        if(variablesOrYesClicked == true)
        {
            variablesOrYesClicked = false;
            if(deviceState == 0 ) //ConnectionController.enRunState
            {                
                loadTagListScreen()
            }
            else if(deviceState == 1) //ConnectionController.enStopState
            {
                if(def.isLogoApp)
                {
                    //"Device is Stop mode. Do you want to change it to RUN mode?"
                    changemodeMsg.text =  stringStore.stopToRunMsg
                    changemodeMsg.open()
                }
                else
                {
                    loadTagListScreen()
                }
            }

        }


    }

    function loadTagListScreen()
    {
        tagController.init(mainObject.currentIndex,connectionModel.getTagIdentifier(mainObject.currentIndex))
        screen_tagList.removeTagElementsOnConnectionError = false
        screen_tagList.screenMode = 0
        stackview.pushPage(screen_tagList)
    }

    function loadIOScreen()
    {
        ioStatusController.init()
        stackview.push(screen_iostateoverview)
    }

    function onGotBasicInfo()
    {
        conController.getDeviceState()
        timer.running = true;
    }

    function onSetPLCModeCompleted()
    {
        conController.setPLCModeCompleted.disconnect(onSetPLCModeCompleted)
        variablesOrYesClicked = true
        conController.getDeviceState()
        timer.running = true;
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in System info" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            busyIndicator.visible = false
            backButtonOperation()
        }
    }

    Component.onCompleted:
    {
        conController.gotBasicInfo.connect(onGotBasicInfo)
    }

    Component.onDestruction: {
        conController.gotBasicInfo.disconnect(onGotBasicInfo)
        if(setClockObj != null)
        {
            Logger.consoleLog(def.logInfo,"Deleting SetClock Object")
            setClockObj.destroy()
            setClockComp = null
        }
    }

    function onReadDiagSuccessfull()
    {
        stackview.push(screen_diag)
        busyIndicator.visible = false
        Logger.consoleLog(def.logInfo,"Diagnostic Monitor clicked");
    }

    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible = false
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        timer.running = false;
        Logger.consoleLog(def.logInfo,"ON CONNECTION ERROR")
        stackview.popCurrentPage(screen_systemInfo)
    }


    function onAuthenticationError()
    {
        Logger.consoleLog(def.logInfo, "ON AUTHENTICATION FAILED")
        busyIndicator.visible = false
        //timer.running = false
        authenticationErrorMsg.open()
    }

    function onPLCModeChange(bSuccess)
    {
        Logger.consoleLog(def.logInfo,"PLC MODE CHANGED")
        timer.running = true

        if(bSuccess === true)  // successfully changed the mode
            toastMsg.toast.show(stringStore.successFullyChangedPLCMode)
        else
            toastMsg.toast.show(stringStore.failedToChangePLCMode)
    }

    onOpacityChanged: {
        if (opacity == 1)
        {
            conController.gotDeviceState.connect(onGotDeviceState)
            diagnosticbuffController.readDiagSuccessfull.connect(onReadDiagSuccessfull)
            diagnosticbuffController.connectionError.connect(onConnectionError)
            conController.changedPLCMode.connect(onPLCModeChange)
            conController.connectionError.connect(onConnectionError)

            //Signals required only for S7 Connection
            if(def.isS7App)
            {
                conController.authenticationError.connect(onAuthenticationError)
            }

            if(def.isOBA7())
            {
                conController.getDeviceState()
            }

            if(runButton.visible == true || runStateButton.visible == true)
            timer.running = true;
        }
        else
        {
            timer.running = false;
            variablesOrYesClicked = false
            conController.gotDeviceState.disconnect(onGotDeviceState)
            diagnosticbuffController.readDiagSuccessfull.disconnect(onReadDiagSuccessfull)
            diagnosticbuffController.connectionError.disconnect(onConnectionError)
            conController.changedPLCMode.disconnect(onPLCModeChange)
            conController.connectionError.disconnect(onConnectionError)
	    flickCtl.contentY = 0
            if(def.isS7App)
                conController.authenticationError.disconnect(onAuthenticationError)
        }
    }

    Timer{
        id: timer
        interval: def.checkStateTime; running: false; repeat:true
        onTriggered: {
            conController.getDeviceState()
           
        }
    }


    ModalDialog {
        id: changemodeMsg
        title: stringStore.logoChangeModeMsgTitle
        button_Yes: true
        button_No: true
        onYesButtonClicked: {
            if(deviceState === 1)
            {
                timer.running = false;
                conController.setPLCModeCompleted.connect(onSetPLCModeCompleted)
                conController.setPLCMode(0) //change to run mode
            }
        }
    }

}



