import QtQuick.Window 2.1
import QtQuick 2.0
import LoggerComponents 1.0

import "." as S7App_Components

Rectangle {
    id:screenTagEdit1
    width: parent.width
    height: parent.height
    color: "transparent"
    property alias addressLabel:addressName.input
    property alias formatLabel:formatName.input
    property alias idLabel:idName.input
    property int screenMode:screen_tagList.screenMode
    property bool flicked:false
    property int formatTypeHeight:headline.height * 3.17
    property alias spinnerFormatType: spinnerFormat
    property bool formatTypeVisible : false
    property bool potraitMode:def.isProtraitMode
    property string formatvalue:""

    ListModel{
        id:s71500_Model
        ListElement{val:"BIN"}                      //00
        ListElement{val:"HEX"}                      //01
        ListElement{val:"DEC+/-"}                   //02
        ListElement{val:"FLOATING_POINT"}           //03
        ListElement{val:"CHARACTER"}                //04
        ListElement{val:"BOOL"}                     //05
        ListElement{val:"TIME"}                     //06
        ListElement{val:"DATE"}                     //07
        ListElement{val:"SIMATIC_TIME"}             //08
        ListElement{val:"TIME_OF_DAY"}              //09
        ListElement{val:"COUNTER"}                  //10
        ListElement{val:"POINTER"}                  //11
        ListElement{val:"BCD"}                      //12
        ListElement{val:"OCTAL"}                    //13
        ListElement{val:"UNICODE CHARACTER"}        //14
        ListElement{val:"DEC_UNSIGNED"}             //15
        ListElement{val:"DEC SEQUENCE"}             //16
        ListElement{val:"DATE_AND_TIME"}            //17
        ListElement{val:"STRING"}                    //18
        ListElement{val:"UNICODE STRING"}           //19
    }

    ListModel{
        id:logoModel
        ListElement{val:"BIT"}
        ListElement{val:"BINARY"}
        ListElement{val:"SIGNED"}
        ListElement{val:"UNSIGNED"}
        ListElement{val:"HEX"}
        ListElement{val:"ASCII"}
    }

    ModalDialog {
        id: tagEditMsg
        title: headline.headline_text
        button_OK: true
        onYesButtonClicked: {
            stackview.popPage()
            hideRoller()
        }
        onOkButtonClicked: {
            headline.savebutton_visible = true
        }
    }

    Headline{
        id: headline
        headline_text: (screen_tagList.screenMode === 0) ? stringStore.addTag : stringStore.editTag
        backbutton_visible: true
        savebutton_visible: true

        onButtonBackClicked: {
            backButtonOperation()

        }

        onButtonSaveClicked: {
            Logger.consoleLog(def.logInfo,"Button Add from Headline clicked!")
            Logger.consoleLog(def.logInfo,"Button Infos for " + addressName.input + " clicked!")

            Logger.consoleLog(def.logInfo,"Address = " + addressName.input + " Value =  " + idName.input)

            tagEditMsg.button_OK = true
            tagEditMsg.button_Cancel = false
            tagEditMsg.button_Yes = false
            tagEditMsg.button_No = false
            hideRoller()

            //Validate empty input fields
            if(addressName.input === "" || idName.input === "" || formatName.input === "")
            {
                tagEditMsg.text = stringStore.tagFieldEmpty
                tagEditMsg.open()
                return ;
            }

            //Validate memory address
            if(!tagController.isValidMemoryAddress(addressName.input))
            {
                tagEditMsg.text = stringStore.invalidMemoryAddress
                tagEditMsg.open()
                return ;
            }

            //Validate whether invalid data format has been selected
            if(def.isLogoApp)
            {
                var addressIsOfTypeV = /([Vv][0-9]+.[0-7])/

                var bError = false
                var selectedIndex = spinnerFormat.currentIndex
                if(!addressIsOfTypeV.test(addressName.input) && selectedIndex === 0)
                {
                    tagEditMsg.text = stringStore.invalidDatFormatForVDAddress
                    bError = true
                }
                else if(addressIsOfTypeV.test(addressName.input) && selectedIndex !== 0)
                {
                    tagEditMsg.text = stringStore.invalidDatFormatForVAddress
                    bError = true
                }

                if(bError)
                {
                    tagEditMsg.open()
                    formatName.input = ""
                    return
                }
                Logger.consoleLog(def.logCritical,"Calling TagUpdate 1")
                saveTag()
            }
            else if(def.isS7App === true)
            {
                if(connectionModel.getPlcType(mainObject.currentIndex) === 0) // S71200
                    tagController.doTagValidation(addressName.input,formatvalue,idName.input)
                else
                    tagController.doTagValidation(addressName.input,spinnerFormat.currentIndex,idName.input)

            }
        }
    }

    Flickable{
        id:flickCtl
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height

        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column{
            id: col
            height: screenTagEdit1.height
            InputField{
                id: addressName
                label: stringStore.tagAddress
                input: ""
                onInputFieldEnabled: {
                    headline.savebutton_visible = true
                }
                MouseArea {
                    anchors.fill: parent
                    onPressed: {
                        hideRoller()
                        mouse.accepted = false
                    }
                }
            }

            InputField{
                id: idName
                label: stringStore.tagName
                input: ""
                onInputFieldEnabled: {
                    headline.savebutton_visible = true
                }
                MouseArea {
                    anchors.fill: parent
                    onPressed: {
                        hideRoller()
                        mouse.accepted = false
                    }
                }
            }

            InputField{
                id: formatName
                label: stringStore.tagFormat
                input: ""
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        showRoller()
                    }
                }
                onInputFieldEnabled: {

                }
            }
        }
    }

    Rectangle{
        id:formatType
        width: flickCtl.width
        height: formatTypeHeight
        y : parent.height
        opacity: 0
        color: "#D0DADF"
        RollerControl {
            id: spinnerFormat
            width: parent.width -2*def.borderDistanceLeft; height: parent.height/1.3
            anchors{ top: shadow.bottom; horizontalCenter: parent.horizontalCenter }
            model:def.isS7App ? s71500_Model : logoModel

            delegate: Component{
                DefaultText {
                    id:dlgText
                    font.pixelSize: (index == spinnerFormat.currentIndex) ? def.headlineHeight*0.33 : def.headlineFontSize;
                    font.bold: (index == spinnerFormat.currentIndex) ? true:false
                    color: (index == spinnerFormat.currentIndex) ? def.colSelectedOn : "black"
                    text: val;
                    scale: 0.8
                }
            }
        }
        Button{
            isTextbutton: true
    	    width: def.standardButtonWidth
            height: def.standardButtonHeight
            button_Text: stringStore.ok
            anchors{ right: spinnerFormat.right; top: spinnerFormat.bottom; topMargin: def.borderDistanceTop}
            onButtonClicked: {
                formatName.input = formatvalue = spinnerFormat.model.get(spinnerFormat.currentIndex).val               
                if(connectionModel.getPlcType(mainObject.currentIndex) === 0 && formatvalue === "DEC+/-")
                    formatvalue = "DEC"
                hideRoller()
                headline.savebutton_visible = true
            }
        }
        Shadow{id:shadow; rotation: 180 }
    }

    ParallelAnimation
    {
        id: animateFormatTypeShow

        PropertyAnimation {
            id:animateFormatType1
            target: formatType
            properties: "y"
            to: parent.height - formatTypeHeight
            duration: 500
        }

        PropertyAnimation {
            id:animateFormatType3
            target: formatType
            properties: "opacity"
            to: 1.0
            duration: 500
        }
    }

    ParallelAnimation
    {
        id:animateFormatTypeHide

        PropertyAnimation {
            target: formatType
            properties: "y"
            to: parent.height
            duration: 500
        }

        PropertyAnimation {
            target: formatType
            properties: "opacity"
            to: 0
            duration: 500
        }
    }

    function showRoller()
    {
        formatName.focus = true
        Qt.inputMethod.hide()
        if(!formatTypeVisible)
        {
            formatType.width = parent.width
            formatTypeVisible = true
            animateFormatTypeShow.start()
        }
    }

    function hideRoller()
    {
        if(formatTypeVisible)
        {
            formatTypeVisible = false
            animateFormatTypeHide.start()
            formatType.width = 0
        }
    }

    onPotraitModeChanged:
    {
        if(formatTypeVisible)
            hideRoller()
    }

    onScreenModeChanged:
    {
        if(screenMode === 0)
        {
            addressName.input = ""
            idName.input = ""
        }
    }

    /*
      Convert from format string to index to set to spinner control
    */
    function formatIndex(formatString)
    {
        if(def.isS7App)
        {
            switch(formatString)
            {
            case "BIN":               return 0;
            case "HEX":               return 1;
            case "DEC":               return 2;
            case "FLOATING_POINT":    return 3;
            case "CHARACTER":         return 4;
            case "BOOL":              return 5;
            case "TIME":              return 6;
            case "DATE":              return 7;
            case "SIMATIC_TIME":      return 8;
            case "TIME_OF_DAY":       return 9;
            case "COUNTER":           return 10;
            case "POINTER":           return 11;
            case "BCD":               return 12;
            case "OCTAL":             return 13;
            case "UNICODE CHARACTER": return 14;
            case "DEC_UNSIGNED":      return 15;
            case "DEC SEQUENCE":      return 16;
            case "DATE_AND_TIME":     return 17;
            case "STRING":            return 18;
            case "UNICODE STRING":    return 19;
            default: return 2;
            }
        }
        else
        {
            switch(formatString)
            {
            case "BIT":         return  0;
            case "BINARY":      return  1;
            case "SIGNED":      return  2;
            case "UNSIGNED":    return  3;
            case "HEX":         return  4;
            case "ASCII":       return  5;
            default: return 2;
            }
        }
    }

    /*
      Convert from index  to format string to set format input
    */

    function getFormat(nIndex)
    {
        Logger.consoleLog(def.logCritical, "---------GET FORMAT--------")
        switch(nIndex)
        {
        case "0":  return  "BIN";
        case "1":  return  "HEX";
        case "2":  return  "DEC+/-";
        case "3":  return  "FLOATING_POINT";
        case "4":  return  "CHARACTER";
        case "5":  return  "BOOL";
        case "6":  return  "TIME";
        case "7":  return  "DATE";
        case "8":  return  "SIMATIC_TIME";
        case "9":  return  "TIME_OF_DAY";
        case "10": return  "COUNTER";
        case "11": return  "POINTER";
        case "12": return  "BCD";
        case "13": return  "OCTAL";
        case "14": return  "UNICODE CHARACTER";
        case "15": return  "DEC_UNSIGNED";
        case "16": return  "DEC SEQUENCE";
        case "17": return  "DATE_AND_TIME";
        case "18": return  "STRING";
        case "19": return  "UNICODE STRING";
        default:   return  "HEX";
        }
    }

    function getSupportedFormatString(validFormats)
    {
        var validFormatList = validFormats.split(",")
        var validFormatString = new Array(validFormatList.length)
         for(var nIndex=0; nIndex < validFormatList.length; nIndex++)
         {
             validFormatString[nIndex] = getFormat(validFormatList[nIndex])
             if(validFormatString[nIndex] === "DEC+/-")
             {
                 var temp = validFormatString[nIndex]
                 validFormatString[nIndex] = validFormatString[0]
                 validFormatString[0] = temp
             }
             Logger.consoleLog(def.logCritical, "---**CONVERTED TO STRING **--- \n " + " VALUEs " + validFormatList +" Strings " + validFormatString)
         }
         return validFormatString.toString()
    }

    /*
      Check whether the format selected is supported by the datatype
    */
    function checkValidFormat(currentIndex, validFormats)
    {
        var validFormatList = validFormats.split(",")

         Logger.consoleLog(def.logCritical, "---**Current INdex**--- = " + currentIndex)
        for(var nIndex=0; nIndex < validFormatList.length; nIndex++)
        {
            Logger.consoleLog(def.logCritical, "---------Check Valid formats-------- :: INDEX = " +nIndex + " VAlue " + validFormatList[nIndex] )
            if(currentIndex == validFormatList[nIndex])
            {
                Logger.consoleLog(def.logCritical, "---**VALID FORMAT**--- \n" + validFormatList)
                return true
            }
        }
        Logger.consoleLog(def.logCritical, "---**IN-VALID FORMAT**--- \n" + validFormatList)

        return false
    }

    function updateTag()
    {
        tagModel.setValue(screen_tagList.currentIndex,screen_writeToPLC.updatedVal)
    }

    function backButtonOperation()
    {
        tagEditMsg.text = stringStore.goBackAlertMessage
        tagEditMsg.button_OK = false
        tagEditMsg.button_Cancel = false
        tagEditMsg.button_Yes = true
        tagEditMsg.button_No = true

        tagEditMsg.open()

    }

    function onTagValidationFinished(validformats,isValidTag)
    {
        Logger.consoleLog(def.logCritical,"onValidationDone : " + isValidTag+" "+validformats)

        var bFormatType = "";
        var _validformats = validformats
        tagEditMsg.button_OK = true
        tagEditMsg.button_Cancel = false
        tagEditMsg.button_Yes = false
        tagEditMsg.button_No = false
        if(String(validformats) === "---" || String(validformats) === "" )
        {
            tagEditMsg.text = stringStore.invalidTagAddressS7
        }
        else
        {
            /**To get validformats string according to PLC type**/
            if(connectionModel.getPlcType(mainObject.currentIndex) === 1) // S7-1500 PLC
            {
                bFormatType = spinnerFormat.currentIndex;
                validformats = getSupportedFormatString(_validformats)
            }
            else
            {
                bFormatType = formatvalue
                validformats  = modifyvalidformats(_validformats)
            }

            /**Format validation**/
            if( !checkValidFormat(bFormatType, _validformats))
                isValidTag = false
            validformats = validformats.replace(/,/g,", ")
            tagEditMsg.text = stringStore.invalidTagFormatsS7 + validformats
        }

        if(isValidTag)
        {
            saveTag() //save tag
        }
        else
        {
            tagEditMsg.open()
        }
    }
function modifyvalidformats(validFormats)
{
    var validFormatList = validFormats.split(",")

    for(var nIndex=0; nIndex < validFormatList.length; nIndex++)
    {
        if(validFormatList[nIndex] === "DEC")
        {
            validFormatList[nIndex] = "DEC+/-"
            var temp = validFormatList[nIndex]
            validFormatList[nIndex] = validFormatList[0]
            validFormatList[0] = temp
            break;
        }
    }
   return validFormatList.toString()
}
    function saveTag()
    {
        Logger.consoleLog(def.logCritical,"Screen mode is :" + screenMode )
        if(screenMode === 0)
        {
            if(def.isS7App && connectionModel.getPlcType(mainObject.currentIndex) === 1) // S7- 1500 PLC
            {
               tagModel.appendElement(addressName.input,idName.input,spinnerFormat.currentIndex,10,tagModel.getSize())
            }
            else
              tagModel.appendElement(addressName.input,idName.input,formatvalue,10,tagModel.getSize())
        }
        else if(screenMode === 1) //Edit screen
        {
            if(def.isS7App && connectionModel.getPlcType(mainObject.currentIndex) === 1) // S7- 1500 PLC
            {
                tagModel.updateElement(addressName.input,idName.input,spinnerFormat.currentIndex,10,screen_tagList.currentIndex)
            }
            else
                tagModel.updateElement(addressName.input,idName.input,formatvalue,10,screen_tagList.currentIndex)
        }
        Logger.consoleLog(def.logCritical,"Poping page")
        screen_tagList.screenMode = 0
        stackview.popPage()
    }

    function onConnectionError(strErrorMsg)
    {
        tagController.stopRefreshTags()
        conController.disConnectFromHost()
        tagModel.removeAllDataElement();
        tagModel.removeAll();
        screen_overview.conFailedMsg.openWithErrorMessage("")
        stackview.replace(screen_overview)
        removeTagElementsOnConnectionError = true;
    }

    function onAuthenticationError()
    {
        Logger.consoleLog(def.logInfo, "ON AUTHENTICATION FAILED")
        authenticationErrorMsg.open()
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Tag Edit" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }

    onOpacityChanged: {
        if(opacity === 1)
        {
            selectTagModel()
            Logger.consoleLog(def.logCritical,"onValidationDone : Connected")
            tagController.tagValidation.connect(onTagValidationFinished)
            conController.connectionError.connect(onConnectionError)
            //Signals required only for S7 Connection
            if(def.isS7App)
                conController.authenticationError.connect(onAuthenticationError)

            headline.savebutton_visible = true
            if(screenMode === 0)
            {
                addressName.input = ""
                idName.input = ""
                formatName.input = ""
                spinnerFormat.currentIndex = 0
            }
            formatType.opacity = 0
            formatType.y = parent.height
        }
        else if(opacity === 0)
        {
           flickCtl.contentY = 0
            Logger.consoleLog(def.logCritical,"onValidationDone : Dis Connected")
            tagController.tagValidation.disconnect(onTagValidationFinished)
            conController.connectionError.disconnect(onConnectionError)
            if(def.isS7App)
                conController.authenticationError.disconnect(onAuthenticationError)
        }
    }

    function selectTagModel()
    {
        if(def.isOBA7() || def.isOBA8())
        {
            spinnerFormat.model = logoModel
        }
        else if(def.isS7App)
        {
            spinnerFormat.model = s71500_Model
        }
    }
}
