import QtQuick 2.0
import LoggerComponents 1.0

import "." as S7App_Components

Item {

    property int screenMode: 0
    property int currentIndex: 0
    property int plcMode: 1
    property alias tagListObj:tag_list
    property bool removeTagElementsOnConnectionError:false
    property bool clearSelectedItem:false
    property real scrollUpLimit:def.headlineHeight+(def.tagListEntry * 1.5)
    property real scrollDownLimit:mainObject.height - (def.tagListEntry * 1.5)

    Column {
        anchors.fill:parent

        Headline {
            id: headline
            width: parent.width

            headline_text: stringStore.taglist
            backbutton_visible: true
            addbutton_visible: true
            editbutton_visible: true
            ipvisible: true
            delete_visible: false

            onButtonBackClicked: {
                Logger.consoleLog(def.logInfo,"ScreenTagList: Button Back from Headline clicked!")
                backButtonOperation()

            }

            onButtonAddClicked: {
                Logger.consoleLog(def.logInfo,"ScreenTagList: Button Add from Headline clicked!")
                tagController.stopRefreshTags()
                stackview.pushPage(screen_tagEdit)
            }
            onButtonEditClicked: {
                tagController.stopRefreshTags()
                Logger.consoleLog(def.logInfo,"ScreenTagList: Button Edit from Headline clicked!")
                screenMode = 1
                Logger.consoleLog(def.logInfo,"connection values = " + mainObject.currentConnection + mainObject.currentIP)

            }

            onButtonDeleteClicked: {

                if(screenMode == 2)
                {
                    screenMode = 1
                }
                if(screenMode == 1)
                {
                    tagModel.removeSelectedElements()
                }
                if(tagModel.getSize() === 0)
                {
                    screenMode = 0
                }
            }
        }

        Rectangle{
            id: shadowSpace
            height: def.headlineShadowHeight
            width: parent.width
            color: "transparent"
        }

        TagList {
            id: tag_list
            width: parent.width
            height: parent.height-bottomline.height-headline.height - shadowSpace.height
        }

        Buttonline {
            id: bottomline
            width: parent.width

            buttonMonitor_visible: true

            onButtonMonitorClicked: {
                Logger.consoleLog(def.logInfo,"ScreenTagList: Button Monitor from Buttonline clicked!")
                {
                    if(tagModel.getSize() > 0)
                    {
                        screen_trendView.clearSelectedTrend = false
                        screen_trendView.readMode = 1
                        screen_trendView.initializeTrendData(tagController,tagModel,trendViewListItem)
                        screen_trendView.trendViewObj.setAxisRange(0,100)
                        screen_trendView.trendViewObj.setTickCount(5)
                        screen_trendView_FullScreen.initializeFullScreenTrendData(tagController,tagModel,dataGridItem)
                        screen_trendView.trendViewObj.initializeTrendView()
                        stackview.pushPage(screen_trendView)
                    }
                }
            }

            onVisibleChanged: {
                if(visible)
                {
                    height = def.buttonlineHeight
                }
                else
                {
                    height = 0
                }
            }
       }

    }
    onScreenModeChanged: {

        if(screenMode === 1)  // editing mode
        {
            headline.addbutton_visible = false
            headline.editbutton_visible = false
            headline.delete_visible = true
            headline.headline_text = stringStore.edittaglist
            bottomline.visible = false
	    appController.appStatusChanged.disconnect(onAppStatusChanged)
            screen_authentication.authentication.startRefreshing.disconnect(onAuthenticationSucceded)
        }
        else if(screenMode === 0) // normal view mode
        {
            headline.addbutton_visible = true
            headline.editbutton_visible = true
            headline.delete_visible = false
            tagController.startRefreshTags()
            headline.headline_text = stringStore.taglist
            bottomline.visible = true
            appController.appStatusChanged.connect(onAppStatusChanged)
            screen_authentication.authentication.startRefreshing.connect(onAuthenticationSucceded)

        }
    }

    function backButtonOperation()
    {
        if(screenMode == 0)
        {
            tagController.stopRefreshTags()
            clearTagData()
            stackview.popCurrentPage(screen_tagList)
           
        }
        else if(screenMode == 1)
            screenMode = 0
        else if(screenMode == 2)
        {
            screenMode = 0
        }

    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Tag List" )
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            busyIndicator.visible = false
            backButtonOperation()
        }
    }
    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible = false
        tagController.stopRefreshTags()
        conController.disConnectFromHost()
        tagModel.removeAllDataElement();
        tagModel.removeAll();
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        stackview.replace(screen_overview)
        removeTagElementsOnConnectionError = true;
    }

    function clearTagData()
    {
        tagModel.removeAll()
    }

    function onTagValidationFinished(isValidTag)
    {
        Logger.consoleLog(def.logCritical,"onValidationDone : " + isValidTag)
        tagController.tagValidation.disconnect(onTagValidationFinished)
        busyIndicator.visible = false
    }

    function onAuthenticationError()
    {
        Logger.consoleLog(def.logInfo, "ON AUTHENTICATION FAILED")
        tagController.stopRefreshTags()
        bottomline.enabled = false
        busyIndicator.visible = false
        tagController.tagValidation.disconnect(onTagValidationFinished)

        authenticationErrorMsg.open()

        headline.addbutton_visible = false
        headline.editbutton_visible = false
    }

    onOpacityChanged: {

        if(screenMode == 0)
        {
            if(opacity == 1)
            {
                Logger.consoleLog(def.logInfo,"TagList: Shown")
                headline.addbutton_visible = true
                headline.editbutton_visible = true
	        appController.appStatusChanged.connect(onAppStatusChanged)
                screen_authentication.authentication.startRefreshing.connect(onAuthenticationSucceded)
		 if(tagModel.getSize() > 0 && tagController.isTagDataAvailable())
                 {
                    busyIndicator.visible = true
                    tagController.startRefreshTags()
                 }
                 else
                 {
                     if(def.isS7App)
                     {
                         busyIndicator.visible = true
                         tagController.tagValidation.connect(onTagValidationFinished)
                         tagController.doTagValidation("QW0", "QW0", "DEC")
                     }
                 }

                bottomline.enabled = true

                conController.connectionError.connect(onConnectionError)

                //Signals required only for S7 Connection
                if(def.isS7App)
                    conController.authenticationError.connect(onAuthenticationError)

            }
            else if(opacity == 0)
            {
                headline.addbutton_visible = true
                headline.editbutton_visible = true
                Logger.consoleLog(def.logInfo,"Disconnect From signal")
                tagController.tagValidation.disconnect(onTagValidationFinished)
                conController.connectionError.disconnect(onConnectionError)
                appController.appStatusChanged.disconnect(onAppStatusChanged)
                screen_authentication.authentication.startRefreshing.disconnect(onAuthenticationSucceded)
                if(removeTagElementsOnConnectionError == true)
                {
                    Logger.consoleLog(def.logInfo,"REMOVE ALL TAG ELEMENTS")
                    tagModel.removeAll()
                    removeTagElementsOnConnectionError = false;
                }

                if(def.isS7App)
                    conController.authenticationError.disconnect(onAuthenticationError)
            }
        }

    }
    
    function onAppStatusChanged()
    {
        if(appController.appStaus !== def.appActive)
        tagController.stopRefreshing()
    }

    function onAuthenticationSucceded()
    {
        tagController.startRefreshTags()
    }
}
