import QtQuick 2.2
import QtQuick.Window 2.1
import LoggerComponents 1.0

import "." as S7App_Components

Item {
    id:trendParent
    property alias trendViewObj:trend_view
    property bool protraitMode:def.isProtraitMode
    property bool atStartup:false
    readonly property int statusBarHeight:100
    property bool removeTagElementsOnConnectionError:false
    property bool clearSelectedTrend:false
    property var dataController:0
    property var dataModel:0
    property var dataListDelegate:undefined
    property int readMode:0

    property alias trendMsg:trendMsg

    ModalDialog {
        id: trendMsg
        button_OK: true
    }


    Column {
        anchors.fill:trendParent

        Headline {
            id: headline
            z: 2
            ipvisible: true

            headline_text: stringStore.trendViewTitle
            backbutton_visible: true

            onButtonBackClicked: {
                Logger.consoleLog(def.logInfo,"ScreenTrendview: Button Back from Headline clicked!")
                backButtonOperation()
            }
        }
    }


        TrendView {
            id: trend_view
            x:0
            y:headline.height
            width: headline.width
            height:def.trendviewHeight
        }

        Rectangle{
            id:shadowSpace2
            x:0
            y:trend_view.height + headline.height
            width: trendParent.width
            height: def.headlineShadowHeight

            gradient: Gradient {
                GradientStop { position: 0.0; color: "transparent"}
                GradientStop { position: 1.0;  color: def.colHeadlineShadow}
            }
        }

        Rectangle{
            id:trendViewList_Container
            x:0
            y:trend_view.height + headline.height + shadowSpace2.height
            width: headline.width
            height: trendParent.height - trend_view.height - (headline.height + shadowSpace2.height)
            color:def.colHeadlineBackground

            TrendViewList {
                id: trend_list
                width:trendViewList_Container.width
                height:trendViewList_Container.height
            }
        }

    onOpacityChanged: {
        if(opacity == 1)
        {
            //This code is specific for desktop assuming that no mobile device is more than 15 inch in screen size
            if(def.screenSize < 16)
                reSizeTrendView()
            if(dataController != 0)
            {
                dataController.dataUpdated.connect(screen_trendView.trendViewObj.onDataUpdate)
                Logger.consoleLog(def.logInfo,"TrendView: Shown")
                appController.appStatusChanged.connect(screen_trendView.trendViewObj.onAppStatusChanged)
                screen_authentication.authentication.startRefreshing.connect(onAuthenticationSucceded)

                conController.connectionError.connect(onConnectionError)

            }

        }
        else if(opacity == 0)
        {
            if(dataController != 0)
            {
                dataController.dataUpdated.disconnect(screen_trendView.trendViewObj.onDataUpdate)
                appController.appStatusChanged.disconnect(screen_trendView.trendViewObj.onAppStatusChanged)
                screen_authentication.authentication.startRefreshing.disconnect(onAuthenticationSucceded)
            }
            conController.connectionError.disconnect(onConnectionError)

            if(removeTagElementsOnConnectionError == true)
            {
                Logger.consoleLog(def.logInfo,"REMOVE ALL TAG ELEMENTS")
                dataModel.removeAll()
                removeTagElementsOnConnectionError = false;
            }
        }

    }

    function backButtonOperation()
    {
        clearSelectedTrend = true
        dataModel.removeAllDataElement();
        trendViewObj.removeAllTagSeries()
        screen_trendView_FullScreen.isTrendActivated = false
        stackview.popCurrentPage(screen_trendView)
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Write to plc")
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            busyIndicator.visible = false
            backButtonOperation()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }

    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible = false
        dataController.stopRefreshing()
        conController.disConnectFromHost()
        clearSelectedTrend = true
        dataModel.removeAllDataElement();
        dataModel.removeAll();
        screen_trendView.trendViewObj.removeAllTagSeries()
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        stackview.replace(screen_overview)
        removeTagElementsOnConnectionError = true;
    }

    onProtraitModeChanged: {
        if(opacity == 1)
           reSizeTrendView()
    }

    function getStatusBarHeight()
    {
        if(Math.abs(Screen.height - mainObject.height) <= statusBarHeight)
        {
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ===================Headline height========================= for Android")
            return (Screen.height - mainObject.height);
        }
        else if(Math.abs(Screen.width - mainObject.height) <= statusBarHeight)
        {
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ===================Headline height========================= for iOS")
            return (Screen.width - mainObject.height);
        }
		 return (Screen.width - mainObject.height);
    }

    function initializeTrendData(dataController,dataModel,dataListDelegate)
    {
        screen_trendView.dataModel = dataModel
        screen_trendView.dataModel.setDataSource()
        screen_trendView.dataController = dataController

        trend_list.trendViewListModel = dataModel
        trend_list.trendViewListDelegate = dataListDelegate

        trend_view.setDataModel(screen_trendView.dataModel)
    }

    function reSizeTrendView()
    {
        if(!protraitMode)
        {
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= Landscape Mode ==============" + headline.width + " "  + def.trendViewWidth + " " + Screen.width + " " + headline.height)
            trend_view.x = 0
            trend_view.y = headline.height  + shadowSpace2.height
            trend_view.width = (Screen.width - ( def.trendViewWidth))
            trend_view.height = Screen.height - (headline.height + shadowSpace2.height +getStatusBarHeight())

            trendViewList_Container.x =   (Screen.width - ( def.trendViewWidth))
            trendViewList_Container.y = headline.height + shadowSpace2.height
            trendViewList_Container.width = ( def.trendViewWidth)
            trendViewList_Container.height = Screen.height -(headline.height +  shadowSpace2.height +getStatusBarHeight())

            shadowSpace2.y = headline.height
            shadowSpace2.x = (Screen.width - ( def.trendViewWidth))
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= Headline Width ==============" + headline.width + " "  + headline.height )
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= main Object Width ==============" + mainObject.width + " "  +  mainObject.height)
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= Screen Width ==============" + Screen.width + " "  + Screen.height)
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= TrendView Container Width ==============" + trendViewList_Container.width + " "  + trendViewList_Container.height )
        }
        else
        {
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= Portrait Mode ==============" + headline.width + " "  + def.trendViewWidth + " " + Screen.width + " " + headline.height)
            trend_view.x = 0
            trend_view.y = headline.height + shadowSpace2.height
            trend_view.width = Screen.width
            trend_view.height = def.trendviewHeight

            trendViewList_Container.x =   0
            trendViewList_Container.y =  trend_view.height + headline.height + shadowSpace2.height
            trendViewList_Container.width  = Screen.width
            trendViewList_Container.height = Screen.height - (trend_view.height + headline.height +  shadowSpace2.height + getStatusBarHeight())

            shadowSpace2.y = trend_view.height + headline.height
            shadowSpace2.x = 0
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= Headline Width ==============" + headline.width + " "  + headline.height )
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= TrendView Width ==============" + mainObject.width  + " "  +  mainObject.height )
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= Screen Width ==============" + Screen.width + " "  + Screen.height)
            Logger.consoleLog(def.logInfo,"ScreenTrendview: ============= TrendView Container Width ==============" + trendViewList_Container.width + " "  + trendViewList_Container.height )
        }
    }
    function onAuthenticationSucceded()
    {
        if(dataController != 0)
        {
            dataController.startRefreshing(readMode)
        }
    }

}

