import QtQuick 2.2
import QtQuick.Window 2.1
import LoggerComponents 1.0

import "." as S7App_Components

Item {
    id:trendParent
    property alias trendViewObj:trend_view_fullscreen
    property bool removeTagElementsOnConnectionError:false
    property bool protraitMode:def.isProtraitMode
    property bool atStartup:false
    property int trendGridCellWidth:parent.width/2
    property int trendGridCellHeight:def.dataGridHeightPortarit/4
    property int fullScreenTrendVisibility:trendParent.opacity
    property bool isTrendActivated:false
    property var dataController:0
    property var dataModel:0
    property var dataListDelegate:0


    Column {
        anchors.fill:trendParent

        Headline {
            id: headline
            z: 2
            ipvisible: false
            appNameVisible:false
            height:def.fullScreenTrendviewHeadlineHeight

            headline_text_fullScreen_trend: stringStore.trendViewTitle
            backbutton_fullScreen_trend_visible: true

            onButtonBackFullScreenTrendViewClicked: {
                busyIndicator.visible = true
                Logger.consoleLog(def.logInfo,"ScreenFullScreenTrendview: Button Back from Headline clicked!")
                backButtonOperation()
            }
        }
    }

    FullScreenTrendView {
        id: trend_view_fullscreen
        x:0
        y:headline.height
        width: headline.width
        height:(mainObject.height -(headline.height+def.dataGridHeightPortarit+shadowSpace.height))
    }

    Rectangle{
        id:shadowSpace
        x:0
        y:trend_view_fullscreen.height + headline.height
        width: trendParent.width
        height: def.headlineShadowHeight

        gradient: Gradient {
            GradientStop { position: 0.0; color: "transparent"}
            GradientStop { position: 1.0;  color: def.colHeadlineShadow}
        }
    }

    Rectangle{
        id:trendTagGrid_Container
        x:0
        y:trend_view_fullscreen.height + headline.height +shadowSpace.height
        width: headline.width
        height: def.dataGridHeightPortarit
        color:def.colHeadlineBackground

        DataGrid{
            id: trend_DataGrid
            width:trendTagGrid_Container.width
            height:trendTagGrid_Container.height
            gridCellWdith:trendGridCellWidth
            gridCellHeight: trendGridCellHeight
        }
    }


    onOpacityChanged: {
        if(opacity == 1)
        {
            //This code is specific for desktop assuming that no mobile device is more than 15 inch in screen size
            if(def.screenSize < 16)
                reSizeTrendView()
            if(dataController != 0)
            {
                dataController.dataUpdated.connect(screen_trendView_FullScreen.trendViewObj.onDataUpdate)
                appController.appStatusChanged.connect(screen_trendView_FullScreen.trendViewObj.onAppStatusChanged)
                screen_authentication.authentication.startRefreshing.connect(onAuthenticationSucceded)
            }
            dataModel.initDataSource()
            Logger.consoleLog(def.logInfo,"TrendView: Shown")
            conController.connectionError.connect(onConnectionError)
        }
        else if(opacity == 0)
        {
            if(dataController != 0)
            {
                dataController.dataUpdated.disconnect(screen_trendView_FullScreen.trendViewObj.onDataUpdate)
                appController.appStatusChanged.disconnect(screen_trendView_FullScreen.trendViewObj.onAppStatusChanged)
                screen_authentication.authentication.startRefreshing.disconnect(onAuthenticationSucceded)
            }

            conController.connectionError.disconnect(onConnectionError)

            if(removeTagElementsOnConnectionError == true)
            {
                Logger.consoleLog(def.logInfo,"REMOVE ALL TAG ELEMENTS")
                dataModel.removeAll()
                removeTagElementsOnConnectionError = false;
            }
        }

    }

    function backButtonOperation()
    {
        
        isTrendActivated = true
        screen_trendView.clearSelectedTrend = false
        screen_trendView_FullScreen.trendViewObj.clearTrendView()
        screen_trendView_FullScreen.trendViewObj.isTrendRunning = true
        screen_trendView.trendViewObj.reStartTrendView()
        dataModel.removeAllArchiveData()
        dataModel.resetDataList()
        stackview.popCurrentPage(screen_trendView_FullScreen)
    }

    onProtraitModeChanged: {
        if(opacity === 1)
            reSizeTrendView()
    }

    function getMainScreenHeight()
    {
        if(Math.abs(Screen.height - mainObject.height) <= screen_trendView.statusBarHeight)
        {
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ===================Headline height========================= for Android")
            return (Screen.height - (Screen.height - mainObject.height));
        }
        else if(Math.abs(Screen.width - mainObject.height) <= screen_trendView.statusBarHeight)
        {
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ===================Headline height========================= for iOS")
            return (Screen.height - (Screen.width - mainObject.height));
        }
	 return (Screen.height - (Screen.width - mainObject.height));
    }

    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Write to plc")
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            busyIndicator.visible = false
            backButtonOperation()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }

    function onConnectionError(strErrorMsg)
    {
        busyIndicator.visible = false
        dataController.stopRefreshing()
        conController.disConnectFromHost()
        screen_trendView.clearSelectedTrend = true
        dataModel.removeAllArchiveData()
        dataModel.removeAllDataElement();
        dataModel.removeAll()
        screen_trendView.trendViewObj.removeAllTagSeries()
        stackview.replace(screen_overview)
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
        removeTagElementsOnConnectionError = true;
    }

    function initializeFullScreenTrendData(dataController,dataModel,dataListDelegate)
    {
        screen_trendView_FullScreen.dataModel = dataModel
        screen_trendView_FullScreen.dataModel.setDataSource()
        screen_trendView_FullScreen.dataController = dataController

        trend_DataGrid.gridModel = dataModel
        trend_DataGrid.gridDelegate = dataListDelegate

        trend_view_fullscreen.setDataModel(screen_trendView_FullScreen.dataModel)
    }

    function reSizeTrendView()
    {
        if(!protraitMode)
        {
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= Landscape Mode ==============" + getMainScreenHeight())
            trend_view_fullscreen.height =  (getMainScreenHeight() - (headline.height+shadowSpace.height+def.dataGridHeightLandScape))
            trendTagGrid_Container.height = def.dataGridHeightLandScape
            trendTagGrid_Container.width = Screen.width

            trendGridCellWidth = trendTagGrid_Container.width/4
            trendGridCellHeight = def.dataGridHeightLandScape/2

            trend_view_fullscreen.y = headline.height
            shadowSpace.y = headline.height + trend_view_fullscreen.height
            trendTagGrid_Container.y = trend_view_fullscreen.height + headline.height +shadowSpace.height
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= Headline Width ==============" + headline.width + " "  + headline.height )
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= main Object Width ==============" + mainObject.width + " "  +  mainObject.height)
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= Screen Width ==============" + Screen.width + " "  + Screen.height)
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= TrendView height ==============" +  trend_view_fullscreen.height)

        }
        else
        {
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= Portrait Mode ==============" + getMainScreenHeight())
            trend_view_fullscreen.height = (getMainScreenHeight() - (headline.height+shadowSpace.height+def.dataGridHeightPortarit))
            trendTagGrid_Container.height = def.dataGridHeightPortarit
            trendTagGrid_Container.width = Screen.width

            trendGridCellWidth = trendTagGrid_Container.width/2
            trendGridCellHeight = def.dataGridHeightPortarit/4

            trend_view_fullscreen.y = headline.height
            shadowSpace.y = headline.height + trend_view_fullscreen.height
            trendTagGrid_Container.y = trend_view_fullscreen.height + headline.height +shadowSpace.height
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= Headline Width ==============" + headline.width + " "  + headline.height )
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: =============  main Object Width ==============" + mainObject.width  + " "  +  mainObject.height )
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= Screen Width ==============" + Screen.width + " "  + Screen.height)
            Logger.consoleLog(def.logInfo,"ScreenFullTrendview: ============= TrendView height ==============" + trend_view_fullscreen.height)
        }
    }
    function onAuthenticationSucceded()
    {
        if(dataController != 0)
        {
            dataController.startRefreshing(screen_trendView.readMode)
        }
    }

}
