import QtQuick 2.0
import LoggerComponents 1.0

import "." as S7App_Components

Rectangle {
    width: parent.width
    height: parent.height
    property alias curValLabel:currValName.input
    property alias newValLabel:newValName.input
    property var currentIndex:0
    property int deviceState : 2
    property bool yesClicked : false
    property bool writeToPLC: false
    property bool newValNameCursorvisible:false

    color: "transparent"

    ModalDialog {
        id: screenMsg
        button_OK: true
        onOkButtonClicked:
        {
            if(!newValNameCursorvisible)
              newValName.startKeyboardFocus()
            newValNameCursorvisible = false
        }
    }

    ModalDialog{
        id: writePLCGoBackMsg
        title: stringStore.writePLCTitle
        text: stringStore.goBackAlertMessage
        button_Yes: true
        button_No: true

        onYesButtonClicked: {
            stackview.popPage()
        }
    }

    ModalDialog {
        id: changeLOGOMode
        title: stringStore.logoChangeModeMsgTitle
        button_No: true
        button_Yes: true
        onYesButtonClicked: {
            if(deviceState === 1)
            {
                yesClicked = true
                conController.setPLCModeCompleted.connect(onSetPLCModeCompleted)
                conController.setPLCMode(0) //change to run mode
            }

        }
    }

    Headline{
        id: headline
        headline_text: stringStore.writePLCTitle
        backbutton_visible: true

        onButtonBackClicked: {
            Logger.consoleLog(def.logInfo,"Clicked back in Write tag to PLC ")
            Qt.inputMethod.commit()
            backButtonOperation()

        }
    }

    Flickable{
        width: parent.width
        height: parent.height - headline.height
        anchors.top: parent.top; anchors.topMargin: headline.height

        contentHeight: col.height
        contentWidth: parent.width
        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.DragOverBounds


        Column
        {
            id: col
            width: parent.width

            InputField{
                id: currValName
                label: stringStore.curValue
                input: ""
                readOnly: true
            }

            InputField{
                id: newValName
                label: stringStore.newValue
                input: ""
                onInputFieldEnabled: {
                    writeToPLC = false
                }
            }

            Button {
                width: def.standardButtonWidthExtendedPlus
                anchors.right: parent.right
                anchors.rightMargin: def.borderDistanceRight

                isTextbutton: true
                button_Text: stringStore.writeBtn

                onButtonClicked: {
                    if(newValName.input === "")
                    {
                        screenMsg.title = headline.headline_text
                        screenMsg.text = stringStore.enterNewValueMsg
                        screenMsg.open()
                    }
                    else
                    {
                        if(def.isLogoApp)
                        {
                            //Check the data format of the new tag Value.
                            if(!validateIntegerLogo())
                            {
                                screenMsg.title = stringStore.invalidFormatMsgTitle
                                screenMsg.text = stringStore.invalidDataFormatOfNewvalue
                                screenMsg.open()
                                newValName.input = ""
                                return
                            }

                            //check the program mode of the PLC
                            yesClicked = false

                            conController.getDeviceState()
                        }
                        else
                        {
                            if(validateDataFormatS7())
                            {
                              writeTagToPLC();
                            }else
                            {
                                screenMsg.title = stringStore.invalidFormatMsgTitle
                                screenMsg.text = stringStore.invalidDataFormatOfNewvalue
                                screenMsg.open()
                                newValName.input = ""
                            }
                        }
                    }
                }
            }
        }
    }

    function writeTagToPLC()
    {
        Logger.consoleLog(def.logInfo,"Clicked Save in Write tag to PLC")

        writeToPLC = true

        if( newValName.input != "")
        {
            var resultList;
            var isValidValue = false;

            //Check if the value is in valid range and return if out of range
            resultList = tagController.checkDataRange(newValName.input,currentIndex)

            if(def.isLogoApp)
            {
                isValidValue = resultList[1]
            }
            else
            {
                isValidValue = true
                resultList[0] = newValName.input
            }

            if(!isValidValue)
            {
                screenMsg.title = stringStore.invalidValueMsgTitle
                screenMsg.text = stringStore.outofRangeValue
                screenMsg.open()
             }
            else
            {
                tagController.setTagValue(tagModel.getAddress(currentIndex),resultList[0])
            }

        }
    }

    function validateIntegerLogo()
    {
        var objRegExp = /(^-?[0-9][0-9]*$)/
        var objRegExpBin = /(^2#[0-1]*$)/
        var objRegExpHex = /(^16#[0-9A-Fa-f]*$)/

        var result = false

        if(objRegExp.test(newValName.input) || objRegExpBin.test(newValName.input) === true
               || objRegExpHex.test(newValName.input) == true)
        {
            result = true
        }

        return result;
    }

    function validateDataFormatS7()
    {
        var regforamp = /(&)/
        if(regforamp.test(newValName.input))
            return false;
        return true;
    }

    function onTagUpdated()
    {
        newValNameCursorvisible = true
        curValLabel = newValLabel;
        screenMsg.title = stringStore.tagWriteSuccessMsgTitle
        screenMsg.text = stringStore.tagWriteSuccessMsg
        screenMsg.open()
    }

    function backButtonOperation()
    {
        if(!writeToPLC && newValName.input != "")
        {
            writePLCGoBackMsg.open()
            return;
        }

        stackview.popPage()
    }

    function onGotDeviceState(nDeviceState)
    {
        deviceState = nDeviceState;

        if(deviceState == 1 ) //ConnectionController.enStopState
        {
            if(yesClicked == false)
            {
                if(def.isLogoApp)
                {
                    changeLOGOMode.text = stringStore.stopToRunMsg //"Device is in Stop mode. Do you want to change it to RUN mode?"
                    changeLOGOMode.open()
                }
                else
                {
                    writeTagToPLC();
                }
            }
        }
        else if(deviceState == 0) //ConnectionController.enRunState
        {
            writeTagToPLC();
        }
        else if(deviceState == 2)
        {
            tagModel.removeAll()
            conController.disConnectFromHost()
            stackview.replace(screen_overview)

            screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)

        }
    }

    function onSetPLCModeCompleted()
    {
        conController.setPLCModeCompleted.disconnect(onSetPLCModeCompleted)
        conController.getDeviceState()
    }
    Keys.onPressed: {
        Logger.consoleLog(def.logInfo,"Keypressed:in Write to plc")
        event.accepted = true;
        if (event.key === Qt.Key_Back)
        {
            backButtonOperation()
        }
        else if (event.key === Qt.Key_Return)
        {
            Qt.inputMethod.hide()
        }
    }

    function onTagWriteError(strErrorMsg)
    {
        newValNameCursorvisible = false
        screenMsg.title = stringStore.tagWriteErrorTitle
        screenMsg.text=strErrorMsg
        screenMsg.open()
    }

    function onConnectionError(strErrorMsg)
    {
        tagModel.removeAllDataElement();
        tagModel.removeAll()
        conController.disConnectFromHost()
        stackview.replace(screen_overview)
        screen_overview.conFailedMsg.openWithErrorMessage(strErrorMsg)
    }

    function onAuthenticationError()
    {
        Logger.consoleLog(def.logInfo, "ON AUTHENTICATION FAILED")
        authenticationErrorMsg.open()
    }

    onOpacityChanged: {
        if(opacity == 1)
        {
            newValName.input = ""
            //Stop the cyclic read while writing tag value to plc
            tagController.stopRefreshTags();
            conController.gotDeviceState.connect(onGotDeviceState)
            tagController.tagWriteError.connect(onTagWriteError)
            conController.connectionError.connect(onConnectionError)
            tagController.tagUpdated.connect(onTagUpdated)

            //Signals required only for S7 Connection
            if(def.isS7App)
                conController.authenticationError.connect(onAuthenticationError)

        }
        else if(opacity === 0)
        {
            newValNameCursorvisible = false
            newValName.input = ""
            conController.gotDeviceState.disconnect(onGotDeviceState)
            tagController.tagWriteError.disconnect(onTagWriteError)
            conController.connectionError.disconnect(onConnectionError)
            tagController.tagUpdated.disconnect(onTagUpdated)

            if(def.isS7App)
                conController.authenticationError.disconnect(onAuthenticationError)
        }
    }  

}
